function varargout = EditData(varargin)
% EDITDATA Application M-file for EditData.fig
%    FIG = EDITDATA launch EditData GUI.
%    EDITDATA('callback_name', ...) invoke the named callback.

% Last Modified by GUIDE v2.0 05-May-2003 02:30:31

if nargin == 0  % LAUNCH GUI

	fig = openfig(mfilename,'reuse');

	% Use system color scheme for figure:
	set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));

	% Generate a structure of handles to pass to callbacks, and store it. 
	handles = guihandles(fig);
	guidata(fig, handles);

	if nargout > 0
		varargout{1} = fig;
	end

elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK

	try
		[varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
	catch
		disp(lasterr);
	end

end

% --------------------------------------------------------------------
function varargout = fileName_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.fileName.
disp('fileName Callback not implemented yet.')


% --------------------------------------------------------------------
function varargout = dataVersion_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.dataVersion.
disp('dataVersion Callback not implemented yet.')


% --------------------------------------------------------------------
function varargout = load_button_Callback(h, eventdata, handles, varargin)
	[filename, pathname] = mygetfile('*.data', 'Load Data...', handles);
	if filename == 0
        return
	end
	try
        fullLoadName = fullfile(pathname, filename);
        busy(handles.figure1)
		load('-mat',fullLoadName)
        idle(handles.figure1)
        
        if ~exist('dataVersion')
            dataVersion = 0;
        end
        if ~exist('YLabelInfo')
            YLabelInfo = [];
        end
        
        set(handles.fileName,'String',fileName)
        set(handles.dataVersion,'String',num2str(dataVersion), 'UserData', dataVersion)
        set(handles.Operations, 'String', Operations)
        set(handles.LineInfo, 'UserData', LineInfo)
        set(handles.YLabelInfo, 'UserData', YLabelInfo)
        if exist('px_per_minute')
            set(handles.px_per_minute,'String',num2str(px_per_minute),'UserData',px_per_minute)
        else
            set(handles.px_per_minute, 'String', 'empty')
        end
	catch
        idle(handles.figure1)
        error(lasterr)
	end

% --------------------------------------------------------------------
function varargout = Operations_Callback(h, eventdata, handles, varargin)
% Stub for Callback of the uicontrol handles.Operations.
disp('Operations Callback not implemented yet.')


% --------------------------------------------------------------------
function varargout = save_button_Callback(h, eventdata, handles, varargin)
    [saveName savePath] = myputfile('*.data', 'Save data...', handles);
    if saveName == 0
        return
    end
    try
        fileName = get(handles.fileName, 'String')
        
        fullSaveName = fullfile(savePath, saveName);
        busy(handles.figure1)
        save(fullSaveName,'fileName', '-append')
        disp(['Appended to ' fullSaveName])
        idle(handles.figure1)
    catch
        idle(handles.figure1)
        errordlg(lasterr,'Save file error','modal'), return
    end


% --------------------------------------------------------------------
function varargout = figure1_KeyPressFcn(h, eventdata, handles, varargin)
if ~isequal(get(h,'UserData'), 'debug_on')
    set(h,'UserData','debug_on')
    disp('Debug Mode On')
    keyboard
    disp('Debug Mode Off')
end


% --------------------------------------------------------------------
function varargout = set_prep_Callback(h, eventdata, handles, varargin)
    S = {'rotate_img_clockwise', 'mirror_h_img', 'clean_img'};
    set(handles.Operations,'String', S)
% ------------------------------------------------------------------------
% MYPUTFILE / MYGETFILE - maintains the last directory the user was in.
function [filename, pathname] = myputfile(filterspec, title, handles)
    if ischar(filterspec)
       filterspec = fullfile(myCD(handles), filterspec);  % Pre-pend the path
    end
    [filename, pathname] = uiputfile(filterspec, title);
    if ~isequal(pathname, 0)
        setMyCD(pathname, handles)
    end
function [filename, pathname] = mygetfile(filterspec, title, handles)
    if ischar(filterspec)
        filterspec = fullfile(myCD(handles), filterspec); % Pre-pend the path
    end
    [filename, pathname] = uigetfile(filterspec, title);
    if ~isequal(pathname, 0)
        setMyCD(pathname, handles)
    end
        
% ------------------------------------------------------------------------
% SETMYCD, MYCD - set and get user's current directory
function setMyCD(path, handles)
%     path = fileparts(string); % Double check
%     if ~isequal(path, string)
%         warning(['Passed non-path: "' path '"  "' string '"'])
%     end
    set(handles.load_button,'UserData',path)
function path = myCD(handles)
    path = get(handles.load_button,'UserData');
    if isempty(path)
        path = pwd;
    end
%     path = fileparts(string)
%     if ~isequal(path, string)
%         warning(['Passed non-path: ' path ' ' string])
%     end

% --------------------------------------------------------------------
% IDLE
function idle(theFigure)
% Idle -- Set the arrow-cursor.
%  Idle(theFigure) sets the arrow-cursor in theFigure.
%   The companion routine is "Busy".
 
% Copyright (C) 1996 Dr. Charles R. Denham, ZYDECO.
% All Rights Reserved.
if ~any(findobj('Type', 'figure')), return, end
if nargin < 1, theFigure = gcf; end
set(theFigure, 'Pointer', 'arrow');

% --------------------------------------------------------------------
% BUSY
function busy(theFigure)
% Busy -- Set the watch-cursor.
%  Busy(theFigure) sets the watch-cursor in theFigure.
%   The companion routine is "Idle".F
 
% Copyright (C) 1996 Dr. Charles R. Denham, ZYDECO.
% All Rights Reserved.
if ~any(findobj('Type', 'figure')), return, end
if nargin < 1
   theFigure = gcf;
end
set(theFigure, 'Pointer', 'watch');