%PREVIEW - Creates the Export window.
%   PREVIEW contains the code to operate the export window.
%   This includes, displaying data, sampling data, and
%   building the Matlab SAC and SAC data structures to be
%   exporting in by GUI.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Steve Chuang
%
% This software is Copyright ? 2002 The Regents of the University of California. 
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its 
% documentation for educational, research and non-profit purposes, without fee, 
% and without a written agreement is hereby granted, provided that the above 
% copyright notice, this paragraph and the following three paragraphs appear in 
% all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting Technology Transfer Office, 9500 Gilman Drive, 
% 411 University Center, University of California, La Jolla, CA 92093-0093
% (858) 534-5815, invent@ucsd.edu
% 
% This software program and documentation are copyrighted by The Regents of the
% University of California. The software program and documentation are 
% supplied "as is", without any accompanying services from The Regents. The 
% Regents does not warrant that the operation of the program will be
% uninterrupted 
% or error-free. The end-user understands that the program was developed for 
% research purposes and is advised not to rely exclusively on the program for any 
% reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO
% ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
% CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING
% OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
% EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF
% CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
% THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND
% THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO
% PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
% MODIFICATIONS.
%
% This software was made with government support under 
% Office of Naval Research Award Number: N000140110403. 
% The government has certain rights in this software.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function varargout = Preview(varargin)
% PREVIEW Application M-file for Preview.fig
%    FIG = PREVIEW launch Preview GUI.
%    PREVIEW('callback_name', ...) invoke the named callback.
% Parameters:
%    Info.final_data
%    Info.Markser
%    Info.retval_handle
%    Info.depmin, etc...
%

% Last Modified by GUIDE v2.0 31-Mar-2003 19:41:01

if nargin == 1  % LAUNCH GUI

	fig = openfig(mfilename,'reuse');

	% Generate a structure of handles to pass to callbacks, and store it. 
	handles = guihandles(fig);
    guidata(fig, handles);

    % ---- INITIALIZE STUFF ------------
    Info = varargin{1};                          % All the data we need
    Info.LineStartEnd = Info.LineStartEnd-1;     % Change from index to time
    Info.final_data = Info.final_data/Info.dpmm; % Scale the final data

    % Parse BRK filenames if possible.
    if isfield(Info, 'image_filename')
        [yr, mo, dy] = parse_brk_filename(Info.image_filename);
        if ~isempty(yr) & ~isempty(mo) & ~isempty(dy)
            set(handles.start_time, 'String', [num2str(mo) '/' num2str(dy) '/19' num2str(yr) ','])
            start_time_Callback(handles.start_time, [], handles)
        end
    end
    
    setNum(handles.scale, 1);   %  SCALE=1, change to 1, and scale data before. Info.dpmm);
    setNum(handles.original_px_per_minute, Info.pixPerMinute);
    setNum(handles.working_px_per_minute, Info.pixPerMinute);
    px_per_sample = setNum(handles.px_per_sample, Info.pixPerMinute/60/1);
    Info.pixPerMinute = 0;      % Set to zero to make sure we don't use it again

    set(handles.figure1,'HandleVisibility','on','ResizeFcn','Preview(''figure1_ResizeFcn'',gcbo,[],guidata(gcbo))');
    % Clear small axes so user tdoes not have to close window everytime!
    delete(get(handles.small_axes,'Children'))  
    handles.Info = Info;                        % Save all the Info for future access
    
    % CREATE a pseudo sample of the data
    nData = length(Info.final_data);
    set(handles.small_axes,'UserData', nData)
    
    create_small_pseudo_plot(Info.final_data, handles)

    % Plot real data stuff
    % Clear big axes so user does not have to close window everytime!
    delete(get(handles.big_axes,'Children'))    
    delete(get(handles.line_markers,'Children'))
    plot(0:(nData-1), Info.final_data,'w-', 'Parent', handles.line_markers, 'Tag','final_data')
    
    SmMarkerLabels = 1:size(Info.LineStartEnd,1);
    MarkerLabels = [];
    for line=1:size(Info.LineStartEnd,1)
        MarkerLabels = strvcat(MarkerLabels,['line ' num2str(line)]);
    end
    set([handles.line_markers handles.big_axes handles.small_axes],'XLim',[0 nData+1])
    set(handles.line_markers,'XTick',Info.LineStartEnd(:,1),'XTickLabel' ...
        ,MarkerLabels,'Color','none')
    set(handles.small_axes,'XTick',Info.LineStartEnd(:,1),'XTickLabel',SmMarkerLabels)
    
    AXES_MARGIN = 1;
    hold on, rectangle('Position',[0 0 1 1],'EdgeColor','r','LineWidth',2,...
        'Tag','box','Parent',handles.small_axes)

    resample_and_plot_data(handles.samples_per_sec, px_per_sample, handles);       % Update big axes plot
    
    figure1_ResizeFcn(handles.figure1,[],handles)
    set(handles.slider,'Value',0)
    set(handles.figure1,'HandleVisibility','Callback','WindowButtonDownFcn',...
        'Preview(''window_down_fcn'',gcbo,[],guidata(gcbo))')
    
    totalTime = toNum(handles.total_time);  % resample_and_plot sets total_time
    setNum(handles.original_total_time, totalTime);
    setNum(handles.view_pts, min(6000,totalTime));
    setNum(handles.b, 0);
    setNum(handles.o, MISSING_DATA);
    setNum(handles.a, MISSING_DATA);
    setNum(handles.f, MISSING_DATA);
    update_window(handles.view_pts, [], handles)

    % Save the initial string of end_time so we can restore it
    set(handles.end_time,'UserData',get(handles.end_time,'String'))
    
    % ----------------------------------
    
	guidata(fig, handles);
    
    if nargout > 0
        varargout{1} = fig;
    end

elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK

	try
		[varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
	catch
		disp(lasterr);
	end

end

% --------------------------------------------------------------------
% Name; UPDATE_WINDOW
function update_window(h, eventdata, handles, varargin)
    if isequal(get(h,'Tag'),'view_pts')
        update_xtick(handles)
    end
    
    % Convert seconds to view to pixels to view
    px_per_sec = toNum(handles.working_px_per_minute)/60;
    nPtsView = round(str2double(get(handles.view_pts,'String'))).*px_per_sec;
    if isempty(nPtsView) | ~isreal(nPtsView)
        helpdlg('Number of data points not a number')
        return
    end
    % Number of data points of big plot.
    nData = length(handles.Info.final_data);
    PAD = min([round(0.1*nData) 50 nPtsView]);
    value = get(handles.slider,'Value');
    set([handles.line_markers handles.big_axes],'XLim',[value value+nPtsView-1])
    if nData-nPtsView+PAD <= 1-PAD
        set(handles.slider,'Enable','off','Min',0,'Max',1,'Value',0)
        value = 0;
    else
        slMin = 1-PAD;
        slMax = nData-nPtsView+PAD;
        set(handles.slider,'Min',slMin,'Max',slMax,'Value',min(max(value,slMin),slMax) ...
            ,'Enable','on','SliderStep',[(min(nPtsView,(slMax-slMin))/3/(slMax-slMin)) 0.1])
    end
    smP = get(handles.small_axes,'Position');
    ylim = get(handles.small_axes,'YLim');
    AXES_MARGIN = 2;        % Top of box can't be seen without adding a margin
    ylim(1) = min(ylim);
    ylim(2) = max(ylim)-AXES_MARGIN;
    set(findobj(handles.small_axes,'Tag','box'),...
        'Position', [value min(ylim) nPtsView max(ylim)-min(ylim)])
    
    set(get(handles.small_axes,'XLabel'),'String','LINE NUMBER'...
        ,'HorizontalAlignment','center', 'Position', ...
        [mean(get(handles.small_axes,'XLim')) 25+max(get(handles.small_axes,'YLim'))])
    DAR = get(handles.big_axes,'DataAspectRatio');
    set(get(handles.big_axes,'YLabel'),'String','Amplitude (millimeters)' ...
        ,'HorizontalAlignment','center','Position', ...
        [min(get(handles.big_axes,'XLim'))-DAR(1)/25 mean(get(handles.big_axes,'YLim'))] )
    
% --------------------------------------------------------------------
% UPDATE_BIG_AXES_PLOT - RESAMPLE_AND_PLOT_DATA - replot data
% When samples/sec changes or the pixels/minute changes, we have to 
% resample the data and plot the data again.
function resample_and_plot_data(h, px_per_sample, handles)
    if isempty(px_per_sample)
        px_per_sample = toNum(handles.px_per_sample);
    end
    if ~isfinite(px_per_sample)
        helpdlg('Invalid number found. Cannot compute sampling rate.')
        return
    end
    
    % Data already scaled?  Check initialization code to see if final_data scaled.
    px_per_sec = toNum(handles.working_px_per_minute)/60;
    maxRate = floor(px_per_sec);
    if px_per_sample < 1
        helpdlg(['Error: Sampling rate exceeds images resolution limit of ' num2str(maxRate) ' samples per second.'],'Cannot Set Sample Rate')
        return
    end

    S = get(handles.sampling_method,'String');          % Get list of methods
    method = S{get(handles.sampling_method,'Value')};   % Get selected method
    
    state = uisuspend(handles.figure1);
    set(handles.figure1, 'Pointer', 'watch');
    [tt, SeisData] = sampleData(px_per_sample, handles.Info.final_data, method);
    uirestore(state)
    set(handles.figure1, 'Pointer', 'arrow');
    
    delete(get(handles.big_axes,'Children'))
    
    % Add epsilon to max value to ensure that YLim is increasing in the case taht
    % all data values are equal
    set([handles.line_markers handles.big_axes],'YLim', ...
        [nanmin(handles.Info.final_data)-abs(eps) nanmax(handles.Info.final_data)+abs(eps)])
    plot(tt,SeisData,'w.','Parent',handles.big_axes,'Tag','big_plot' ...
        ,'MarkerSize',6, 'MarkerEdgeColor', 'y', 'MarkerFaceColor', 'y')
    % Scaled now, dont' needt o change labels -> change_ylabel_px_to_mm(handles.big_axes, handles)  
    set(handles.big_axes,'YTickMode','auto','YTickLabelMode','auto')
    
    % Set min string and max string
    setNum(handles.depmin, nanmin(SeisData), 3);
    setNum(handles.depmax, nanmax(SeisData), 3);
    setNum(handles.depmen, nanmean(SeisData), 3);
    
    % Update real total time
    % Compare two ways to calculate time, test of precision errors
    samples = length(SeisData);
    totalTime = samples/(px_per_sec/px_per_sample);
    time_check = samples/toNum(handles.samples_per_sec);
   
    if time_check ~= totalTime
        warning(['Mismatched total time calculations (' ...
                  num2str(totalTime) ' ~= ' num2str(timeCheck) ').'])
    end
    setNum(handles.b,0);
    setNum(handles.total_time, totalTime);
    match_end_time_to_total_time(totalTime, handles)
    
    update_window([], [], handles)

    
% ------------------------------------------------------------
% CHANGE_YLABEL_PX_TO_MM
function change_ylabel_px_to_mm(h, handles)
if ~ishandle(h)
    error('Invalid handle')
else
    set(h, 'YTickMode', 'auto') % Take advantage of some info the auto provides
    ytickmm = get(h,'YTick')/handles.Info.dpmm;
    maxmm = ceil(max(ytickmm));
    minmm = floor(min(ytickmm));
    ytlabel = minmm:1:maxmm;
    ytick = [minmm:maxmm].*handles.Info.dpmm;
    
    set(h,'YTick', ytick, 'YTickLabel', ytlabel)
end
    
% ------------------------------------------------------------
% Name: WORKING_PX_PER_MINUTE_Callback
function working_px_per_minute_Callback(h, eventdata, handles)
    px_per_sample_Callback(handles.px_per_sample,[],handles)
    update_xtick(handles)
% ------------------------------------------------------------
% Name: WORKING_PX_PER_MINUTE_Callback
function update_xtick(handles)
    n_x_pxls = length(handles.Info.final_data); % Number of pixels
    px_per_minute = toNum(handles.working_px_per_minute);
    
    % Set big axes
    V =Inf;    % Min visibility to label
    W = .1;
    nMinutesView = round(str2double(get(handles.view_pts,'String')))./60;
    Z = min(V,W*nMinutesView);

    xtick = 0:Z.*px_per_minute:(n_x_pxls+px_per_minute);  % Set a tick at each minute, go at least to end
    xticklabel = [0:length(xtick)].*Z ;                 % Scale to Z
    xticklabel = round(xticklabel.*100)./100;   % Chop to 2 decimal places
    
    set(handles.big_axes, 'XTick', xtick, 'XTickLabel', xticklabel)
    
% --------------------------------------------------------------------
% Name: PX_PER_SAMPLE_Callback
function px_per_sample_Callback(h, samples_per_sec, handles)
    if isempty(samples_per_sec)
        samples_per_sec = toNum(handles.samples_per_sec);
    end
    px_per_sec = toNum(handles.working_px_per_minute)/60;
    px_per_sample = px_per_sec/samples_per_sec;
    setNum(handles.px_per_sample, px_per_sample);
    resample_and_plot_data(h, px_per_sample, handles);
    
% --------------------------------------------------------------------
% Name: SAMPLES_PER_SEC_Callback
function samples_per_sec_Callback(h, eventdata, handles)
    samples_per_sec = initNum(h,[]);
    px_per_sample_Callback(h, samples_per_sec, handles)
    
% --------------------------------------------------------------------
function f= msgwindow(String)
    FigPos = get(0,'defaultFigurePosition');
    FigPos(3:4) = [ 360 65 ];
    f =dialog('MenuBar','none','Interruptible','off',...
        'Resize','on','NumberTitle','off','IntegerHandle','off','Units','pixels',...
        'Position',FigPos,'Name',String,'WindowStyle','modal');
    txt = text(mean(get(gca,'XLim')), mean(get(gca,'YLim')),String,'HorizontalAlign','center');
    set(findobj(f,'Type','axes'),'Visible','off')


% Name: SAMPLEDATA_ SAMPLING_
% sampleRate - pixels per sample
% final_data - connected raw pixel values of data
% Returns: tt - time (seconds, start from 0), SeisData - amplitude
function [tt, SeisData] = sampleData(sampleRate, final_data, method, fig)

round_method = 'round';     % Needed if method is 'integral'

% Method of choice hardwired to be NAN_SPLINE. Anything else
% does not produce as accurate of results.
method = 'nan_spline';

switch lower(method)
case 'integral'
    % Method 0: round sampleRate to nearest pixel
   
    switch round_method
    case 'floor', newSampR = ceil(sampleRate);
    case 'ceil', newSampR = floor(sampleRate);
    case 'round', newSampR = round(sampleRate);
    otherwise error('Unknown round method')
    end
    
    warning(['Sampling at not quite samples_per_sec, samples/pixel: ' num2str(newSampR) ', originally: ' num2str(sampleRate)])
    xx = 1:newSampR:length(final_data);
    SeisData = final_data(xx);
    
case 'nan_nearest'
    % Method 1: Round to nearest pxl
    xx = 1:sampleRate:length(final_data);
    SeisData = final_data(round(xx));
    
case 'nan_spline'
    % Method 2: Interpolate
    xx = 1:sampleRate:length(final_data);
    BoolArray = isfinite(final_data(round(xx)));
    xx_real = xx(BoolArray);
    
    f =  msgwindow('Sampling data using bicubic interpolation...');
    drawnow
    SeisData = spline(find(isfinite(final_data)),final_data(isfinite(final_data)),xx_real);
    if ishandle(f)
        close(f);    
    end

    gaps = zeros(size(find(~BoolArray)));
    gaps(:) = NaN;
    SeisData = [SeisData(:); gaps(:)];
    idx = [find(BoolArray); find(~BoolArray)];
    Data = sortrows([idx SeisData]);
    SeisData = Data(:,2);
case 'resample_m'
    if any(isnan(final_data(:)))
        warning(['Cannot use ' method ' method for sampling if NaNs exist. ' ...
                 'Removing ' num2str(length(find(isnan(final_data)))) ' NaNs.'])
        final_data = final_data(~isnan(final_data));
    end
    
    start = clock;  % Set a timer
    SeisData = resample(final_data, 100, round(100*sampleRate)); % Resample at 1/sampleRate
    etime(clock,start)  % Report the time used to sample
   
    xx = linspace(1,length(final_data),length(SeisData));
    if length(SeisData) ~= length(xx)
        error('Algorithm for calculating time values does not match resample.m.')
    end
otherwise
    % Method 3: Interpolate without dealing with NaNs to be faster
    if any(isnan(final_data(:)))
        warning(['Cannot use ' method ' method for sampling if NaNs exist. ' ...
                 'Removing ' num2str(length(find(isnan(final_data)))) ' NaNs.'])
        final_data = final_data(~isnan(final_data));
    end
    xx = 1:sampleRate:length(final_data);
    
    start = clock;  % Set timer
    SeisData = interp1(1:length(final_data), final_data, xx, method);
    etime(clock,start)  % Report time used to sample
end

tt = xx-1;      % Convert index to time starting at 0

    
% --------------------------------------------------------------------
% Name: BUTTON_Callback
function varargout = button_Callback(h, eventdata, handles, varargin)
%     set(handles.figure1,'UserData','ok')
eventdata = get(h,'Tag');

switch eventdata
case {'sac_button', 'mat_button', 'both_button'}
    drawnow     % Attempt to resolve any editbox Callbacks in progress
    samples_per_sec = 1;
    Values = {'nzyear' 'nzjday' 'nzhour' 'nzmin' 'nzsec' 'nzmsec'...
              'b' 'o' 'a' 'f' 'samples_per_sec' 'depmin' 'depmax' 'depmen'};
    for n=1:length(Values)
        % If you can figure a way to have strcat not return a type cell, go for it.
        string = strcat(Values(n), '= toNum(handles.', Values(n), ');');
        eval(string{:});
    end
	% Check that strings entered are valid numbers.
    errorString = [];
    if length(get(handles.kevnm,'String')) > 16
        errorString = strvcat(errorString, 'Event name is limited to 16 characters.'); end
    if length(get(handles.kstnm,'String')) > 8
        errorString = strvcat(errorString, 'Station name is limited to 8 characters.'); end
    if length(get(handles.kcmpnm,'String')) > 8
        errorString = strvcat(errorString, 'Component name is limited to 8 characters.'); end
	if isnan(nzyear) | nzyear < 1000 | nzyear > 2999
        errorString = strvcat(errorString, 'Invalid year value.'); end
    if isnan(nzjday) | nzjday < 1 | nzjday > 366
        errorString = strvcat(errorString, 'Invalid Julian day.'); end
	if isnan(nzhour) | nzhour < 0 | nzhour > 59
        errorString = strvcat(errorString, 'Invalid nzhour value.'); end
    if isnan(nzmin) | nzmin < 0 | nzmin > 59
        errorString = strvcat(errorString, 'Invalid nzmin value.'); end
	if isnan(nzsec) | nzsec < 0 | nzsec > 59
        errorString = strvcat(errorString, 'Invalid nzsec value.'); end
	if isnan(nzmsec) | nzmsec < 0 | nzmsec > 1000
        errorString = strvcat(errorString, 'Invalid nzmsec value.'); end
	if isnan(b)
        errorString = strvcat(errorString, 'Invalid Event Origin value.'); end
    if isnan(a)
        errorString = strvcat(errorString, 'Invalid First Arrival value.'); end
    if isnan(f)
        errorString = strvcat(errorString, 'Invalid Event End value.'); end
    if ~any(isMissingData([toNum(handles.a) toNum(handles.o)])) & (a < o) 
        errorString = strvcat(errorString, 'First arrival must be greater than Event Origin and less than Event End.'); end
    if ~any(isMissingData([toNum(handles.a) toNum(handles.f)])) & (a > f) 
        errorString = strvcat(errorString, 'First arrival must be less than Event End.'); end
    if ~any(isMissingData([toNum(handles.o) toNum(handles.f)])) & (o > f) 
        errorString = strvcat(errorString, 'Event Origin must be less than Event End.'); end
	if isnan(samples_per_sec) | samples_per_sec <= 0
        errorString = strvcat(errorString, 'Invalid samples_per_sec value.'); end
	if isnan(depmin)
        errorString = strvcat(errorString, 'Invalid depmin value.'); end
	if isnan(depmax) | depmin > depmax
        errorString = strvcat(errorString, 'Invalid depmax value.'); end
	if isnan(depmen) | depmen < depmin | depmen > depmax
        errorString = strvcat(errorString, 'Invalid depmen value.'); end
    
    % Print out error dialog
    if ~isempty(errorString)
        errordlg(strvcat('Errors:',errorString),'Invalid Values','modal')
    else
        set_retval(eventdata, handles)  % Return from this window
    end

case 'cancel_button'
    set_retval(eventdata, handles)  % Return from this window
otherwise
    error(['Error: Unrecognized output option: ' button_name])
end

% --------------------------------------------------------------------
% Name: SET_RETVAL
function set_retval(button, handles)

switch button
case 'cancel_button'
    SACdata = [];
    SeisData = [];
otherwise
    S = get(handles.sampling_method,'String');          % Get list of methods
    method = S{get(handles.sampling_method,'Value')};   % Get selected method

    if ~isequal(method,'nan_spline')
        answer = GUI('myquestdlg','Using nearest neighbor sampling is unreliable. Continue?','Warning' ...
                ,'Yes','No','No');
        if isequal(answer,'No')
            return
        end
    end
    SeisData = get(findobj(handles.big_axes,'Tag','big_plot'),'YData');
    % Create SACDATA header struct
    SACdata = ...
    struct(...
        'times', struct(... % Same as sac2000
            'delta', 1/toNum(handles.samples_per_sec) ...  % seconds/sample
		    ,'b', toNum(handles.b) ...
		    ,'e', toNum(handles.b)+toNum(handles.total_time) ...    % -1 or no? Is end time the start or end of the last data point?
            ,'o', toNum(handles.o) ...
            ,'a', toNum(handles.a) ...
            ,'t0',[],'t1',[],'t2',[],'t3',[],'t4',[],'t5',[],'t6',[],'t7',[],'t8',[],'t9',[] ...
            ,'f', toNum(handles.f) ...
            ,'ko','','ka','','kt0','','kt1','','kt2','','kt3','','kt4','','kt5','','kt6','','kt7','','kt8','','kt9','','kf','') ...
        ,'station', struct(...  % Same as sac2000
            'stla', [] ...
            ,'stlo', [] ...
            ,'stel', [] ...
            ,'stdp', [] ...
            ,'cmpaz', [] ...
            ,'cmpinc', [] ...
            ,'kstnm', get(handles.kstnm,'String') ...
		    ,'kcmpnm', get(handles.kcmpnm,'String') ...
            ,'knetwk', '' ) ...
        ,'event', struct(...    % Same as sac2000
            'evla', [] ...
            ,'evlo', [] ...
            ,'evel', [] ...
            ,'evdp', [] ...
            ,'nzjday', toNum(handles.nzjday) ...
		    ,'nzyear', toNum(handles.nzyear) ...
		    ,'nzhour', toNum(handles.nzhour) ...
		    ,'nzmin', toNum(handles.nzmin) ...
		    ,'nzsec', toNum(handles.nzsec) ...
		    ,'nzmsec', toNum(handles.nzmsec) ...
            ,'kevnm', get(handles.kevnm,'String') ...
            ,'mag', [] ...
            ,'imagtyp', [] ...
            ,'imagsrc', [] ) ...
     ...,'user', ???? 
     ...,'descrip', ????
     ...,'evsta', ????
     ...,'llnl',  ????
     ...,'response', ????
        ,'trcLen',  length(SeisData) ...    % Same as sac2000
        ...%%% Below this line lies stuff not in sac2000 %%%...
        ,'SeisDig', struct(...
            'num_pixels', length(handles.Info.final_data) ...
            ,'px_per_min', toNum(handles.working_px_per_minute) ...  % Changed by total time changing
		    ,'samples_per_sec', toNum(handles.samples_per_sec) ...
            ,'px_per_delta', toNum(handles.px_per_sample) ...
            ,'original_px_per_min', toNum(handles.original_px_per_minute) ...
            ,'sampling_method', method ...   % Include or not?
            ,'min_per_line', [] ...      % Time per trace
            ,'physical_length', [] ...  % Inches per trace
            ,'dpi', [] ...      % pixels per inch
            ,'dpmm', [] ... % pixels per mm
            ,'image_filename', [] ...
            )...
    );
end

if length(SACdata) > 1
    error('Sac header struct not created properly.')
end

switch button
case 'cancel_button'
    close(gcbf)     % This must go before setting retval_handle cuz it
                    % delete func sets its own value for retval
end
retval = {button, SACdata, SeisData};
set(handles.Info.retval_handle, 'UserData', retval)

% Call GUI's export function
GUI('export_sac',retval, [], guidata(handles.Info.retval_handle))

% --------------------------------------------------------------------
% FIGURE1_RESIZEFCN
% Pos = [x y w h]
function varargout = figure1_ResizeFcn(h, eventdata, handles, varargin)
    MARGIN = 10;  SL_Y = 155; SL_H = 20; SM_H = 30; T_MARG = 30;
    MARG_L = 50;
    MARG_R = 55;
    figP = fix(get(handles.figure1,'Position'));
    
    set(handles.small_axes,'Position',...
        [MARG_L figP(4)-SM_H-T_MARG max(1,figP(3)-MARG_R) SM_H]);
    set([handles.line_markers handles.big_axes],'Position',...
        [MARG_L SL_Y+SL_H max(1,figP(3)-MARG_R) max(1,figP(4)-SL_Y-MARG_L-SM_H-T_MARG-12)]);
    set(handles.slider,'Position',...
        [MARG_L SL_Y max(figP(3)-MARG_R) SL_H]);

    % Set the TICK LENGTH via CODE instead of GUIDE
    set(handles.small_axes,'TickLength', [0.0025 0.0025])
    set(handles.big_axes,'TickLength', [0.005 0.005])
    
    W = 165;
    big_axes_Pos = get(handles.big_axes,'Position');
    xlabel_Pos = [big_axes_Pos(1)+big_axes_Pos(3)/2-W/2, big_axes_Pos(2)+big_axes_Pos(4)+18, W, 17];
    
    xlabel_h = findobj(handles.figure1,'Tag','big_axes_xlabel');
    if length(xlabel_h) > 1
        delete(xlabel_h), warning('Corrupt big axes x title deleted.')
        xlabel_h = [];
    end
    if isempty(xlabel_h)
        uicontrol(handles.figure1,'Style','text','Tag','big_axes_xlabel' , 'BackgroundColor', get(handles.figure1,'Color') ...
            ,'Position', xlabel_Pos,'String','Time (minutes)','FontWeight','demi');
    else set(xlabel_h,'Position', xlabel_Pos)
    end
    
% --------------------------------------------------------------------
% WINDOW_DOWN_FCN
function window_down_fcn(h, eventdata, handles)
    smPt = get(handles.small_axes,'CurrentPoint');
    smXLim = get(handles.small_axes,'XLim');
    smYLim = get(handles.small_axes,'YLim');
    x = smPt(1,1);
    y = smPt(1,2);
    
    if x >= smXLim(1) & x <= smXLim(2) & y >= smYLim(1) & y <= smYLim(2)
        slMin = get(handles.slider,'Min');
        slMax = get(handles.slider,'Max');
        x = min(max(x,slMin),slMax);        % Make sure in range
        set(handles.slider,'Value',x)
        update_window(handles.figure1,[],handles)
    end
    
% --------------------------------------------------------------------
% PREV_NEXT_CHECKBOX
function prev_next_checkbox_Callback(h, eventdata, handles)
    if get(h,'Value') == get(h,'Max')
        set(h,'String', 'Nmax')
    else set(h,'String','Line')
    end
    set(h, 'UserData', [])  % Reset count 
    
% --------------------------------------------------------------------
% NEXT_BUTTON_Callback
function varargout = next_button_Callback(h, eventdata, handles, varargin)
if get(handles.prev_next_checkbox,'Value') == get(handles.prev_next_checkbox,'Max')
    find_suspicious(h, eventdata, handles)
else
    %%% Normal Code Below
    Markers = get(handles.line_markers,'XTick');
    xlim = get(handles.big_axes,'XLim');
    d = xlim(2) - xlim(1);
    value = get(handles.slider,'Value');
    value = Markers(min(find(Markers > value+d)))-d/2;
    if ~isempty(value)
        set(handles.slider,'Value',value)
        update_window(h,[],handles)
    end
end

% --------------------------------------------------------------------
% PREV_BUTTON_Callback
function varargout = prev_button_Callback(h, eventdata, handles, varargin)
if get(handles.prev_next_checkbox,'Value') == get(handles.prev_next_checkbox,'Max')
    find_suspicious(h, eventdata, handles)
else
    %%% Normal Code Below
    Markers = get(handles.line_markers,'XTick');
    xlim = get(handles.big_axes,'XLim');
    d = xlim(2) - xlim(1);
    value = get(handles.slider,'Value');
    value = Markers(max(find(Markers < value)))-d/2;
    if ~isempty(value)
        set(handles.slider,'Value',value)
        update_window(h,[],handles)
    end
end

% --------------------------------------------------------------------
% NUM_TXT_Callback
function num_txt_Callback(h, eventdata, handles)
    initNum(h, []);

% --------------------------------------------------------------------
% TONUM - read number in userdata or in string of handle.
%   Returns NaN if not a valid number.
function num = toNum(h)
	num = get(h,'Value');
    
    if fix(str2double(get(h,'String'))) ~= fix(num)
        warning(['Mismatch number: ' get(h,'Tag') '(value = ' num2str(num) ', string = ' get(h,'String') '). Using string to sychronize.'])
        num = str2double(get(h,'String'))
        set(h,'Value', num) % String to sync
%                 set(h,'String', num2str(get(h,'Value')))  % Value to sync
    end
    
	if isempty(num)
        %disp(['Value not set in ' get(h,'Tag')])
        num = str2double(get(h, 'String'));
	end
% --------------------------------------------------------------------
% INSYNC
function bool = insync(h)
    bool = fix(str2double(get(h,'String'))) == fix(get(h,'Value'));
    
% --------------------------------------------------------------------
% INITNUM - sets the value based on the String
function num = initNum(h, num)
    if isempty(num)
        num = str2double(get(h, 'String'));
        set(h, 'Value', num)
    else
        warning('Warning to programmer: Using outdated opertion of initNum. Use setNum instead.')
        set(h, 'Value', num, 'String', num2str(num))
    end
    
% --------------------------------------------------------------------
% SETNUM - Performs optional rounding operations. Uses input.
function num = setNum(h, number, varargin)
	if length(varargin) > 0
        num = chop(number,varargin{1});
        set(h,'Value',number,'String',num2str(num))
	else % No display length specified.
        num = number;
        set(h,'Value',number,'String',num2str(num))
	end
    
% --------------------------------------------------------------------
% o_Callback
function o_Callback(h, eventdata, handles)
    nSacPts = length(get(findobj(handles.big_axes,'Tag','big_plot'),'XData'));
    try
        origin = initNum(handles.o, []);
    catch
        warning(lasterr)
        origin = 0;
    end

    num = [];
    if isnan(origin)
        helpdlg( 'Invalid Event Origin value.');
        delete(findobj(handles.figure1,'Tag','BeginMark'))
    elseif ...
    (~isMissingData(toNum(handles.f)) & toNum(handles.f) < origin) | ...
    (~isMissingData(toNum(handles.a)) & toNum(handles.a) < origin) ...
        helpdlg('Event Origin must be less than First Arrival and Event End.');
        num = setNum(handles.o,MISSING_DATA);
    else
        num = setNum(handles.o, origin);
    end
    
    num = num*toNum(handles.working_px_per_minute)/60;
    setEventLine(num, 'BeginMark', handles)
% --------------------------------------------------------------------

function setEventLine(num, Tag, handles)
    marker_h = findobj(handles.big_axes,'Tag',Tag);
    if isempty(num) | isMissingData(num)
        delete(marker_h)
    else
        ylim = get(handles.big_axes,'YLim');
        if isempty(marker_h)
            set(handles.big_axes,'NextPlot','add')
            plot(num+zeros(size(ylim)), ylim, 'm-', 'Tag',Tag ...
                ,'Parent', handles.big_axes, 'LineWidth', 1)
            set(handles.big_axes,'NextPlot','replacechildren')
        else
            set(marker_h,'XData', num+zeros(size(ylim)),'YData', ylim)
        end
    end
% --------------------------------------------------------------------
% A_Callback
function a_Callback(h, eventdata, handles)
    nSacPts = length(get(findobj(handles.big_axes,'Tag','big_plot'),'XData'));
    try
        arrive = initNum(handles.a, []);
    catch
        warning(lasterr)
        arrive = 0;
    end
    
    num = [];
    if isnan(arrive)
        helpdlg('Invalid First Arrival value.');
        delete(findobj(handles.figure1,'Tag','ArriveMark'))
    elseif ...
    (~isMissingData(toNum(handles.o)) & arrive < toNum(handles.o)) | ...
    (~isMissingData(toNum(handles.f)) & arrive > toNum(handles.f)) ...
        helpdlg('First Arrival value must be greater than Event Origin and less than Event End.');
        num = setNum(handles.a, MISSING_DATA);
    else
        num = setNum(handles.a, arrive);
    end
    
    num = num* toNum(handles.working_px_per_minute)/60;
    
    setEventLine(num, 'ArriveMark', handles)

% --------------------------------------------------------------------
% F_Callback
function f_Callback(h, eventdata, handles)
    nSacPts = length(get(findobj(handles.big_axes,'Tag','big_plot'),'XData'));
    try
        endd = initNum(handles.f, []);
    catch
        warning(lasterr)
        endd = 1;
    end
    
    num = [];
    if isnan(endd)
        helpdlg('Invalid Event End value.');
        delete(findobj(handles.figure1,'Tag','EndMark'))
    elseif ...
    (~isMissingData(toNum(handles.o)) & endd < toNum(handles.o)) | ...
    (~isMissingData(toNum(handles.a)) & endd < toNum(handles.a)) ...
        helpdlg('Event End value must be greater than Event Origin and First Arrival.');
        num = setNum(handles.f, MISSING_DATA);   % Set to missing data cuz using other values may be missing data
    else
        num = setNum(handles.f, endd);
    end
    
    num = num* toNum(handles.working_px_per_minute)/60;
    
    setEventLine(num, 'EndMark', handles)

% --------------------------------------------------------------------
% ISMISSINGDATA
function bool = isMissingData(x)
    bool = (x == MISSING_DATA);
function x = MISSING_DATA
    x = -12345;
    
% --------------------------------------------------------------------
% GET_INPUT
function get_input(h, o_a_f, handles)
if get(h,'Value') == get(h,'Max')
    set(h,'UserData',get(h,'BackgroundColor'))  % Set user data to old bgcolor
    set(h,'BackgroundColor', 'm')
    
    [x,y] = ginput(1);
    x = round(x./toNum(handles.working_px_per_minute)*60);
    
    switch o_a_f
    case 'origin'
        setNum(handles.o, x);
        o_Callback(handles.b, [], handles)
    case 'arrival'
        setNum(handles.a, x);
        a_Callback(handles.a, [], handles)
    case 'fini' 
        setNum(handles.f, x);
        f_Callback(handles.f, [], handles)
    otherwise
        error(['Invalid button type: ' o_a_f])
    end
end
set(h,'Value',get(h,'Min'))
set(h,'BackgroundColor', get(h,'UserData'))

% ------------------------------------------------------------
% Name: TOTAL_TIME_Callback
function total_time_Callback(h, eventdata, handles)
	totalTime = initNum(handles.total_time,[]);

	nData = length(handles.Info.final_data);
	wPixPerMinute = setNum(handles.working_px_per_minute, nData/(totalTime/60));   % Updates pixels per minute!
	
	% Delegate resampling to working_px_per_minute_Callbcak    
	working_px_per_minute_Callback(handles.working_px_per_minute,[],handles)
    
	% Update the end time to match
	match_end_time_to_total_time(totalTime, handles)
    
% --------------------------------------------------------------------
% Name: MATCH_END_TIME_TO_TOTAL_TIME
function match_end_time_to_total_time(totalTime, handles)
try
    PIVOTYEAR = 1903;
    FORMAT = 2;         % Must be this format to fix up year
    start_time = datenum(get(handles.start_time,'String'), PIVOTYEAR);
    end_time = start_time + totalTime/(24*60*60);    % Convert seconds to days
    
    Year = datevec(start_time, PIVOTYEAR);
    Year = num2str(Year(1));
    Year = Year(1:2);       % First two digits only
    Date = datestr(end_time,FORMAT,PIVOTYEAR);
    Date = [Date(1:6) Year Date(7:end)];    % Fix up year, assume FORMAT = 2
    Time = datestr(end_time,14,PIVOTYEAR);
    set(handles.end_time,'String', ...
       [Date ', ' Time])

    % Save so we can restore if user hits no later
    set(handles.end_time,'UserData',get(handles.end_time,'String'))
catch
    % Ignore improperly formatted start time.
end

% --------------------------------------------------------------------
% CREATE_SMALL_PSEUDO_PLOT
function create_small_pseudo_plot(final_data, handles)
    Pos = fix(get(handles.small_axes,'Position'));
    w = Pos(3); % width
    h = Pos(4); % height
    img = zeros(h,w);
    rate = length(final_data)/w;
    ratio = max(abs(eps),  (max(final_data)-min(final_data))/h);   % Must be greater than 0 to avoid div0
    r = [];
    c = 1;
    for x=1:rate:length(final_data)
        xx = round([x:min(length(final_data),x+rate)]);   % Keep in array bounds.
        r = round(max(1,min(h/2+final_data(xx)/ratio,h)));
        img(r,c) = 255;
        c = c+1;
    end
    image(img, 'Parent', handles.small_axes, 'XData', [1 length(final_data)])
    colormap([0 0 0; 1 1 1])
    set(handles.small_axes, 'YLim', [1 h])
    
% ------------------------------------------------------------
% FIND_SUSPICIOUS
function find_suspicious(h, eventdata, handles)
    count = get(handles.prev_next_checkbox,'UserData');
    if isempty(count), count = 0;, end

    if h == handles.prev_button
        count = max(1,count-1);
    else
        count = count+1;
    end
    
    ydata = abs(get(findobj(handles.big_axes,'Tag','big_plot'),'YData'));
    xdata = get(findobj(handles.big_axes,'Tag','big_plot'),'XData');
    for I=1:count
        idx = min(find(ydata == max(ydata)));
        ydata(idx) = -Inf;
    end
    value = xdata(idx);
        
    xlim = get(handles.big_axes,'XLim');
    d = xlim(2) - xlim(1);
    value = xdata(idx)-d/2;
    
    if ~isempty(value)
        set(handles.slider,'Value',value)
        update_window(h,[],handles)
    end
    set(handles.prev_next_checkbox,'UserData', count)

% -----------------------------------------------------------
% Name: START_TIME_Callback
% Description: converts standard string into individual numbers
function start_time_Callback(hObject, eventdata, handles)
    try 
        PIVOTYEAR = 1903;
        [y,m,d,hr,mi,s] = datevec(get(handles.start_time,'String'), PIVOTYEAR);
        jday = d2j([y,m,d,hr,mi,s]);                 % Convert to Julian 0-365or366
        setNum(handles.nzjday, jday);
        setNum(handles.nzyear, y);
        set(handles.nzmonth,'Value',m);
        setNum(handles.nzhour, hr);
        setNum(handles.nzmin, mi);
        setNum(handles.nzsec, fix(s));
    catch
        helpdlg(strvcat('Incorrectly entered start time format.', get(handles.start_time,'String'),' Use: mm/dd/yyyy, hr:mn:ss'),'Date string error')
        lasterr
    end
    % Update end time if callback object is the start string
    match_end_time_to_total_time(toNum(handles.total_time), handles)
    
% -----------------------------------------------------------
% Name: END_TIME_Callback
% Description:
function end_time_Callback(hObject, eventdata, handles)
    try
        PIVOTYEAR = 1903;
        [y,m,d,hr,mi,s] = datevec(get(handles.start_time,'String'), PIVOTYEAR);
        end_date = datevec(get(handles.end_time,'String'), PIVOTYEAR);
        if isempty(end_date), error('Empty end date'), end
        seconds = round(etime(end_date, [y,m,d,hr,mi,s]));
        oldseconds = toNum(handles.original_total_time);
        if oldseconds ~= seconds
            answer = GUI('myquestdlg', strvcat(['Digitized total time: ' num2str(oldseconds)], ...
                                      ['Entered total time: ' num2str(seconds)], ...
                     'Use entered total time?'), 'Total Time', 'Yes','No','Yes');
        else
            answer = 'No';
        end
        if isequal(answer,'Yes')
            newnum = setNum(handles.total_time, seconds);            
          
            total_time_Callback(handles.total_time, [], handles)
        else
            DefaultString = 'mm/dd/yyyy, hr:mn:ss';
            if isempty(get(handles.end_time,'UserData'))
                set(handles.end_time,'String',DefaultString)
            else
                % Restore old string to undo changes
                set(handles.end_time,'String',get(handles.end_time,'UserData'))
            end
        end
    catch
        % End time ignored.
    end
    
% -----------------------------------------------------------
% Name: COMPUTE_TIME_DIFF
% Description: Takes two datevecs and computes the seconds in between.
function seconds = compute_time_diff(date1, date0)
    seconds = 0;
    try
        seconds = etime(date1, date0);
    catch
        error('Possible invalid datevec format.')
    end

function X = chop(Xin,n,unit)
%CHOP   CHOP(X,n) rounds elements of X to n significant figures.
%       CHOP(X,n,unit) rounds the elements of X to n significant
%   figures whose digits (mantissa) are exactly divisible
%   by unit. 
%       
%       e.g. chop(3.141592,5)   returns 3.141600000..
%       e.g. chop(3.141592,3,5) returns 3.150000000..
%            chop(3.141592,3,3) returns 3.150000000..
%            chop(3.141592,3,2) returns 3.140000000..
%      

%   Copyright 1986-2000 The MathWorks, Inc. 
%   $Revision: 1.75 $  $Date: 2003/02/27 06:44:52 $

% Set last sig. fig. rounding to 1 if only two input arguments.
if nargin<3,
   unit=1; 
end

% Cater for -ve numbers  and numbers = 0.
X = abs(Xin) +(Xin==0);
[nx,mx] = size(X);
exponent = unit.*((10*ones(nx,mx)).^(floor(log10(X))-n+1));
X = round(X./exponent).*exponent;

% Put back sign and zeros
X = sign(Xin).*X.*(Xin~=0);
% end chop



%*********************************************************************
function julian_day=d2j(ddate)
%
%	function julian_day=date2julian(ddate)
%
%  Returns the "Julian day" of the "year" starting at 0000 hr of "year" 
%  giving Julian days 1-365 (366 for leap years). 
% 
%  "ddate" is a vector of the form:  [ year month day hr min sec ]
%            (hr min sec are not required)
%*********************************************************************
if isempty(ddate), fprintf(1,' Error:  "ddate"  NOT specified \n'); return; end
ld=length(ddate);
if ld<3,
  day=1; else day=ddate(3);
end
if ld<2,
  fprintf(1,' Warning:  "month"  Not specified;  January  assumed \n'); 
  month=1;
else 
  month=ddate(2);
end
year=ddate(1);

month_days=[31 28 31 30 31 30 31 31 30 31 30 31];

if leap(year), month_days(2)=29; end
mdays=sum(month_days(1:month-1));    % Julian days to begining of "month"

julian_day = mdays+day;
if nargout==0, fprintf(1,'   Julian:   %g \n',julian_day); end

%----------------------------------------------------------
function lp=leap(year)
%	function lp=leap(year)
%
% Returns 1 for each element of "year" that is a leap year,
%         0 otherwise
%  ("year" can be 2-digits for 1950-2050)
%----------------------------------------------------------
if year>=50 & year<100, year=year+1900; elseif year<50, year=year+2000; end
ly=length(year);
lp=ones(size(year));

inds=find(rem(year,4)~=0 | (rem(year,100)==0 & rem(year,400)~=0));

lp(inds)=0;