%% FIX_FILE_REFERENCE
% This function takes in two file expressions.
% The first is the file expression of files you wish to open.
% This must be in the form of /pathtofiles/*.ext
% Only the extension is checked.
%
% The second expression is the modification.
% The modification string must be in the form of 
% /goodpath/prefix* 
function fix_file_reference(openfiles, newstr)
    [srcpath,srcfile,srcext] = fileparts(openfiles);
    D = dir(srcpath);

    if isempty(D)
        disp(['No files in directory'])
    end
    
    response = 'n';
    for I=1:length(D)
        if ~D(I).isdir
            workingfile = D(I).name;
            workingfile = fullfile(srcpath, workingfile);
            [p,f,e] = fileparts(workingfile);
            
            % Check if correct file type. Empty means accept all file
            % extensions.
            if isempty(srcext) || isequal(e,srcext)
                if ~isequal(response,'a')
                    disp(workingfile)
                    response = lower(input('Operate on this file? (y/n/a) => ', 's'));      % Ignore case
                end
                
                switch response
                case {'a', 'y'}
                    try
                        load(workingfile, 'fileName', '-mat');
                        
                        % Check if it exists
                        if ~exist('fileName')
                            error('Could not find file reference')
                        end
                        
                        disp(['Old file name: ' fileName])
                        fileName = fix1(fileName, newstr);
                        disp(['New file name: ' fileName])
                        
                        % VERY IMPORTANT THAT WE -APPEND, INSTEAD OF
                        % OVERWRITE!
                        save(workingfile, 'fileName', '-append') 
                        disp(['Saved to ' workingfile])
                        disp('')
                        
                    catch
                        error(lasterr)
                    end
                end
            end
        end
    end
    
% -----------------
function newFileName = fix1(fileName, newstr)
    [newpath, newfile, newext] = fileparts(newstr);
    
    % Make all separators uniform.
    fileName(fileName == '\' | fileName == '/') = filesep;
    [srcpath, srcfile, srcext] = fileparts(fileName);
    [yr mo dy] = parse_brk_filename(srcfile);
 
    yrstr = num2str(yr);
    mostr = num2str(mo);
    dystr = num2str(dy);
    if length(yrstr) == 1
        yrstr = ['0' yrstr];
    end
    if length(mostr) == 1
        mostr = ['0' mostr];
    end
    if length(dystr) == 1
        dystr = ['0' dystr];
    end
    srcfile = ['brk_' mostr '_' dystr '_' yrstr];

    prefix = strtok(newfile, '*');
    newFileName = fullfile(newpath, [prefix srcfile srcext]);
    
    % Check if newFileName exists
    D = dir(newFileName);
    if isempty(D)
        disp(newFileName)
        error('New file reference does not exist!')
    end
 
