%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Steve Chuang
%
% This software is Copyright ? 2002 The Regents of the University of California. 
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its 
% documentation for educational, research and non-profit purposes, without fee, 
% and without a written agreement is hereby granted, provided that the above 
% copyright notice, this paragraph and the following three paragraphs appear in 
% all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting Technology Transfer Office, 9500 Gilman Drive, 
% 411 University Center, University of California, La Jolla, CA 92093-0093
% (858) 534-5815, invent@ucsd.edu
% 
% This software program and documentation are copyrighted by The Regents of the
% University of California. The software program and documentation are 
% supplied "as is", without any accompanying services from The Regents. The 
% Regents does not warrant that the operation of the program will be
% uninterrupted 
% or error-free. The end-user understands that the program was developed for 
% research purposes and is advised not to rely exclusively on the program for any 
% reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO
% ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
% CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING
% OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
% EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF
% CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
% THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND
% THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO
% PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
% MODIFICATIONS.
%
% This software was made with government support under 
% Office of Naval Research Award Number: N000140110403. 
% The government has certain rights in this software.


% --------------------------------------------------------------------
% GET_CONNECTED_PIXELS - vectorized algorithm to do simple task.
% Returns the indices of pixels of the white line plus one on top and botm.
function [y_index, top, botm] = get_connected_pixels(x,y, Img, g_width, Merge)
    [top, botm] = getRange(x,y, g_width*2, Img, Merge);
    y_index = top:botm;
	% -----------------------------------
	% GETRANGE - Also detects if we ran off the cropped image!
	function [top, botm] = getRange(x,y, width, Img, Merge)
        nRows = size(Img,1);            % Max botm value.
        maxWidth = max(y-1,nRows-y);    % Know when to stop (at Img size limits)
        [top, botm] = getTopAndBotm(x,y,width, Img, nRows, Merge);
        
        % Expand range while necessary to capture top and bottom of line.
        while range_not_big_enough(top, botm)
%             disp(['width expanded at subX =' num2str(x)])   %::DEBUG::
            width = width + 10;
            [top, botm] = getTopAndBotm(x,y,width, Img, nRows, Merge);   % Repeat
            if width > maxWidth
                % ::DEBUG::
%                 figure(x), imagesc(Img(:,max(1,x-10):min(x+10,size(Img,2)))), title('Debugging information')
                if isempty(top) | isempty(botm)
                    disp(['Trace exceeded max amplitude at subX=' num2str(x)])
                    if isempty(top), top = 1, end
                    if isempty(botm), botm = nRows, end
                end
                return
            end
        end% End while loop to catch top and bottom of line
    % ---------------------------------
	% GETTOPANDBOTM
    function [top, botm] = getTopAndBotm(x,y,g_width, Img, nRows, Merge)
        [nSlice, subY, offset] = getInvertedSlice(x,y, g_width, Img, nRows, Merge);
        blacks = find(nSlice);
        top = max(blacks(blacks<=subY)) + offset;    % Row above top of line
        botm = min(blacks(blacks>=subY)) + offset;   % Row below bottom of line
	% ---------------------------------
	% GETINVERTEDSLICE - get inverted slice from big image
    % Merge - how many lines total to merge together, forward biased.
    % This function rounds g_width and y to get indices.
	function [nSlice, subY, offset] = getInvertedSlice(x,y, g_width, Img, nRows, Merge)
        top = max([1, floor(y-g_width) ]);        % Top of range (smaller number)
        botm = min([ ceil(y+g_width) ,nRows]);    % Botm of range (larger number)
        y_range = top:botm;                     % get indices, rows and cols
        x_range = max([1,(x-floor((Merge-1)/2))]):min([size(Img,2),(x+ceil((Merge-1)/2))]);
        nSlice = ~any(Img(y_range,x_range),2);  % Merge colmns together
        offset = top-1;
        subY = y-offset;
    % ---------------------------------
	% RANGE_NOT_BIG_ENOUGH - make sure range big enough to surround line
	function retval = range_not_big_enough(top, botm)
        retval = isempty(top) | isempty(botm);
