%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Steve Chuang
%
% This software is Copyright ? 2002 The Regents of the University of California. 
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its 
% documentation for educational, research and non-profit purposes, without fee, 
% and without a written agreement is hereby granted, provided that the above 
% copyright notice, this paragraph and the following three paragraphs appear in 
% all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting Technology Transfer Office, 9500 Gilman Drive, 
% 411 University Center, University of California, La Jolla, CA 92093-0093
% (858) 534-5815, invent@ucsd.edu
% 
% This software program and documentation are copyrighted by The Regents of the
% University of California. The software program and documentation are 
% supplied "as is", without any accompanying services from The Regents. The 
% Regents does not warrant that the operation of the program will be
% uninterrupted 
% or error-free. The end-user understands that the program was developed for 
% research purposes and is advised not to rely exclusively on the program for any 
% reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO
% ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
% CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING
% OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
% EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF
% CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
% THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND
% THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO
% PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
% MODIFICATIONS.
%
% This software was made with government support under 
% Office of Naval Research Award Number: N000140110403. 
% The government has certain rights in this software.


% --------------------------------------------------------------------
% GET_NEXT_PT - find connected line
% x - target x to get a new point at.
function [x, y, dif] = get_next_pt(x,prev_y,y_index, Img, varargin)

    %::DEBUG::
    if isempty(y_index), error('_GET_NEXT_PT: y_index empty, indexing error.'); end
    if x > size(Img,2) | x < 1, error(['_GET_NEXT_PT: x is out of bounds:' num2str(x)]); end

%     if length(varargin) > 0
%         Merge = varargin{1};
%     else Merge = 1;
%     end
%     % Incorporate merging
%     M1 = floor((Merge-1)/2); % # of extra columns caught due to merging in get_connected.
%     M2 = ceil((Merge-1)/2);
%     Slice = double(any(Img(y_index,max(1,(x-M1)):min(size(Img,2),(x+M2))),2));
    
    Slice = double(Img(y_index,x));    % No Merge
    
    Slice = [0; Slice; 0];      % Pad with zeros  
    
    dif = diff(Slice);
    flipB = find(dif < 0);      % White to Black (1 to 0)
    switch length(flipB)
    case 0                      % No white lines
        y = [];
    case 1                      % Single white line
        y = pick_any_y(x, y_index, Img);
    otherwise                   % Multiple lines
%         disp(['Multiple lines at subX= ' num2str(x)])   %::DEBUG::
        y = get_paths(flipB, dif, x, y_index);
        x = x(ones(size(y)));    % Provide matching x for each y.
    end
	% -----------------------------------
	% Simple pick any path method.
	function y = pick_any_y(x, y_index, Img)
        y = y_index(find(Img(y_index,x)));
        if ~isempty(y)
            y = y(ceil(end/2));
        end
	% -----------------------------------
	% GET_PATHS - list possible paths.
	%  Sorts the paths by most likely on top.
	function [y, dif] = get_paths(flipB, dif, x, y_index)
        flipW = find(dif > 0)-1;              % Black to White (0 to 1)
        FLIP_OFFSET = min(y_index);         % +1 for top black pixel, -1 for index offset.
        y = get_mids(flipB, flipW) + FLIP_OFFSET;
	% -----------------------------------
	% GET_MIDS
	function y = get_mids(flipB, flipW)
        y = flipW + (flipB-flipW)./2-1;       % Grab mid pt of each block.
        