%GUI Main SeisDig file.
%   GUI provides a wrapper for most of the code that provides 
%   the functionality of SeisDig.
%   The functions within this file can be called by passing in
%   the function name as a character string followed by the arguments
%   normally passed to the function.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Steve Chuang
%
% This software is Copyright ? 2002 The Regents of the University of California. 
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its 
% documentation for educational, research and non-profit purposes, without fee, 
% and without a written agreement is hereby granted, provided that the above 
% copyright notice, this paragraph and the following three paragraphs appear in 
% all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting Technology Transfer Office, 9500 Gilman Drive, 
% 411 University Center, University of California, La Jolla, CA 92093-0093
% (858) 534-5815, invent@ucsd.edu
% 
% This software program and documentation are copyrighted by The Regents of the
% University of California. The software program and documentation are 
% supplied "as is", without any accompanying services from The Regents. The 
% Regents does not warrant that the operation of the program will be
% uninterrupted 
% or error-free. The end-user understands that the program was developed for 
% research purposes and is advised not to rely exclusively on the program for any 
% reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO
% ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
% CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING
% OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
% EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF
% CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
% THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND
% THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO
% PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
% MODIFICATIONS.
%
% This software was made with government support under 
% Office of Naval Research Award Number: N000140110403. 
% The government has certain rights in this software.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function varargout = GUI(varargin)
if nargin == 0  % LAUNCH GUI
    fig = openfig(mfilename,'new');
    
     % Use system color scheme for figure:
    set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));

    % Generate a structure of handles to pass to callbacks, and store it. 
    handles = guihandles(fig);

    % ------------------------------------------------------
    % INITIALIZATION CODE GOES HERE
    % ------------------------------------------------------
    resize_both_axes(handles)       % Resize the big and small axes

    % Set shortcut keys for submenu items
    set(handles.accelerators,'Visible', 'off')
    Keys = get(handles.accelerators,'Children');
    set(Keys,'Callback','GUI(''key_press_fcn'',guidata(gcbo),get(gcbo,''Label''))')
    for n=1:length(Keys)
        set(Keys(n),'Accelerator', get(Keys(n),'Label'))
    end
    set(handles.undo_button,'Accelerator','u')

    set(handles.figure1,'HandleVisibility','on') % Must turn on for load img to find axes
    LOGO = [SEISDIGPATH filesep 'logo.gif'];     % File name of the logo
    if ~isempty(dir(LOGO))
        handles = load_img_from(LOGO,handles);
    else
        warning(['Cannot find logo image ''' LOGO '''. Loading blank image.'] )
        handles = load_img_from(zeros(100,200),handles);
    end
    %set(handles.figure1,'HandleVisibility','Callback')
    
    setMyCD(pwd, 'default', handles);    % Set current path to current dir
    file_expr_Callback(handles.file_expr, pwd, handles)
    
    % After everything is loaded we can procede to set the
    % Mouse motion buttons and the resize functions.
    set(fig...
        ,'WindowButtonMotionFcn', 'GUI(''generic_motion_fcn'',guidata(gcbo))'...
       ,'WindowButtonUpFcn', ''...
       ,'WindowButtonDownFcn', 'GUI(''generic_down_fcn'',guidata(gcbo))'...
        ,'KeyPressFcn', 'GUI(''key_press_fcn'',guidata(gcbo))'...
        ,'ResizeFcn','GUI(''figure1_ResizeFcn'',gcbo,[],guidata(gcbo))'...
        )
    
    
    set(findobj(handles.figure1,'Style','text'),'Visible','off')
    % ------------------------------------------------------
    guidata(fig, handles);  % Save the handle structure

elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK
    try
        if (nargout)
            [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
        else
            feval(varargin{:}); % FEVAL switchyard
        end
    catch
        disp(lasterr);
    end
end


% -------------------
% SEISDIGPATH - finds the director that seisdig was invoked from
function S = SEISDIGPATH
    [pth, nm, ext] = fileparts(which('seisdig'));
    S = pth;


% --------------------------------------------------------------------
% Name: RESIZE_BOTH_AXES
% Description: Resizes the big axes to match the window size.  Resizes 
%              the height of the small to match the window size.  
%              The width is determined by LOAD_IMG_FROM to match the 
%              proportions of the image.  The height as of 7/15/02 
%              remains the same throughout the program, but later versions 
%              may want to resize it when the window is enlarge or reduced.
% Pos = [x y w h]
% SLIDER_THICK - the thickness of the sliders for positioning the big axes.
%
function resize_both_axes(handles)
    % Margins
    TOP = 20+20; LEFT = 120; RITE = 20; BOTTM =180; % +20 Leave room for Inches axis label
    
    fig_Pos = get(handles.figure1, 'Position');         % Get position of figure
    big_axes_Pos = get(handles.big_axes, 'Position');   % Get position of axis
    
    big_axes_Pos(1) = LEFT;       % set x position: measured from top
    big_axes_Pos(2) = BOTTM;      % set y: measure from bottom
    
    % Axes width: fig width - left margin -  right margin.   We ignore yslider width.
    % Axes height:  fig height - top margin - bottm margin.  We ignore xslider width
    big_axes_Pos(3) = max(BOTTM, fig_Pos(3) - LEFT - RITE);
    big_axes_Pos(4) = max(BOTTM, fig_Pos(4) - TOP - BOTTM);
    set(handles.big_axes,'Position', big_axes_Pos)
    
    W = 165;
    xlabel_Pos = [big_axes_Pos(1)+big_axes_Pos(3)/2-W/2, big_axes_Pos(2)+big_axes_Pos(4)+20, W, 20];
    xlabel_h = findobj(handles.figure1,'Tag','big_axes_xlabel');
    if length(xlabel_h) > 1
        delete(xlabel_h), warning('Corrupt big axes x title deleted.')
        xlabel_h = [];
    end
    if isempty(xlabel_h)
        uicontrol(handles.figure1,'Style','text','Tag','big_axes_xlabel' ...
            ,'Position', xlabel_Pos,'String','Trace Length (inches)','FontWeight','demi');
    else set(xlabel_h,'Position', xlabel_Pos)
    end
    
    % RESET THE SMALL_AXES 
    T_MARGIN = 40;  B_MARGIN = 5;  SM_RITE = 5;  SM_LEFT = 630;
    sm_axes_Pos = get(handles.small_axes,'Position');
    sm_axes_Pos(4) = BOTTM - T_MARGIN;                  % Height
    sm_axes_Pos(2) = B_MARGIN;                          % y distance from bottom
    sm_axes_Pos(1) = fig_Pos(3)-sm_axes_Pos(3)-SM_RITE;
    
    img_handle = findobj(handles.big_axes,'type','image');
    if length(img_handle) ~= 1,
        img_siz = [1 1];
    else, img_siz = size(get(img_handle,'CData'));
    end
    
    % Do not constrain the width of the small axes
    
    % Load values into object
    set(handles.small_axes,'Position', sm_axes_Pos)
    
    reset_slider(handles, big_axes_Pos, img_siz)

% --------------------------------------------------------------------
% Name: RESET_SLIDER
% Description: Sets up the x and y sliders to line up with the big_axes.  
%              Sets the range of the sliders so that the minimum value 
%              is the index of the top left most pixel in the image, and 
%              the maximum is the top left most pixel if we are viewing 
%              the bottom right corner of the image.
% Parameters:  axes_Pos - Position vector of big_axes
%               img_siz - size vector of the image on the big_axes
% Uses:        When big_axes resized or ZOOM changed.
% Postion: [x y w h]
function reset_slider(handles, axes_Pos, img_siz)
    THICK = 16;         % Slider thickness 
    OFF_MAX = 2;
    OFF_MIN = 1; 
    
    axes_Pos = get(handles.big_axes, 'Position');
    
    % Set position and dimensions of slider to match big axes
    x_slid_Pos = [axes_Pos(1), axes_Pos(2)-THICK, axes_Pos(3), THICK];
    y_slid_Pos = [axes_Pos(1)+axes_Pos(3)+1, axes_Pos(2), THICK, axes_Pos(4)];
    set(handles.x_slider, 'Position', x_slid_Pos)
    set(handles.y_slider, 'Position', y_slid_Pos)
    
    ZOOM = getZoom(handles);
    STRETCH = getStretch(handles);
    [num_y_pixels_shown, num_x_pixels_shown] = num_pixels_2show(img_siz, axes_Pos, ZOOM, STRETCH);
    
    y_max = img_siz(1) - num_y_pixels_shown + 1;    % 1 value means slider should be off
    x_max = img_siz(2) - num_x_pixels_shown + 1;    % 1 value means slider should be off
    
    % Set slider step values
    % 1st = arrow, 2nd = trough
    y_slider_step(2) = min(1.0 ,(1/2 * num_y_pixels_shown) / y_max);
    x_slider_step(2) = min(1.0, (1/2 * num_x_pixels_shown) / x_max);
    y_slider_step(1) = min(1.0 ,(1/5 * num_y_pixels_shown) / y_max);
    x_slider_step(1) = min(1.0, (1/5 * num_x_pixels_shown) / x_max);
    
    % Make sure step size is at least one pixel so that show_img will show a new pixel
    if y_max > 1,
        if y_slider_step(1)*(y_max-1) < 1, y_slider_step(1) = 1/(y_max-1); end
        if y_slider_step(2)*(y_max-1) < 1, y_slider_step(2) = 1/(y_max-1); end
    end
    if x_max > 1,
        if x_slider_step(1)*(x_max-1) < 1, x_slider_step(1) = 1/(x_max-1); end
        if x_slider_step(2)*(x_max-1) < 1, x_slider_step(2) = 1/(x_max-1); end
    end
    
    % Save slider values and check in range
    current_y = get(handles.y_slider, 'Value');
    current_x = get(handles.x_slider, 'Value');
    if current_y > y_max, current_y = y_max; end
    if current_x > x_max, current_x = x_max; end
    
    if y_max > 1,
        set(handles.y_slider,'Enable', 'on', 'min',1,'max',y_max,'Value',current_y,'sliderstep',y_slider_step)
    else  
        set(handles.y_slider,'Enable', 'off','Value',OFF_MAX,'min',OFF_MIN,'max',OFF_MAX)
    end 
    
    if x_max > 1,
        set(handles.x_slider,'Enable', 'on', 'min',1,'max',x_max,'Value',current_x,'sliderstep',x_slider_step)
    else  
        set(handles.x_slider,'Enable', 'off','Value',OFF_MIN,'min',OFF_MIN,'max',OFF_MAX)
    end

% --------------------------------------------------------------------
% Name: RESET_ROW_VALUES
function reset_row_values(handles)
    set(handles.row_time,'String','NaN','Value',[])
    set(handles.row_length,'String','NaN','Value',[])

% --------------------------------------------------------------------
% Name: RELOAD_IMG    
function [handles, errors] = reload_img( this_img, handles )
    answer = myquestdlg('Delete all objects and reload image?', 'Reload?', 'Yes','Cancel','Cancel');
    if isequal( answer, 'Yes' )
        load_img_from(this_img,handles);
    end
    
% --------------------------------------------------------------------
% Name: LOAD_IMG_FROM
% Description:  Creates the big image object and the small image 
%               object from a file or a variable.  Also deletes any 
%               previous data, sets the ZOOM to 1, creates the 
%               navigation box in small_axes, resets the sliders, 
%               makes the width of the small_axes proportional to the 
%               image, and set the mouse motion mode to normal. 
% Parameters:
%               this_img - filename of image to load, or a matrix with image
%               handles - handles struct associated with figure1
% Returns:
%               handles - updated handles structure
%               errors - empty if no errors, else contains error msgs
% WARNING:  This function will clear your current working image!!!!
function [handles, errors] = load_img_from( this_img, handles )
    errors = [];                % Initialize as empty

    busy(handles.figure1)       % Turn on the watch cursor
    
    if ischar(this_img)
        fullfilename = this_img;
        drawnow     % Refresh the window one last time before the long wait
        disp(['Loading image from ' fullfilename])
        [path,name,ext] = fileparts(fullfilename);
        
        try
            %tic % Time how long its taking to load
            Img = imread(fullfilename);
            %disp(['imread time: ' num2str(toc)])
        catch
            idle(handles.figure1), 
            if nargout < 2
                warndlg(lasterr,'File loading error.','modal'), 
            else
                errors = strcat(errors, lasterr);
            end
            return
        end

        Img = make_black_and_white(Img);

        update_titlebar(fullfilename, handles)
    elseif isnumeric(this_img) | islogical(this_img)
        Img = this_img;
    else 
        idle(handles.figure1), 
        if nargout < 2
            warndlg('Unknown object passed into load_img_from.'),  
        else
            errors = strcat(errors, 'Unknwon object passed');
        end
        return
    end
    
    ZOOM = reset_zoom(handles);
    reset_row_values(handles)

    setLineNum(1, handles)                                  % Line number initially 1
    set(handles.line_number_txt,'UserData',{})              % Clear line labeling information   
    set(handles.big_axes, 'NextPlot','add','XColor','k', ...
        'XTickLabelMode', 'auto','XTickMode','auto')        % Change axes
    
    clear_undo_history(handles)

    fig_Pos = get(handles.figure1, 'Position');
    big_axes_Pos = get(handles.big_axes, 'Position');
    sm_axes_Pos = get(handles.small_axes, 'Position');
    img_siz = size(Img);
    
    reset_slider(handles, big_axes_Pos, img_siz)            % New image loaded, time to change those slider ranges
    
    delete(get(handles.big_axes,'Children'))                % Need this to get rid of all objects.
    delete(get(handles.small_axes,'Children'))              % Delete objects off small_axes
    
    % Create new big image object
    STRETCH = [1 1 1]; %getStretch(handles);
    [num_y_pixels_2show, num_x_pixels_2show] = num_pixels_2show(img_siz, big_axes_Pos, ZOOM, STRETCH);
    img_handle = image(Img, 'Parent', handles.big_axes,'Tag','Img');        % New image object.
    set(handles.big_axes...                                                 % Fix up the axes scaling.
        ,'YLim', [0.5, num_y_pixels_2show+0.5],'XLim', [0.5, num_x_pixels_2show+0.5]...
        )
    reset_ylabel(handles.big_axes)  % Sets y labeling to auto and black.
    
    [g_width line_spacing] = analyze_image(img_handle);
    
    figure(handles.figure1)
    handles.img_handle = img_handle;
    handles.img_siz = img_siz;
    handles.g_width = g_width;
    handles.line_spacing = line_spacing;
    guidata(handles.figure1,handles)            % Save new handles
    

    init_edge_lines(handles)    % Create green lines. Needs line spacing.
    
    % Set the colormap
    colormap([0 0 0; 1 1 1])
    
    % Change the width of the small axes
    MARGIN = 10;
    LEFT_MARGIN = 600;
    sm_axes_Pos(3) = sm_axes_Pos(4) * img_siz(2)/img_siz(1); % New width = height*img_wid/img_height
    sm_axes_Pos(1) = fig_Pos(3) - sm_axes_Pos(3) - MARGIN;   % New x_pos = fig_wid - axes_wid - margn
    set(handles.small_axes,'Position', sm_axes_Pos)
    
    % Create new small image object
    % Maintain width/height ratio.
    I_small = imresize(Img, [sm_axes_Pos(4) sm_axes_Pos(4)*img_siz(2)/img_siz(1)]); 
    image(I_small ...
        ,'Parent', handles.small_axes...
        ,'EraseMode', 'normal'...
        ,'XData', [1 img_siz(2)]...
        ,'YData', [1 img_siz(1)]...
        )
    set(handles.small_axes...
        ,'YLim', [0.5, img_siz(1)+0.5]...
        ,'XLim', [0.5, img_siz(2)+0.5]...
        )
    
    % Display new rectangle object at location (1,1)
    box_h = num_y_pixels_2show; 
    box_w = num_x_pixels_2show;
    rectangle('Position',[1 1 box_w box_h]... 
              ,'Tag', 'loc_box' ...
              ,'EdgeColor', 'red'...
              ,'LineWidth', 2 ...
              ,'Parent', handles.small_axes)

    % Move sliders to position 1,1
    set(handles.x_slider, 'Value', 1)
    set(handles.y_slider, 'Value', get(handles.y_slider,'Max'))
    
    reset_stretch(handles)
    stretch_Callback(handles)
    
    % Set x labeling to current inches, requires handles.img_handle to be set
    % and both axes to be created
    set_scaled_xlabel(handles.big_axes, handles)  

    
    idle(handles.figure1)        % Set mouse cursor to arrow

    
% --------------------------------------------------------------------
% Name: SET_SCALED_XLABEL
function set_scaled_xlabel(ax_h, handles)
    n_x_pxls = size(get_image(handles), 2); % Number of columns
    px_per_inch = getNumOrValue(handles.px_per_inch_txt);
    
    % Set big axes
    V =1;    % Min visibility to label
    W = 0.25;
    Z = min(V,W*1/getZoom(handles));             % Label at least every 1 inch
    xtick = 0:Z.*px_per_inch:(n_x_pxls+px_per_inch);    % Set a tick at each inch, go at least to end
    xticklabel = [0:length(xtick)].*Z ;                 % Scale to Z
    xticklabel = round(xticklabel.*100)./100;   % Chop to 2 decimal places
    
    set(handles.big_axes, 'XTick', xtick, 'XTickLabel', xticklabel)
    
    % Set small axes
    h = handles.small_axes;
    set(h,'XTickMode','auto')
    nticks = 2*length(get(h, 'XTick')); % Double the default chosen is experimentally good
    total_inches = n_x_pxls/px_per_inch;
    inches_per_tick = max(1,round(total_inches/nticks));
    pxls_per_tick = inches_per_tick*px_per_inch;

    xtick = px_per_inch:pxls_per_tick:n_x_pxls;
    xticklabel = [1:inches_per_tick:total_inches];
    if length(xtick) ~= length(xticklabel)
        warning('Bad xtick and xticklabels, correcting.')
        idx = (1:min(length(xtick),length(xticklabel)));
        xtick = xtick(idx);
        xticklabel = xticklabel(idx);
    end
    set(handles.small_axes,'XTick', xtick, 'XTickLabel', xticklabel)
    
% --------------------------------------------------------------------
% Name: MAKE_BLACK_AND_WHITE
function Img = make_black_and_white(Img)
    if ndims(Img) ~= 2,
        Img = uint8(Img(:,:,1));                    % Choose only one color channel
    end
    if ~islogical(Img),                             % Change image into black & white
        Img = im2double(Img);
        Img = Img > (max(Img(:))-min(Img(:)))/2;             % Pick values greater than middle value
        Img = uint8(Img);                           % Turn back to uint8
    end
    
% --------------------------------------------------------------------
% Name: CLEAR_UNDO_HISTORY
function clear_undo_history(handles)
    set(handles.undo_button,'UserData',{})                  % Clear action history.
    
% --------------------------------------------------------------------
% Name: STRETCH_Callback
% Important Note: the stretchX and stretchY boxes are actually
%    switched around on the figure to give the effect that increasing
%    the number magnified that axis, which is opposite of how
%    DataAspectRatio normally works in Matlab.  So the textbox 
%    above the label 'X' actually has the tag 'stretchY' and vice versa.
function stretch_Callback(handles, STRETCHX, STRETCHY)
    if nargin == 1
        STRETCHX = str2double(get(handles.stretchX,'String'));
        STRETCHY = str2double(get(handles.stretchY,'String'));
    else
        set(handles.stretchX,'String', num2str(STRETCHX))
        set(handles.stretchY,'STring', num2str(STRETCHY))
    end
    STRETCH = [STRETCHX STRETCHY 1];
    
    setStretch(STRETCH, handles)
    reset_slider(handles, get(handles.big_axes,'Position'), handles.img_siz)
    x_center = mean(get(handles.big_axes,'XLim'));
    y_center = mean(get(handles.big_axes,'YLim'));
    center_view_on(x_center,y_center, [], STRETCH, handles)
    
% --------------------------------------------------------------------
% Name: RESET_STRETCH
function reset_stretch(handles)
    set(handles.stretchX,'String','1', 'UserData', [])
    set(handles.stretchY,'String','1', 'UserData', [])
    setStretch([1 1 1], handles)
    
% --------------------------------------------------------------------
% GETSTRETCH
function Stretch = getStretch(handles)
    Stretch = get(handles.big_axes,'DataAspectRatio');
% --------------------------------------------------------------------
% SETSTRETCH
function setStretch(Stretch, handles)
    set(handles.big_axes,'DataAspectRatio',Stretch)
    
% --------------------------------------------------------------------
% Name: RESET_ZOOM
function ZOOM = reset_zoom(handles)   
    ZOOM = 1;
    set(handles.zoom_txt,'String', '100','UserData',ZOOM)   % ZOOM set to 100% initially
    set(handles.set_zoom_right,'Enable','on')
    set(handles.set_zoom_left,'Enable','on')
% --------------------------------------------------------------------
% Name: GETZOOM
function ZOOM = getZoom(handles)
    ZOOM = get(handles.zoom_txt,'UserData');

% --------------------------------------------------------------------
% Name: LOADING_BAR
% Description - Shows a messge that says the image is loading
% Returns - handle to figure window
% Use the return value to close the loading figure window.
function f= Loading_bar
    FigPos = get(0,'defaultFigurePosition');
    FigPos(3:4) = [ 360 65 ];
    f =dialog('MenuBar','none','Interruptible','off',...
        'Resize','on','NumberTitle','off','IntegerHandle','off','Units','pixels',...
        'Position',FigPos,'Name','loading','WindowStyle','modal');
    txt = text(mean(get(gca,'XLim')), mean(get(gca,'YLim')),'Loading...','HorizontalAlign','center');
    set(findobj(f,'Type','axes'),'Visible','off')

% --------------------------------------------------------------------
% NUM_PIXELS_2SHOW
function [num_y_pxl_2show, num_x_pxl_2show] = num_pixels_2show(img_siz, big_axes_Pos, ZOOM, STRETCH)
    num_y_pxl_2show = min( img_siz(1), ceil(big_axes_Pos(4)/ZOOM/STRETCH(1)) );
    num_x_pxl_2show = min( img_siz(2), ceil(big_axes_Pos(3)/ZOOM/STRETCH(2)) );
    
% --------------------------------------------------------------------
% Name: SHOW_IMG
% Description - Changes the axes limits to show a part of the image.  
%               Determines the ZOOM by grabbing the current string in 
%               zoom_txt. Show_img rounds the x-y values.  This function 
%               also make sure the navigation box and the sliders 
%               match what is shown.
% big_axes_Pos = [x y width height]
function show_img(x, y, handles)
    if isnan(y) | isnan(x)
        warning('Error: NaN passed to show_img'), return
    end

    % Error check image handle and image size
    if ~ishandle(handles.img_handle)
        disp('Updating handles.image handle in show_img')
        handles.img_handle = get_img_h(handles);
    end
    
    % Check if we need to update the image size field
    if ~all(handles.img_siz == ...
    [max(get(handles.img_handle,'YData')) max(get(handles.img_handle,'XData'))])
        handles.img_siz = size(get(handles.img_handle, 'CData'));
        guidata(handles.figure1, handles)
    end
    img_siz = handles.img_siz;
    img_handle = handles.img_handle;

    big_axes_Pos = get(handles.big_axes, 'Position');

    ZOOM = getZoom(handles);;
    STRETCH = getStretch(handles);
    [num_y_pixels_2show, num_x_pixels_2show] = num_pixels_2show(img_siz, big_axes_Pos, ZOOM, STRETCH);

    % Show whole pixels only
    x = round(x);   y = round(y);
    % Make sure index in range
    if x<1,  
        x=1;
    elseif x>img_siz(2)-num_x_pixels_2show+1,
        x=img_siz(2)-num_x_pixels_2show+1; 
    end
    if y<1, 
        y=1;
    elseif y>img_siz(1)-num_y_pixels_2show+1,
         y=img_siz(1)-num_y_pixels_2show+1; 
    end

    % Change up the axes scaling 
    set(handles.big_axes...
        ,'YLim', [y-0.5, y+num_y_pixels_2show-0.5]...
        ,'XLim', [x-0.5, x+num_x_pixels_2show-0.5]...
        )
   
    % Resyncronize the thumbnail box    
    box_h = num_y_pixels_2show;
    box_w = num_x_pixels_2show;
    if box_h > img_siz(1), box_h = img_siz(1); end
    if box_w > img_siz(2), box_w = img_siz(2); end
    
    box_handle = findobj(handles.small_axes, 'type', 'rectangle');
    if length(box_handle) ~= 1,
        error(strvcat('Error in show_img: none or more than one loc box: ',num2str(box_handle)))
        return
    end
    set(box_handle, 'Position',[x y box_w box_h])
    
    % Resyncronize the sliders (if movement caused by clicking on thumbnail)
    set(handles.x_slider, 'Value', x)
    set(handles.y_slider, 'Value', get(handles.y_slider,'Max')-y+1)
   
    % Relabel big axes yticklabels
    update_ytick_only(getYLabelInfo(handles), handles)

% --------------------------------------------------------------------
% Name: SLIDER_Callback
% Description: Reloads the image based on the current slider positions.
%      Calls show_img to do most of the work.
function slider_Callback(handles)
    if isequal( 'on', get(handles.x_slider,'Enable') )
        x = get(handles.x_slider, 'Value');
    else x = 1; 
    end
    
    if isequal( 'on', get(handles.y_slider,'Enable') ),
        y = get(handles.y_slider,'Max') - get(handles.y_slider, 'Value') + 1; 
    else y = get(handles.y_slider,'Max');
    end
    show_img(x, y, handles)
    
% --------------------------------------------------------------------   
% CENTER_VIEW_ON - center the view on a point clicked.
function center_view_on(x,y, ZOOM, STRETCH, handles)
    if isempty(ZOOM)
        ZOOM = getZoom(handles);
    end
    if isempty(STRETCH)
        STRETCH = getStretch(handles);
    end

    big_axes_Pos = get(handles.big_axes,'Position');
    x_offset = big_axes_Pos(3)/2 / ZOOM / STRETCH(2); % Offset used to center image on point clicked

    % Allow motion in all directions
    y_offset = big_axes_Pos(4)/2 / ZOOM / STRETCH(1); % Offset used to center image on point clicked
    show_img(x-x_offset,y-y_offset, handles)

% --------------------------------------------------------------------
% INVERT_IMG flips the image along the horizontal direction
function invert_img(handles)
    uistate = uiclearmode(handles.figure1);
    busy(handles.figure1)    % Turn on watch cursor
    
    % Change big image
    img_handle = get_img_h(handles);
    cdata = get(img_handle,'CData');
    set(img_handle,'CData',~cdata)
    
    % Change small image
    thumb_handle = findobj(handles.small_axes,'type','image');
    cdata = get(thumb_handle,'CData');
    set(thumb_handle,'CData',~cdata)
        
    record_operation('invert_img',img_handle, handles)
    
    % Must reanalyze image when we flip white and black
    [g_width line_spacing] = analyze_image(img_handle);
    
    handles.g_width = g_width;
    handles.line_spacing = line_spacing;
    guidata(handles.figure1,handles)            % Save new handles
    % Note: Does not update handles immediately
    
    uirestore(uistate)
    idle(handles.figure1)    % Turn on arrow cursor
    
% --------------------------------------------------------------------
% MIRROR_H_IMG flips the image along the horizontal direction, around y-axis
function mirror_h_img(handles)
    uistate = uiclearmode(handles.figure1);
    busy(handles.figure1)    % Turn on watch cursor
    
    % Change big image
    img_handle = get_img_h(handles);
    cdata = get(img_handle,'CData');
    set(img_handle,'CData',cdata(:,end:-1:1))
    
    % Change small image
    thumb_handle = findobj(handles.small_axes,'type','image');
    cdata = get(thumb_handle,'CData');
    set(thumb_handle,'CData',cdata(:,end:-1:1))
        
    record_operation('mirror_h_img',img_handle, handles)
    
    uirestore(uistate)
    idle(handles.figure1)    % Turn on arrow cursor
    
% --------------------------------------------------------------------
% MIRROR_V_IMG flips the image along the vertical direction, around x-axis
function mirror_v_img(handles)
    uistate = uiclearmode(handles.figure1);
    busy(handles.figure1)    % Turn on watch cursor
    
    % Change big image
    img_handle = get_img_h(handles);
    cdata = get(img_handle,'CData');
    set(img_handle,'CData',cdata(end:-1:1,:))
    
    % Change small image
    thumb_handle = findobj(handles.small_axes,'type','image');
    cdata = get(thumb_handle,'CData');
    set(thumb_handle,'CData',cdata(:,end:-1:1))
        
    record_operation('mirror_v_img',img_handle, handles)
    
    uirestore(uistate)
    idle(handles.figure1)    % Turn on arrow cursor
    
% --------------------------------------------------------------------
% Name: ROTATE_IMG_CLOCKWISE
% Description: Rotates the current working image CLOCKWISE.  Slow.
function rotate_img_clockwise(handles)

    % Don't let the user make the mistake of deleting the user's data
    % by rotating.
    if ~isempty(findobj(handles.big_axes,'Tag','trace_line')) ...
    |  ~isempty(findobj(handles.big_axes,'Tag','CtrlPt'))
        errordlg('Control point or trace data exists on image already! Rotating will delete data. Aborting.', 'Cannot Rotate', 'modal')
        return
    end

    uistate = uiclearmode(handles.figure1);
    busy(handles.figure1)    % Turn on watch cursor
    
    img_handle = get_img_h(handles);
    Img = get(img_handle,'CData');
    
    busy(handles.figure1)    % Turn on watch cursor
    tic
    Img = Img(end:-1:1,:).';  % rotate 90 degrees clockwise
    toc
    
    handles = load_img_from( Img, handles); % Rotate and load to reset stuff
    img_handle = handles.img_handle;        % Need to get new img_handle
    record_operation('rotate_img_clockwise',img_handle, handles)

    uirestore(uistate)
    idle(handles.figure1)    % Turn on arrow cursor
    
% --------------------------------------------------------------------
% Name: ROTATE_IMG_COUNTER
% Description: Rotates the current working image COUNTER-clockwise.  Slow.
function rotate_img_counter(handles)

    % Don't let the user make the mistake of deleting the user's data
    % by rotating.
    if ~isempty(findobj(handles.big_axes,'Tag','trace_line')) ...
    |  ~isempty(findobj(handles.big_axes,'Tag','CtrlPt'))
        error('Data exists on image already! Rotating will delete data. Aborting.')
        return
    end

    uistate = uiclearmode(handles.figure1);
    busy(handles.figure1)    % Turn on watch cursor

    img_handle = get_img_h(handles);
    Img = get(img_handle,'CData');
    
    busy(handles.figure1)    % Turn on watch cursor
    tic
    Img = Img(:,end:-1:1).';  % rotate 90 degrees counterclockwise
    toc
    
    handles = load_img_from( Img, handles); % Rotate and load to reset stuff
    img_handle = handles.img_handle;        % Need to get updated img_handle
    record_operation('rotate_img_counter',img_handle, handles)

    uirestore(uistate)
    idle(handles.figure1)    % Turn on arrow cursor

    
% --------------------------------------------------------------------
% Name: FLIP_IMG
% Description: Rotates the current working image.  Slow.
function flip_img(handles)
    uistate = uiclearmode(handles.figure1);
    busy(handles.figure1)    % Turn on watch cursor

    % Change main image
    img_handle = get_img_h(handles);
    cdata = get(img_handle,'CData');
    set(img_handle,'CData',cdata(end:-1:1,end:-1:1))
    
    % Change small image
    thumb_handle = findobj(handles.small_axes,'type','image');
    cdata = get(thumb_handle,'CData');
    % rotate 180 degrees
    set(thumb_handle,'CData',cdata(end:-1:1,end:-1:1))
        
    record_operation('flip_img',img_handle, handles)
    
    uirestore(uistate)
    idle(handles.figure1)    % Turn on arrow cursor

% -------------------------------------------------------------------- 
% Function VERSIONNUMBER - returns version as numberic number
function n = versionNumber
    v = version;
    v = v( (v >='0' & v <= '9') | v == '.');
    dot_idx = find(v == '.');
    v(dot_idx(2:end)) = '';
    n = str2double(v);
    
% --------------------------------------------------------------------
% Name: ERODE_IMG
% Description: erodes the current working image.  Slow.
function erode_img(handles)

if versionNumber >= 6.1
    uistate = uiclearmode(handles.figure1);
    busy(handles.figure1)    % Turn on watch cursor
    
    [Img img_handle] = get_image(handles);
    
    Img = myerode(Img);
    set(img_handle,'CData',Img)

    record_operation('erode_img',img_handle, handles)
    
    uirestore(uistate)
    idle(handles.figure1)    % Turn on arrow cursor
else
    % Version restriction
    warndlg('Erode not very slow on versions less than 6.1')
    
    uistate = uiclearmode(handles.figure1);
    busy(handles.figure1)    % Turn on watch cursor
    
    [Img img_handle] = get_image(handles);
    
    set(img_handle,'CData',myerode(Img))
    
    record_operation('erode_img',img_handle, handles)

    uirestore(uistate)
    idle(handles.figure1)    % Turn on arrow cursor
end

% --------------------------------------------------------------------
% MYERODE - erodes using quick or slow way
function Img = myerode(Img)
SE = ones(3,3);             % Erosion 3x2

if versionNumber >= 6.1
    M = size(Img,1);
    BWP = bwpack(Img);
    BWP = imerode(BWP, SE,'ispacked', M);
    Img = bwunpack(BWP, M);
else
    Img = erode(Img, SE, 'spatial');
end


% --------------------------------------------------------------------
% Name: DILATE_IMG
% Description: dilates the current working image.  Slow.
function dilate_img(handles)

SE = ones(3,3);                 %  dilate structuring element
if versionNumber >= 6.1
    uistate = uiclearmode(handles.figure1);
    busy(handles.figure1)    % Turn on watch cursor
    
    [Img img_handle] = get_image(handles);

    M = size(Img,1);
    BWP = bwpack(Img);
    BWP = imdilate(BWP,SE,'ispacked');
    Img = bwunpack(BWP, M);
    set(img_handle,'CData',Img)
    
    record_operation('dilate_img',img_handle, handles)

    uirestore(uistate)
    idle(handles.figure1)    % Turn on arrow cursor
else
    % Version restriction
    warndlg('Dilate much slower on versions less than 6.1')
    uistate = uiclearmode(handles.figure1);
    busy(handles.figure1)    % Turn on watch cursor

    [Img img_handle] = get_image(handles);

    set(img_handle,'CData',dilate(Img, SE, 'spatial'))
    record_operation('dilate_img',img_handle, handles)

    uirestore(uistate)
    idle(handles.figure1)    % Turn on arrow cursor
end

% --------------------------------------------------------------------
% Name: CLEAN_IMG
% Description: Removes single pixels and fills single holes
function clean_img(handles)    
    uistate = uiclearmode(handles.figure1);
    busy(handles.figure1)    % Turn on watch cursor
    
    % Change big image
    [Img img_handle] = get_image(handles);
    set(img_handle,'CData',bwmorph(Img, 'majority', 1))
    
    % Don't bother changing small image
    
    record_operation('clean_img',img_handle, handles)

    uirestore(uistate)
    idle(handles.figure1)    % Turn on arrow cursor

% --------------------------------------------------------------------
% Name: GET_IMG_H
function img_h = get_img_h(handles)
    if isfield(handles, 'img_handle')
        img_h = handles.img_handle;
    else img_h = -1;
    end
    confirm = findobj(handles.big_axes,'Tag','Img','type','image');
    if confirm ~= img_h
        img_h = confirm;
        handles.img_handle = confirm;
        guidata(handles.figure1, handles);
    end
    
    if length(img_h) ~= 1
        error('Bad img handle. (Not singular)')
    end
    
% --------------------------------------------------------------------
% Name: GET_IMAGE
function [Img, img_handle] = get_image(handles)
    img_handle = get_img_h(handles);
    Img = get(img_handle,'CData');
    
% --------------------------------------------------------------------
% Name: RECORD_OPERATION
% Description: Saves the operation for when saving state.
function record_operation(OpString, img_handle, handles)
    Operations = get(img_handle,'UserData');
    if isempty(Operations)
        Operations = {OpString};
    else
        Operations = {Operations{:} OpString};
    end
    set(img_handle,'UserData',Operations)

% --------------------------------------------------------------------
% Name: CLEAR_OPERATION_HISTORY
% Description: Clears the history of image operations performed.
function clear_operation_history(handles)
img_handle = get_img_h(handles);
if ishandle(img_handle)
    set(img_handle,'UserData', []);
else
    disp('Updating image handle.')
end

% --------------------------------------------------------------------
% Name: DISPLAYUNDOID - Disabled for now because too many outputs clutter the
% screen.
function displayUndoID(actionID)
%disp(['Undo action: ' getUndoID(actionID)])

% --------------------------------------------------------------------
% Name: GETUNODID
function S = getUndoID(actionID)
switch actionID
case 0, S = 'UNIVERSAL';
case 1, S = 'NEW_CTRL_PT';
case 2, S = 'INTERPOLATE';
case 3, S = 'FILTER';
case 4, S = 'DIGITIZE';
case 5, S = 'MOVE_EDGE';
case 6, S = 'LINE_NUMBER';
case 7, S = 'DELETE_TRACE';
case 8, S = 'CLEAR_POINTS';
case 9, S = 'DELETE';
otherwise
    S = 'unknown';
end

% --------------------------------------------------------------------
% Name: UNDO _UNDO_
% Description: 
%   Undos certain enumerated actions.
% How does UNDO_BUTTON keep track of what the last move was?
%   Each action is enumerated. The undo_button->UserData keeps a list of
%   numbers corresponding the the action.  The most recent action is listed
%   first.
% How does UNDO_BUTTON restore the previous state?
%   The data is stored with the action number by MAKE_UNDOABLE.
function undo(handles)
% ID numbers for what the last action was
UNIVERSAL = 0;
NEW_CTRL_PT = 1;
INTERPOLATE = 2;
FILTER = 3;
DIGITIZE = 4;
MOVE_EDGE = 5;
LINE_NUMBER = 6;
DELETE_TRACE = 7;
CLEAR_POINTS = 8;
DELETE = 9;
CHG_LINE_NUM = 10;

EventStack = get(handles.undo_button,'UserData');
if isempty(EventStack) | stackempty(EventStack)
    return
end
temp = top(EventStack);                               % Save the info holding actionID & actionData
actionID = temp{1};
actionData = temp{2};
lineNum = temp{3};

displayUndoID(actionID)

try
	switch actionID                                     % Identify what type of action and restore state
	case UNIVERSAL
	% Undo anything
        % Delete all text and lines
        delete(findobj(handles.figure1,'Type','line'))
        delete(findobj(handles.big_axes,'Type','text'))
        
        % Restore lines and text
        Line_info = actionData{1};
        Text_info = actionData{2};
        
        restore_objects(Line_info, handles)
        restore_objects(Text_info, handles)
        
	case NEW_CTRL_PT
        line_xdata = actionData{1};         % Points to delete
        line_handle = findobj(handles.big_axes,'Tag','CtrlPt','UserData',lineNum,'XData',line_xdata);
        
        % Delete the line handle
        if ishandle(line_handle)
            delete(line_handle)
        else warning('Programmer error in undo: bad line handle in undo NEW_CTRL_PT')
        end
        
        % Restore the control points
        ptInfo = actionData{2};         % Restore any control points that were sucked up
        restore_objects(ptInfo, handles)
        
        % 3. Restore the trace line
        traceLineInfo = actionData{3};      % Original modified parts of trace data
        if ~isempty(traceLineInfo)
            restore_trace_data(traceLineInfo, [], handles)  % No lines to delete
        else
            restore_trace_data([], lineNum, handles)        % Delete current line cuz it did not exist
        end
        update_ctrlpt_stuff(lineNum, handles)
        
	case INTERPOLATE
        x = actionData{1};                      % New x point
        lineInfo = actionData{2};               % Original line info
        line_xdata = sort([lineInfo.XData x]);  % New xdata to identify plot that we're working w/
        traceLineInfo = actionData{3};          % Original trace state
        
        % Find the line_handle
        line_handle = findobj(handles.big_axes,'Tag','CtrlPt','UserData',lineNum,'XData',line_xdata);
        
        % Restore the interpolate range points
        if ishandle(line_handle) & length(line_handle) == 1
            set(line_handle,'XData', lineInfo.XData,'YData', lineInfo.YData)
        else
            warning('Bug in code: bad handle or multiple line handles in undo INTERPOLATE')
        end
        
        % 2. Restore the trace line
        trace_line = findobj(handles.big_axes,'Tag','trace_line','UserData',lineNum);
        if ~isempty(traceLineInfo)
            restore_objects(traceLineInfo, handles, trace_line)
        else
            warning('Error undoing placing point in interpolate range: no trace data to revert to!')
            delete(trace_line)
        end
	
	case FILTER
        trace_line = findobj(handles.big_axes,'Tag','trace_line','UserData',lineNum);

        % Check for corrupt data
        if length(trace_line) ~= 1
            errordlg('Corrupted trace line caught in UNDO. cont','corrupted trace line','modal')
        end
        
        if ~isempty(actionData)
            restore_objects(actionData, handles, trace_line)
        else
         % No trace to revert to. Delete trace_line
            delete(trace_line)
        end
	case DIGITIZE
        traceInfo = actionData{1};      % New traces
        noTraceInfo = actionData{2};    % Line numbers without traces
        restore_trace_data(traceInfo, noTraceInfo, handles)
    case MOVE_EDGE
        [BigLeft, BigRight, SmLeft, SmRight] = getEdgeLines(handles);
        oldLeftX = actionData{1};
        oldLeftY = actionData{2};   % YData should not change as of now,
        oldRightX = actionData{3};   % but we don't want to make this assumption
        oldRightY = actionData{4};  % that may handicap future changes.
        
        init_edge_lines(handles)        % Will create if there aren't edge lines
        set([BigLeft, SmLeft], 'XData', oldLeftX, 'YData', oldLeftY);
        set([BigRight, SmRight], 'XData', oldRightX, 'YData', oldRightY);
	
	case LINE_NUMBER
        oldLineNum = actionData{1};
        CtrlPts = actionData{2};
        if any(~ishandle(CtrlPts))
            warning('Invalid control point in undo line number')
        end
        setLineNum( oldLineNum, handles)
        set(CtrlPts(:),'UserData',oldLineNum)
        fixYLabelInfo(handles);
	case DELETE_TRACE
        if length(actionData) == 1
            restore_objects(actionData, handles)
        else
            warning('Error: unable to undo. No previous trace data saved or corrupted data.')
        end
	case CLEAR_POINTS
        % RESTORE CLEARED POINTS
        restore_objects(actionData, handles)
	
	case DELETE
        objInfo = actionData;
        restore_objects(objInfo, handles)
        update_ctrlpt_stuff( lineNum, handles)
    case CHG_LINE_NUM
        Xcell = actionData{1};
        OldDatCell = actionData{2};
        NewDatCell = actionData{3};
        
        % Must find handles in two loops because we cannot change as we go or else
        % we might catch something we changed something to.
        Lhandle = zeros(length(Xcell), 1);
        temp = 0;
        for I=1:length(Xcell)
            line_xdata = Xcell{I};
            lineNum = NewDatCell{I};

            temp = findobj(handles.big_axes,'Tag','CtrlPt','UserData',lineNum,'XData',line_xdata);
            if length(temp) == 1
                Lhandle(I) = temp;
            else
                warning('Bad undo state: duplicate CtrlPt handles')
            end
        end
        for I=1:length(Xcell)
            lineNum = NewDatCell{I};
            old_line_num = OldDatCell{I};
            line_handle = Lhandle(I);
        
            % Change line number and set selected on
            if ishandle(line_handle)
                
                set(line_handle, 'UserData', old_line_num, 'Selected', 'on')
                
                % Update both what we changed from and what we changed to
                update_ctrlpt_stuff(lineNum, handles)
                update_ctrlpt_stuff(old_line_num, handles)
            else warning('Bad undo state: bad line handle in undo NEW_CTRL_PT')
            end
        end
	otherwise
        errordlg('Unknown case in UNDO','Undo error','modal'), return
	end
catch
    error(lasterr)
end

setLineNum( num2str(lineNum), handles)
line_number_txt_Callback(handles.line_number_txt,0,handles)   % 0 flags no changing the line numbers

% Remove the action from the list
EventStack = pop(EventStack);
set(handles.undo_button,'UserData',EventStack)

if stackempty(EventStack)
    set(handles.undo_button,'UserData',[])
end

update_smooth_button(handles);

% --------------------------------------------------------------------
% Name: RESTORE_TRACE_DATA
% Need a general way to restore trace data without saving all of it.
% There are two cases:
% case 1:  No previous trace data exists. Must save the entire trace.
% case 2:  Previous trace data exists and was modified.
% The input information needed is the traceInfo and the line numbers that
% do not have traces. The negative space of line numbers is chosen because
% in general we are dealing with 1 line which may or may not have existed.
% In which case, the noTraceInfo is just a flag to say that the trace did
% not previously exist.
% Input: traceInfo - trace info of the parts of the trace that was modified.
%        noTraceInfo - line numbers that were checked to not have previously existed.
function restore_trace_data(traceInfo, noTraceInfo, handles)
    for N=1:length(noTraceInfo)
        delete(findobj(handles.big_axes,'Tag','trace_line','UserData',noTraceInfo(N)))
    end
    
    if ~isempty(traceInfo)
        % Merge the xdata values into current trace state, replacing if necessary
        for N=1:length(traceInfo)
            tempLineNum = traceInfo(N).UserData;
            
            % Perform some heuristic checks on if tempLineNum is valid
            if ~isnumeric(tempLineNum) | length(tempLineNum)~=1 | round(tempLineNum)~=tempLineNum
                errordlg('Corrupted trace line caught in UNDO. cont','corrupted trace line','modal')
            end
            trace_line = findobj(handles.big_axes,'Tag','trace_line','UserData',tempLineNum);
            
            % Need to account for possibility that trace doesn't exist anymore
            if length(trace_line) > 1
                errordlg('Corrupted trace line caught in UNDO, cont. (not singular)','corrupted trace line','modal')
            elseif isempty(trace_line)
                % No trace exists, create new trace out of saved data
                restore_objects(traceInfo(N), handles)
            else
                % Merge data into existing trace
                xdata_A = traceInfo(N).XData;
                ydata_A = traceInfo(N).YData;
                xdata_B = get(trace_line,'XData');
                ydata_B = get(trace_line,'YData');
                [xdata, ydata] = mergeData(xdata_A,ydata_A, xdata_B,ydata_B);
                
                set(trace_line,'XData',xdata,'YData',ydata)
            end
        end
    end

% --------------------------------------------------------------------
% Name: MERGEDATA - merge A into B, replacing if duplicates
function [xdata, ydata] = mergeData(xdata_A,ydata_A,xdata_B,ydata_B)
    % Merge so that values in A inserted into B
    [xdata, IB, IA] = union(xdata_B, xdata_A);
    
    % This gets messy because we can't assume anything is in order
    xdata_A = xdata_A(IA);
    ydata_A = ydata_A(IA);
    xdata_B = xdata_B(IB);
    ydata_B = ydata_B(IB);
    
    xdata = [xdata_A(:); xdata_B(:)];
    ydata = [ydata_A(:); ydata_B(:)];
    data = sortrows([xdata ydata]); % Put side by side and sort by xdata
    xdata = data(:,1);
    ydata = data(:,2);
    
% --------------------------------------------------------------------
% Name: RESTORE_OBJECTS
% Description: Replot or restore a line based on a structs past in.
% Return handles to objects modified or created if user wants it.
function h = restore_objects(ObjectInfo, handles, ObjHandle)
if nargin == 2
    for n=1:length(ObjectInfo)
        % Try to identify parent by tag name first, else assume parent is big_axes.
        if isfield(ObjectInfo, 'ParentTag')
            parent = findobj(handles.figure1,'Tag',ObjectInfo(n).ParentTag);
        else
            %disp('No parent tag saved. Using big_axes as parent.')
            parent = handles.big_axes;
        end
        % Parent must have hold on or else plots will disappear
        switch ObjectInfo(n).Type
        case 'line'
            ObjHandle(n) = plot(1,1, 'Parent', parent);
        case 'text'
            ObjHandle(n) = text(1,1,'', 'Parent', parent);
        otherwise
            % Do nothing for unknown objects
        end
    end
elseif nargin == 3
    if length(ObjectInfo) ~= length(ObjHandle)
        error('Mismatched Info and Handle lengths')
    end
end

for n=1:length(ObjectInfo)
    switch ObjectInfo(n).Type 
    case 'line'
        set(ObjHandle(n) ...
        ,'XData',     ObjectInfo(n).XData...
        ,'YData',     ObjectInfo(n).YData...
        ,'EraseMode', ObjectInfo(n).EraseMode...
        ,'LineStyle', ObjectInfo(n).LineStyle...
        ,'LineWidth', ObjectInfo(n).LineWidth...
        ,'Color',     ObjectInfo(n).Color...
        ,'Marker',    ObjectInfo(n).Marker...
        ,'MarkerEdgeColor', ObjectInfo(n).MarkerEdgeColor...
        ,'MarkerFaceColor', ObjectInfo(n).MarkerFaceColor...
        ,'MarkerSize',      ObjectInfo(n).MarkerSize...
        ,'Tag',       ObjectInfo(n).Tag...
        ,'UserData',  ObjectInfo(n).UserData...
        ,'Color',     ObjectInfo(n).Color...
        )
    case 'text'
        set(ObjHandle(n)...
        ,'Position',  ObjectInfo(n).Position...
        ,'String',    ObjectInfo(n).String...
        ,'EraseMode', ObjectInfo(n).EraseMode...
        ,'Color',     ObjectInfo(n).Color...
        ,'Parent',    ObjectInfo(n).Parent...
        ,'Tag',       ObjectInfo(n).Tag...
        ,'FontName',  ObjectInfo(n).FontName...
        ,'FontSize',  ObjectInfo(n).FontSize...
        ,'FontUnits', ObjectInfo(n).FontUnits...
        ,'FontWeight',          ObjectInfo(n).FontWeight...
        ,'HorizontalAlignment', ObjectInfo(n).HorizontalAlignment...
        ,'VerticalAlignment',   ObjectInfo(n).VerticalAlignment...
        ,'UserData',  ObjectInfo(n).UserData...
        )
        % Try to identify parent by tag name first, else assume parent is big_axes.
        if isfield('ParentTag', ObjectInfo)
            set(ObjHandle(n), 'Parent', findobj(handles.figure1,'Tag',ObjectInfo(n).ParentTag))
        else
            set(ObjHandle(n), 'Parent', handles.big_axes)
        end
    otherwise
        warning('Unknown type in restore_objects')  
    end
end %end for loop

% Return handles if user wants it.
if nargout > 0
    h = ObjHandle;
end

% --------------------------------------------------------------------
% Name: MAKE_UNDOABLE
% Description: Saves information to the cell array in undo_button->UserData
%   to make an action undoable.
% data to pass in:
function make_undoable(actionString, actionData, handles)
    UNIVERSAL = 0;
    NEW_CTRL_PT = 1;
    INTERPOLATE = 2;
    FILTER = 3;
    DIGITIZE = 4;
    MOVE_EDGE = 5;  % actionData = {oldLeftX oldLeftY oldRightX oldRightY}
    LINE_NUMBER = 6;
    DELETE_TRACE = 7;
    CLEAR_POINTS = 8;
    DELETE = 9;
    CHG_LINE_NUM = 10;
    switch lower(actionString)
    case 'universal'
        actionID = UNIVERSAL;
        actionData = {};
        Lines = findobj(handles.big_axes, 'Type', 'line');
        actionData{1} = get(Lines(:));        % Nx1 struct array
        Texts = findobj(handles.big_axes, 'Type', 'text');
        actionData{2} = get(Texts(:));
    case 'new ctrl pt'
        actionID = NEW_CTRL_PT;
        line_handle = actionData{1};    % Handle to line so we can get line number
        ptInfo = get(actionData{2});    % Information to restore any points sucked up by new range
        new_xdata = actionData{3};      % xdata of new ctrl pt/line to identify it with
        traceInfo = actionData{4};      % Parts of trace that was modified
        if ~isempty(line_handle)
            lineNum = get(line_handle,'UserData');
        else error('No line_handle for new ctrl pt')
        end    
        actionData = {new_xdata ptInfo traceInfo};
    case 'interpolate'
        actionID = INTERPOLATE;
        x = actionData{1};                  % New single point
        line_handle = actionData{2};        % Handle to old line
        lineInfo = get(line_handle);        % Old line info to restore line with.
        traceLineInfo = get(findobj(handles.big_axes,'Tag','trace_line' ...
                            ,'UserData',lineInfo.UserData));
        actionData = {x lineInfo traceLineInfo};
    case 'filter'
        actionID = 3;
        lineNumber = actionData;
        actionData = get(findobj(handles.big_axes,'Tag','trace_line'...
                         ,'UserData',lineNumber));
    case 'digitize'
        actionID = 4;
        
        % Undo has these variables:
        % traceInfo = actionData{1};      % Trace data that was modified
        % noTraceInfo = actionData{2};    % Line numbers without traces
    case 'move_edge'
        actionID = 5;     
        % Undo has these variables:
        % actionData = {oldLeftX oldLeftY oldRightX oldRightY}
        
    case 'line number'
        actionID = 6;
        oldLineNum = actionData;
        actionData = {oldLineNum findobj(handles.big_axes,'Tag','CtrlPt','UserData',oldLineNum)};
    case 'delete trace'
        actionID = 7;
        traceLineH = actionData;
        actionData = get(traceLineH);
        for n=1:length(actionData)
            actionData(n).ParentTag = get(actionData(n).Parent,'Tag');
        end
    case 'clear points'
        actionID = 8;
        actionData = get(actionData);
    case 'delete'
        actionID = 9;
        actionData = get(actionData);      % actionData holds handle to object deleted
        for n=1:length(actionData)
            actionData(n).ParentTag = get(actionData(n).Parent,'Tag');
        end
    case 'chg_line_num'
        actionID = 10;  % {Xdata, UserData old, UserData new} for each ctrl pt
    otherwise
        errordlg(strcat('Unknown string-',actionString),'Cannot save info for undo','modal'), return
    end
    
    % Get the undo stack data
    EventStack = get(handles.undo_button,'UserData');
    
    % Create new event object
    lineNum = getLineNum(handles);
    eventData = {actionID; actionData; lineNum};
    
    % Create new stack if it doesn't exist
    MAX_SIZ = 2^21;
    if isempty(EventStack)      % First time creating stack
        EventStack = stack('EventStack', MAX_SIZ);
    end
    EventStack = push(EventStack, eventData);   % Push the new event onto the stack.
    
    set(handles.undo_button,'UserData',EventStack,'Enable','on')

% --------------------------------------------------------------------
% Name: DELETE_LINE_PTS CLEAR_PTS
% Description:  Deletes all Ctrl pts if all of them are valid handles.
%   Updates the handles struct and the strings in the UI.
function delete_line_pts(handles)
    lineNum = getLineNum(handles);
    CtrlPts = findobj(handles.big_axes,'Tag','CtrlPt', 'UserData', lineNum);

    make_undoable('clear points',[CtrlPts(:); []],handles)  % Do not pass in trace line, don't delete

    if ~isempty(CtrlPts)
        delete(CtrlPts)
    end

    update_ctrlpt_stuff(lineNum, handles)
  
% --------------------------------------------------------------------
% Name: ZOOM_TXT_Callback
function zoom_txt_Callback(handles)
    MULT = ZOOM_VALUES;     % Call function to get constant values
    MAX_ZOOM = max(MULT);
    MIN_ZOOM = min(MULT);
    
    img_handle = get_img_h(handles);
    ZOOM = (str2double(get(handles.zoom_txt, 'String'))) / 100;   % Convert to percentage
    if isnan(ZOOM)
        %disp('Bad zoom txt')
        return
    end
    set(handles.zoom_txt,'UserData',ZOOM)   % Synchronize String and UserData
    
    % Reset the slider ranges
    reset_slider(handles,get(handles.big_axes,'Position'),...
        size(get(img_handle, 'CData')))
    
    if ZOOM <= MIN_ZOOM
        set(handles.set_zoom_left,'Enable','off')
    elseif ZOOM >= MAX_ZOOM,
        set(handles.set_zoom_right,'Enable','off')
    else
        set(handles.set_zoom_left,'Enable','on')
        set(handles.set_zoom_right,'Enable','on')
    end
    
    %% Get old center
    x_center = mean(get(handles.big_axes,'XLim'));
    y_center = mean(get(handles.big_axes,'YLim'));

    
    % Change xlabel if zoom
    set_scaled_xlabel(handles.big_axes, handles)
    
    x = min(get(handles.big_axes,'XLim'));
    y = min(get(handles.big_axes,'YLim'));
    
    center_view_on(x_center,y_center,ZOOM,[],handles)

    
% --------------------------------------------------------------------
% SET_ZOOM_TXT - increment is positive or negative
function output = set_zoom_txt(increment, handles)
    MULT = ZOOM_VALUES;     % Call function to get constant values
    MAX_ZOOM = max(MULT);
    MIN_ZOOM = min(MULT);
    
    % Get next larger or next smaller zoom value
    value = get(handles.zoom_txt,'UserData');
    if increment > 0
        value = max([value, MULT(min(find(MULT>value)))]);
    elseif increment < 0
        value = min([value, MULT(max(find(MULT<value)))]);
    end
    
    if value <= MIN_ZOOM, value = MIN_ZOOM; 
        set(handles.set_zoom_left,'Enable','off')
    elseif value >= MAX_ZOOM, value = MAX_ZOOM;
        set(handles.set_zoom_right,'Enable','off')
    else
        set(handles.set_zoom_left,'Enable','on')
        set(handles.set_zoom_right,'Enable','on')
    end
    
    % ZOOM_TXT_Callback changes zoom_txt UserData to value in String
    S = num2str(round(value*100));
    set(handles.zoom_txt,'String',S)
    zoom_txt_Callback(handles)
    
    if nargout == 1
        output = value;
    end
    
% ---- Subfunction to create global constant ----
% ZOOM_VALUES
% Must be in order from small to large, 2 decimal places only.
function vals = ZOOM_VALUES
    vals = [.03 .05 .08 .10 .17 .3 .5 1 1.5 2 4 8 16];  

% --------------------------------------------------------------------
% Name: IS_WHITE_PX
function retval = is_white_px(x, y, handles)
    % - - - - - - - - - - - - - - - - - - - -
    % Initialize some variables
    WHITE = 1;
    [Img img_handle] = get_image(handles);
    x = round(x);
    y = round(y);
    % - - - - - - - - - - - - - - - - - - - -

    % Test for WHITE pixel
    retval = (Img(y,x) == WHITE);

% --------------------------------------------------------------------
% Name: IS_NEW_PT
% Note: Pass in the y parameter for consistency with is_white_px(x,y,handles),
% even though it is not used.  Also leaves room for future changes.
function retval = is_new_pt(x, y, handles)
    % - - - - - - - - - - - - - - - - - - - -
    % Initialize some variables
    TRUE = 1; FALSE = 0;
    CtrlPts = findobj(handles.big_axes,'Tag','CtrlPt','UserData', getLineNum(handles));
    % - - - - - - - - - - - - - - - - - - - -
    
    % Locate all the X values already used.
    % Note: xdata could be empty
    xdata = get(CtrlPts(:), {'XData'});
    xdata = [xdata{:}];
    
    if isempty(xdata) | isempty(find(round(x)==round(xdata)))
        retval = TRUE;
    else
        helpdlg('Control point already exists there.')
        retval = FALSE;
    end
    

% --------------------------------------------------------------------
% Name: GET_MID_Y
% This slower version of get_mid_y grabs the image from the
% big axes.  It also checks for walking off the image.
function mid_y= get_mid_y(x, y, handles)
    % - - - - - - - - - - - - - - - - - - - -
    % Initialize some variables
    WHITE = 1;
    [Img img_handle] = get_image(handles);
    [mRows nCols] = size(Img);
    x = round(x);
    y = round(y);
    if x < 1 | x > nCols
        mid_y = y; return
    end
    top_y = y;
    bot_y = y;
    % - - - - - - - - - - - - - - - - - - - 
    while top_y>1 & top_y<mRows & Img(top_y-1,x)==WHITE, top_y=top_y-1; end
    while bot_y>1 & bot_y<mRows & Img(bot_y+1,x)==WHITE, bot_y=bot_y+1; end
    mid_y = (bot_y + top_y)/2;

% --------------------------------------------------------------------
% Name: SHOW_POINTS
function show_points(handles)
    zero_line = findobj(handles.big_axes,'Tag','zero_line');
    CtrlPts = findobj(handles.big_axes,'Tag','CtrlPt');
    on_or_off = get(handles.show_points,'Checked');     % Returns 'on' or 'off'
    
    % Toggle state
    if isequal(on_or_off, 'on'), on_or_off = 'off';
    else on_or_off = 'on';
    end
    set(handles.show_points,'Checked',on_or_off)
    set(CtrlPts(:), 'Visible', on_or_off)
    set(zero_line, 'Visible', on_or_off)
    
    restack(CtrlPts(:),'top')
    
% --------------------------------------------------------------------
% Name: SHOW_EDGE_IMG
function show_edge_img(handles)
    on_or_off = get(handles.edge_img,'Checked');     % Returns 'on' or 'off'
    
    % Toggle state
    if isequal(on_or_off, 'on'), on_or_off = 'off';
    else on_or_off = 'on';
    end
    set(handles.edge_img,'Checked',on_or_off)

% --------------------------------------------------------------------
% Name: SHOW_NORM_Callback
function show_norm_Callback(h, eventdata, handles)
    TAG = 'normal_line';
    on_or_off = get(handles.show_norm,'Checked');
    
    % Toggle the Show Normalizing Line menu item
    if h == handles.show_norm
        switch on_or_off
        case 'off', on_or_off = 'on'; set(h, 'Checked', 'on');
        otherwise  on_or_off = 'off'; set(h, 'Checked', 'off');
        end
    else
    % Turn on only one degree option (like radio button)
        set(handles.degree_123, 'Checked', 'off')
        set(h, 'Checked', 'on')
    end
    
    delete(findobj(handles.big_axes,'Tag',TAG))
    
    switch on_or_off
    case 'on'
        plotNormalLine(TAG, handles)
    end

% --------------------------------------------------------------------
% Name: SHOW_ALL_BASELINES
function show_all_baselines(handles)
    on_or_off = get(handles.show_all_baselines,'Checked');  % Returns 'on' or 'off'
   
    % Toggle state
    if isequal(on_or_off, 'on'), on_or_off = 'off';
    else on_or_off = 'on';
    end
    
    set(findobj(handles.big_axes,'Tag','zero_line'), 'Visible', on_or_off)
    % Always set current line on
    set(findobj(handles.big_axes,'Tag','zero_line','UserData', getLineNum(handles)), 'Visible', 'on')
    
    set(handles.show_all_baselines,'Checked', on_or_off)
    
% --------------------------------------------------------------------
% Name: SHOW_TRACE
function show_trace(handles)
    TraceLines = findobj(handles.big_axes,'Tag','trace_line');
    on_or_off = get(handles.show_trace,'Checked');  % Returns 'on' or 'off'
   
    % Toggle state
    if isequal(on_or_off, 'on'), on_or_off = 'off';
    else on_or_off = 'on';
    end
    
    set(TraceLines(:), 'Visible', on_or_off)
    set(handles.show_trace,'Checked', on_or_off)
    
    restack(TraceLines(:),'top')

% --------------------------------------------------------------------
% SET_CUSTOM_POINTER
function set_custom_pointer(type, handles)
    [pointerShape, pointerHotSpot] = CreatePointer(type);
    set(handles.figure1, 'Pointer', 'custom','PointerShapeCData', pointerShape, ...
                         'PointerShapeHotSpot', pointerHotSpot);
% --------------------------------------------------------------------
% Name: USER_INPUT
% Allows only mouse presses only.  Keyboard presses are passed to figure's keypress fcn.
function plot_handle = user_input(handles)
    % Move control points to top so they can be clicked on.
    restack(findobj(handles.big_axes,'Tag','CtrlPt'),'top')
    
    LEFT = 'normal'; MIDDLE = 'extend'; RIGHT = 'alt'; DOUBLE = 'open';
    ESC = 27;
    lineNumber = getLineNum(handles);
    T = waitforbuttonpress;

    while T == 1 | ~button_equal(LEFT, handles.figure1)
        % Escape key cancels
        if T == 1 & ESC==double(get(handles.figure1,'CurrentCharacter'))
            plot_handle = []; return
        % Keypress that is not esacpe passes on duties onto 
        % the key_press_fcn.
        elseif T == 1
            key_press_fcn(handles)
        % If its not a key_press, it must be a mouse-click. 
        % Check if it's a RIGHT click and center on that point like usual.
        elseif T == 0 & button_equal(RIGHT, handles.figure1)
            pt = get(handles.big_axes, 'CurrentPoint'); 
            x = round(pt(1,1));     center_y = mean(get(handles.big_axes,'YLim'));
            center_view_on(x,center_y,[], [], handles)
        end
        T = waitforbuttonpress;
    end
    
    pt = get(handles.big_axes, 'CurrentPoint'); 
    x = round(pt(1,1));     y = pt(1,2);
    
    % Test if point is valid.  If so, set new point.
    line_handle = find_CtrlPts(x, handles); % Find any control points touching
    isInterpZone = ~isempty(line_handle);
    curLoc = Where_am_I(handles);
    curobj = curObj(handles.figure1);       % Allow user to click on an existing ctrl pt.
    
    lineNum = getLineNum(handles);
    CtrlPts = findobj(handles.big_axes,'Tag','CtrlPt','UserData',lineNum);
    
    plot_handle = [];
    % Check if user clicked on a point
    if (isequal(get(curobj,'Tag'),'CtrlPt') & isequal(get(curobj,'UserData'),lineNum))
        plot_handle = curobj;    
    else
        curobj = findobj(CtrlPts,'XData',x);
        % Not object, but still over point
        if ~isempty(curobj) & ...
        abs(get(curobj,'YData')-y) < get(curobj,'MarkerSize')
            plot_handle = curobj;
        elseif curLoc==handles.big_axes & is_white_px(x,y,handles) & is_new_pt(x,y,handles) & ~isInterpZone
            plot_handle = set_new_point(x,y, lineNumber, handles, []);
        end
    end
    
% --------------------------------------------------------------------
% Name: CUROBJ
function handle = curObj(fig )
    handle = get(fig,'CurrentObject');

% --------------------------------------------------------------------
% Name: BUTTON_EQUAL
function bool = button_equal( button, fig )
    bool = isequal(get(fig,'SelectionType'), button);

% --------------------------------------------------------------------
% Name: DIGITIZE_PTS_Callback - PT to PT
function digitize_pts_Callback(h, eventdata, handles)
button_state = get(handles.digitize,'Value');
if button_state == get(handles.digitize,'Max')       % toggle button is on
    
    % SUSPEND FIGURE
    uistate = uiclearmode(handles.figure1);
    
    set_custom_pointer('one', handles);
    Plot_handle = user_input(handles);
                                
    % Double click indicates use only one control point
    if ~isempty(Plot_handle)        % If first point valid, grab a 2nd point.
        set_custom_pointer('end', handles);
        Plot_handle = [Plot_handle user_input(handles)];
        busy(handles.figure1)       % Set to busy cursor
        drawnow     % Get new point to display before tracing.
    end

    XInput = get(Plot_handle,{'XData'});    % Return data as cell 
    XInput = sort([XInput{:}]);   % Convert to array and sort
    
    if length(Plot_handle) == 2      % Check if two clicks are made
        lineNum = getLineNum(handles);
        Points_h = getPointsBetween(min(XInput), max(XInput), lineNum, handles);
        
        busy(handles.figure1)
        digitize(Points_h, handles, handles.digitize)
        idle(handles.figure1)
    else   
        %disp([num2str(length(XInput)) ' inputs found.'])
    end
    
    % Set various buttons on and off
    update_smooth_button(handles)
    set(handles.digitize, 'Value', get(handles.digitize,'Min'))
    
    % RESTORE FIGURE
    uirestore(uistate)
else
    set(handles.digitize, 'String', 'Pt 2 Pt', 'BackgroundColor', [0.8 0.8 0.9], 'ForegroundColor','r')
end

% ----- Subfunction returns points between a range
% GETPOINTSBETWEEN
function Points_h = getPointsBetween(x1, x2, lineNum, handles)
    LinePts = findobj(handles.big_axes,'Tag','CtrlPt', 'UserData', lineNum);
    xdata = [];
    Points_h = zeros(size(LinePts));
    % Loop through points, testing if any are in range
    for n=1:length(LinePts), xdata = get(LinePts(n),'XData'); if any(xdata >= x1) & any(xdata <= x2), Points_h(n) = LinePts(n); end, end
    Points_h = Points_h(Points_h>0);    % Return only valid handles.

% --------------------------------------------------------------------
% Name: DIGITIZE_LINE_Callback
function digitize_line_Callback(h, eventdata, handles, varargin)
if get(h,'Value') == get(h,'Max')
    lineNum = getLineNum(handles);
    % Test if trace line exists and give a warning if it does
    if ~isempty(findobj(handles.big_axes,'Tag','trace_line','UserData',lineNum))
        ButtonName=questdlg( ... 
                   strvcat('Caution:  Addition of control points shifts the base line,',...
                           'possibly causing previously digitized data to change.',...
                           'Consider digitizing with "Pt 2 Pt".'), ...
                       'Overwrite Entire Line?', ...
                       'Continue','Abort','Continue');
    else
        ButtonName = 'Continue';
    end
                   
    if isequal(ButtonName, 'Continue')
        digitize_line(h, lineNum, handles)
    end
    set(h,'Value',get(h,'Min'))     % Turn button off
else
    set(handles.togglebutton33, 'String', 'Line', 'BackgroundColor', [0.8 0.8 0.9], 'ForegroundColor','r')
end

% ----------------------------
% Name: DIGITIZE_LINE
function digitize_line(digit_button_h, lineNum, handles)
    % Get line points
    Points_h = findobj(handles.big_axes,'Tag','CtrlPt','UserData',lineNum);
    digitize(Points_h, handles, digit_button_h)

% --------------------------------------------------------------------
% Name: DIGITIZE_ALL_Callback
function digitize_all_Callback(h, eventdata, handles, varargin)
% loop through selected items
%   load variables
%   load image
%   load lines
%   loop through lines
%       trace(array of control points)
%       save all the lines back into the same file
if get(h,'Value') == get(h,'Max')
    List = get(handles.queue_listbox,'String');
    % Check if anythining queue
    if isempty(List)
        set(h,'Value',get(h,'Min'))     % Turn button off
        warndlg('Nothing in the queue.'), return
    end
    Idx = get(handles.queue_listbox,'Value');
    List = List(Idx,:);
    if isempty(List)
        warndlg('Nothing selected.')
    end
    
    % Confirm with user
    answer = questdlg('This will erase current work and load from queue. Procede?','Digitize queue?','Yes','No','No');
    if isequal(answer,'No'), set(h,'Value',get(h,'Min')), return, end
    
    Idxs = get(handles.queue_listbox,'Value');
    wb = waitbar2(0,0,'Digitizing...');   % Create waitbar
    
    % Fix in windows position of waitbar that doesn't work on windows.
    set(wb,'Position',[200 300 400 80])
    movegui(wb,[],'center')
    
    M = size(List,1);
    for m = 1:M
        saveFile = deblank(List(m,:));                % Remove trailing blanks also
        if isdir(saveFile) | any(saveFile(:) == '*')
            continue                                  % Skip non-files
        end
        
        if isempty(fileparts(saveFile))
            Dir = fileparts(get(handles.file_expr,'String'));
            saveFile = fullfile(Dir, saveFile);
        end
        errors = load_pts_file(saveFile, handles, 1); % 1 enables auto image loading
        
        if ~isempty(errors)
            warning(['Error in ' saveFile ': ' errors]), continue
        end
        
        if isequal('ON','') % Autofilter
            disp('AUTOFILTER ON')
            clean_img(handles)
            dilate_img(handles)
        end
        
        % When digitizing all again, delete all data first
        delete(findobj(handles.figure1,'Tag','trace_line'))
        
        % Loop through each line
        CtrlPts = findobj(handles.big_axes,'Tag','CtrlPt');
        LineNums = get(CtrlPts(:),{'UserData'});   LineNums = unique([LineNums{:}]);
        N = length(LineNums);
        
        % Count gap data
        PX_PER_MINUTE = zeros(N,1);
        MINUTES = zeros(N,1);
        PIXELS = zeros(N,1);
        img_handle = get_img_h(handles);
        for n = 1:N
            setLineNum( LineNums(n), handles )
            line_number_txt_Callback(h, eventdata, handles)

	    % First try digitizing
            try, 
                digitize_line(h, LineNums(n), handles)       % DIGITIZE_CALL
            catch, warning(['Digitize error ' lasterr])
            end
	    % Next try counting gaps
            try,   
                AllPts = findobj(handles.big_axes,'Tag','CtrlPt','UserData',LineNums(n)); % All pts on line
                AllPts = Sort_the_point_handles(AllPts);
                [px_per_minute, gap_count, px_total] = count_gaps(...
                                                       img_handle,AllPts,handles.g_width,'gap label',getNumOrValue(handles.px_per_inch_txt));
                PX_PER_MINUTE(n) = px_per_minute;
                MINUTES(n) = gap_count;
                PIXELS(n) = px_total;   

            catch, warning(['Gap count error ' lasterr])
            end
        
	    % Update the waitbar
            if ishandle(wb) & get(h,'Value')==get(h,'Max')
                waitbar2((n-1)/N,(m-1)/M+(n-1)/N*m/M,wb)
            else
                set(h,'Value',get(h,'Min'))
                if ishandle(wb), delete(wb), end
                return
            end
        end
        
	    % Automatically update the PX_PER_MINUTE value in main window
        PX_PER_MINUTE = removeGapOutliers(PX_PER_MINUTE, MINUTES, PIXELS);
        newvalue = nanmean(PX_PER_MINUTE(:));

	if ~isfinite(newvalue)
        % Give a special warning for bad values
        warndlg(['Pixels Per Minute set to ' num2str(newvalue)],'Minute Mark Gaps')
	else
        % Give a simple message at the command prompt
	    disp(['Pixels Per Minute calculated to be ' num2str(newvalue)])
	end
        setNum(handles.px_per_minute_txt, newvalue);

        [path name ext] = fileparts(saveFile);
        outFile = fullfile(path, ['tr-' name ext]);
        fileName = get(handles.filename_txt,'UserData');
        Operations = get(findobj(handles.big_axes,'Tag','Img'),'UserData');
        LineInfo = get(findobj(handles.figure1,'type','line'));
        for n=1:length(LineInfo)
            LineInfo(n).ParentTag = get(LineInfo(n).Parent,'Tag');
        end
        disp(['Saving trace data to ' outFile])
        try
            save(outFile,'fileName','LineInfo','Operations','-mat')
        catch
            warning(lasterr)
        end
        
        gapFile = fullfile(path, ['gaps-' name '.mat']);
        disp(['Saving minute mark variables to ' gapFile])
        try
            save(gapFile,'PX_PER_MINUTE','MINUTES','PIXELS','-mat')
        catch
            warning(lasterr)
        end
        
        if ishandle(wb)
            waitbar2(m/M,m/M,wb)
        else
            set(h,'Value',get(h,'Min')), 
            return
        end
    end 
    if ishandle(wb)
        close(wb)
    end
    
    line_number_txt_Callback(h, eventdata, handles)

    set(h,'Value',get(h,'Min'))     % Turn button off
    clear all
else
    % Restore button
    set(handles.digitize_all, 'String', 'Digitize Files', 'BackgroundColor', [0.8 0.8 0.9], 'ForegroundColor','r')
end

% --------------------------------------------------------------------
% GETSPECS - returns info needed to call trace_it
function [MAX_PERIOD] = getSpecs(handles)
    MAX_PERIOD = str2double(get(handles.max_period,'String'));
% -----------------
function bool = notOK(n)
    bool = isnan(n) | n < 1 | n > 999;

% --------------------------------------------------------------------
% Name: _DIGITIZE_
% Takes in control points and uses those to digitize.
function digitize(Points_h, handles, button)
    [TRACE_HIGHLITE, PT_HIGHLITE, TRACE_BACKGND, PT_BACKGND] = COLORS;


    lineNumber = get(Points_h,{'UserData'});    % Returns a cell
    lineNumber = unique([lineNumber{:}]);       % Convert cell to array and check homogeneity
    if length(lineNumber) > 1  % ::Error check::
        warning('Error: did not find a singular line number. Digitizing anyway using first line number.')
        lineNumber = lineNumber(1)
    end
    
    traceLine_h = findobj(handles.big_axes,'Tag','trace_line','UserData',lineNumber);
    if length(traceLine_h) > 1      % ::Error Check::
        errordlg('Error: multiple lines with same line number found. Please clear trace line.'), return
    end

    Points_h = Sort_the_point_handles(Points_h);
    
    % Save data so we can undo later, 2 cases: trace exists or does not exist
    if ~isempty(traceLine_h)
        traceInfo = get(traceLine_h);
        noTraceData = [];
    else
        traceInfo = [];
        noTraceData = lineNumber;
    end
    
    % trace 
    if length(Points_h) > 0
      % Indicate not done
      uistate = uiclearmode(handles.figure1);
      oldBG = get(button,'BackgroundColor');
      oldFG = get(button,'ForegroundColor');
      oldString = get(button, 'String');
      set(button, 'BackgroundColor', 'r', 'ForegroundColor','w',  'String', 'Stop')
      
      [MAX_PERIOD] = getSpecs(handles);
      if notOK(MAX_PERIOD)
          warning('MAX_PERIOD is not a number. Using Inf.')
          MAX_PERIOD = Inf;
      end

      % Convert MAX_PERIOD from seconds into pixels
      px_per_sec = getNumOrValue(handles.px_per_minute_txt) / 60;
      MAX_PERIOD = MAX_PERIOD * px_per_sec;

      try
        img_handle = get_img_h(handles);
        if isempty(img_handle) & ~ishandle(img_handle)  %::Error Check::
            error('Image object (Tag=''Img'') not found!')
        end
        xPts = get(Points_h(:),{'XData'});  % Cell arrays
        yPts = get(Points_h(:),{'YData'});
        ZERO_P = getP(lineNumber, handles);
        dpi = getNumOrValue(handles.px_per_inch_txt);

        turn_buttons('off', handles, button)         % Turn off user interaction
        [xxx yyy] = trace_it(img_handle,xPts,yPts,handles.g_width, button ...
                   , MAX_PERIOD, ZERO_P, dpi);
        turn_buttons('on', handles, [])             % Resume interaction

      catch
          error(['Trace error while digitizing line ' num2str(lineNumber) ':' lasterr])
      end

      % Case 1: create new trace
      if isempty(traceLine_h)
          STYLE = '*-';
          h = plot(xxx, yyy,STYLE,'Tag','trace_line','MarkerSize',6,'Color',TRACE_HIGHLITE...
                           ,'Parent',handles.big_axes,'UserData',lineNumber);

          %% Obsolete Code Fragment: lineNumber can be less than 1.
%           handles.trace_line{lineNumber} = h;
%           guidata(handles.figure1, handles)
          
      % Case 2: add/replace with new data and sort
      else
          xTemp = get(traceLine_h,'XData'); 
          yTemp = get(traceLine_h,'YData');
          if length(xTemp) ~= length(yTemp)          %::Error check::
              errordlg('Mismatched old trace data.','Trace error','modal'), return
          end
          index = find(xTemp >= min(xxx) & xTemp <= max(xxx));
          xTemp(index) = [];    % Remove all points in range of digitizing
          yTemp(index) = [];    % in the event that some regions don't get traced
          traceInfo.XData = traceInfo.XData(index);
          traceInfo.YData = traceInfo.YData(index);

          [xdata,ydata] = mergeData(xxx,yyy, xTemp,yTemp);  % Merge in new digitized data
          set(traceLine_h,'XData', xdata, 'YData', ydata)
      end% If number of ctrl pts > 0
      
      % Indicate done
      uirestore(uistate)
      set(button, 'BackgroundColor', oldBG, 'ForegroundColor', oldFG, 'String', oldString)
      
      make_undoable('digitize',{traceInfo, noTraceData}, handles)
  end


% --------------------------------------------------------------------
% SORT_THE_POINT_HANDLES
% Make sure the control point handles are in order from start to end.
% Uses insertion sort algorithm cuz handles are likely to be near sorted.
function Points_h = Sort_the_point_handles(Points_h)
    j = [];
    n = length(Points_h);
    for i = n-1:-1:1
        temp_h = Points_h(i);
        j = i+1;
        while j <= n & isless(Points_h(j),temp_h)
            Points_h(j-1) = Points_h(j);
            j = j+1;
        end
        Points_h(j-1) = temp_h;
    end
     
    % ISLESS - used to compare control point handles in sorting
    function retval = isless(h1, h2)
        retval = min(get(h1,'XData')) < min(get(h2,'XData'));
    
% --------------------------------------------------------------------
% Name: FILTER_Callback
function filter_Callback(h, eventdata, handles)
    lineNum = getLineNum(handles);
    make_undoable('filter',lineNum,handles)
    
    trace_line = findobj(handles.big_axes,'Tag','trace_line','UserData',lineNum);
    if length(trace_line) ~= 1
        errordlg('Bad  number of trace_lines.','Filter error','modal'), return    
    end
    xdata = get(trace_line, 'XData');
    ydata = get(trace_line, 'YData');
    xdata = xdata(~isnan(ydata));       % Remove NaNs
    ydata = ydata(~isnan(ydata));

    % 3-point filter
    n = 2;                  % Initialize
    Idx = [];               % Preallocate memory
    B = [0.25 0.5 0.25];    % Filter kernal with a 3-point filter
    
    busy(handles.figure1)
    
    D = find(diff(xdata) ~= 1);
    while n <= length(xdata)
        Idx = n-1;   % Reset
        n = min([length(xdata), min(D(D>Idx))]);
        Idx = Idx:n;
        n = n+1;
            
        temp = ydata(Idx);
        temp = filter2(B,[temp(1) temp temp(end)]);     % Pad the ends
        ydata(Idx) = temp(2:end-1);                     % Remove padding
        n = n+1;
    end

    idle(handles.figure1)

    set(trace_line,'YData',ydata, 'XData', xdata)   % need to set xdata if we remove some NaNs
    
% --------------------------------------------------------------------
% Name: TURN_BUTTONS
% Description: Disable or enable buttons
% Varargin holds buttons to leave enabled
function turn_buttons( on_or_off, handles, varargin )
    set(findobj(handles.figure1,'Type', 'uicontrol', 'Style', 'pushbutton'),'Enable', on_or_off)
    set(findobj(handles.figure1,'Type', 'uicontrol', 'Style', 'togglebutton'),'Enable', on_or_off)
    set(handles.zoom_txt,'Enable',on_or_off)
    set(handles.line_number_txt,'Enable',on_or_off)
    set(handles.max_period,'Enable',on_or_off)
    set(handles.queue_listbox,'Enable',on_or_off)
    switch on_or_off
    case 'off'
        set([varargin{:}], 'Enable', 'on');    % Do opposite
    otherwise
        set([varargin{:}], 'Enable', 'off');
    end
    drawnow
function toggleCheckbox(h)
    switch get(h,'Value')
    case get(h,'Max')
        set(h,'Value',get(h,'Min'))
    otherwise
        set(h,'Value',get(h,'Max'))
    end

% --------------------------------------------------------------------
% Name: UPDATE_SMOOTH_BUTTON
function update_smooth_button(handles)
    lineNum = getLineNum(handles);
    trace_line = findobj(handles.big_axes,'Tag','trace_line','UserData',lineNum);
    if ~isempty(trace_line) & ishandle(trace_line),
        set(handles.filter, 'Enable', 'on')
    else
        set(handles.filter, 'Enable', 'off')
    end
    
% --------------------------------------------------------------------
% Name:   IMAGE_MENU_Callback
% Description -     Decides which submenus are on or off.
function varargout = image_menu_Callback(h, eventdata, handles, varargin)
    if ~isempty(findobj(handles.big_axes,'Tag','Img'))
        on_or_off = 'on';
    else
        on_or_off = 'off'
    end
    set(handles.rotate_button,'Enable', on_or_off)
    set(handles.rotate_button_counter, 'Enable', on_or_off)
    set(handles.erode_button,'Enable',  on_or_off)
    set(handles.dilate_button,'Enable', on_or_off)
    set(handles.clean_button,'Enable',  on_or_off)

% --------------------------------------------------------------------
% Name:   FILE_MENU_Callback
% Description -     Decides which submenus are on or off.
function varargout = file_menu_Callback(h, eventdata, handles, varargin)
    % Turn submenus on or off
    TraceLines = findobj(handles.big_axes,'Tag','trace_line');
    if isempty(TraceLines)
         set(handles.save_submenu,'Enable','off')
    else set(handles.save_submenu,'Enable','on')
    end
    
    img_handle = get_img_h(handles);
    if isempty(get(img_handle,'UserData'))
        set(handles.save_img_submenu,'Enable','off')
    else 
        set(handles.save_img_submenu,'Enable','on')
    end

% --------------------------------------------------------------------
% Name:   EDIT_MENU_Callback
% Description -     Decides which submenus are on or off.
function varargout = edit_menu_Callback(h, eventdata, handles, varargin)
    if isempty(get(handles.undo_button,'UserData'))
        set(handles.undo_button,'Enable','off')
    else
        set(handles.undo_button,'Enable','on')
    end

    if isTraceOnCurrentLine(getLineNum(handles), handles)
         set(handles.delete_trace, 'Enable', 'on')
    else  set(handles.delete_trace, 'Enable', 'off')
    end
    
    if isempty(findobj(handles.big_axes,'Tag','gap label'))
        set(handles.del_gap_labels,'Enable','off')
    else set(handles.del_gap_labels,'Enable','on')
    end


% --------------------------------------------------------------------
% Name:   SAVE_SUBMENU_CALLBACK
function varargout = save_submenu_Callback(h, eventdata, handles, varargin)

% Grab the pixel per minute value
px_per_minute = getNumOrValue(handles.px_per_minute_txt);
Saved_traces = findobj(handles.big_axes,'Tag','trace_line');
leftEdge = findobj(handles.big_axes,'Tag','left_edge');
rightEdge = findobj(handles.big_axes,'Tag','right_edge');
minutesPerLine = getNumOrValue(handles.row_time);

if isnan(px_per_minute) & isnan(minutesPerLine)
    errordlg('Please set the length per line and pixels per minute.','Error', 'modal'), return
elseif isnan(px_per_minute)
    errordlg('Please set the pixels per minute.','Error','modal'), return
elseif isnan(minutesPerLine)
    errordlg('Please set the length per line.','Error','modal'), return
elseif length(leftEdge) ~= 1 | length(rightEdge) ~= 1
    errordlg('Please set the edge lines.','Error','modal'), return
elseif isempty(Saved_traces) | ~all(ishandle(Saved_traces))
    errordlg('No saved traces to save.','Error','modal'), return
else
% Error checks done, so we can begin saving.
    try
        [final_data, LineStartEnd] = splice_trace_lines(Saved_traces, leftEdge, rightEdge, handles);
    catch
        warning(lasterr)
        return
    end
        
    % Duplicate: same as px_per_minute
    pixPerMinute = getNumOrValue(handles.px_per_minute_txt);% Dots per minute
    estimatedDuration = length(final_data) ./ pixPerMinute;
    
    dpi = getNumOrValue(handles.px_per_inch_txt);
    dpmm = dpi/ 25.4;    % Dots per millimeter
    
    % Get info from the user
    % Preview needs to take in a struct with the fields:
    Info = struct('final_data', final_data ...
                  ,'LineStartEnd', LineStartEnd ...
                  ,'estimatedDuration', estimatedDuration ...
                  ,'pixPerMinute', pixPerMinute ...
                  ,'dpmm', dpmm ...
                  ,'retval_handle', handles.save_submenu ...    % Save it in object's UserData);
                  ,'image_filename', get(handles.filename_txt,'UserData') ... % Need for start time
                  );
    fig_handle = Preview(Info);
    
    % Save the Preview figure so we can delete it if main fig closes
    set(handles.save_submenu,'UserData', fig_handle)
end

% --------------------------------------------------------------------
% Name: SPLICE_TRACE_LINES _SPLINE_
function [final_data, LineStartEnd] = splice_trace_lines(Saved_traces, leftEdge, rightEdge, handles)
    % Move waitbar to middle of main figure window
    wb = waitbar(0, 'Connecting lines together...');  % Create waitbar
    
    % Fix the position error that shows up in windows
    Pos = get(handles.figure1,'Position');
    Pos = [((Pos(1)+Pos(3))/2-150) ((Pos(2)+Pos(4))/2-25) 300 50];
    set(wb,'Position',Pos)
    
    num_lines = length(Saved_traces);
    num_gaps = num_lines - 1;               % Number of gaps between traces
    
    % Check that all contrl point numbers have a corresponding trace
    CtrlPts = findobj(handles.big_axes,'Tag','CtrlPt');
    CPlineNums = get(CtrlPts,{'UserData'});
    CPlineNums = [CPlineNums{:}];
    CPlineNums = unique(CPlineNums);
    for I=1:length(CPlineNums)
        if isempty(findobj(handles.big_axes,'Tag','trace_line','UserData',CPlineNums(I)))
            error('Ctrl Point number without a trace found, cannot connect trace lines together.')
            delete(wb)
        end
    end
    
    %%
    %% Choose if we want to go by mean of Y or by the line numbers.
    %%
    RESORT_TRACE_ORDER = 1;     % 0 means we trust the user's line number.
    if RESORT_TRACE_ORDER
        Saved_traces = sort_traces(Saved_traces);
    else
        % Sort by line number
        temp = Saved_traces;
        Saved_traces = zeros(size(temp));
        for I=1:length(temp)
            n = get(temp(I),'UserData');
            Saved_traces(n) = temp(I);
            if any(Saved_traces==0)
                error('Missing trace number')
            end
        end
    end
    
    % Give a warning if traces are not numbered in increasing order
    LineNums = get(Saved_traces,{'UserData'});
    LineNums = [LineNums{:}];
    if any(diff(LineNums) ~= 1 )
        warndlg('Caution: Line numbers not in order.', 'Line Numbers', 'modal')
        LineNums
    end
    
    if ishandle(wb)
        waitbar(.1,wb)                              % Update waitbar
    else
        error('exit function')
        return  % exit function
    end

    % Left Edge info
    xtemp = get(leftEdge,'XData');      
    ytemp = get(leftEdge,'Ydata');
    idx = min(find(xtemp==min(xtemp)));
    leftMostPt = [xtemp(idx) ytemp(idx)];   % The reference pt farthest to the left x,
    leftSlope = get(leftEdge,'UserData');   % Bottom of line
    
    % Right Edge info
    xtemp = get(rightEdge,'XData');     
    ytemp = get(rightEdge,'Ydata');
    idx = max(find(xtemp==max(xtemp)));
    riteMostPt = [xtemp(idx) ytemp(idx)];   % The reference pt farthest to the right in x,y
    riteSlope = get(rightEdge,'UserData');
    
    MAX_LINE_LEN = riteMostPt(1) - leftMostPt(1) -1;       % (right x-1) - (left x+1) +1

    if ishandle(wb)
        waitbar(.15,wb)                           % Update waitbar
    else
        error('exit function')
        return  % exit function
    end
    
    % Assemble all the lines together
    ydata = [];   trace_handle = [];   final_data = [];    % Preallocate memory
    X = []; Y = [];
    SPLICE_GAP = 50;                                        % Maximum gap to spline over
    riteX = []; riteGapLen = [];    riteGap = [];
    leftX = []; leftGapLen = [];    leftGap = [];
    LineStartEnd = [];
    for n=1:num_lines        
        trace_handle = Saved_traces(n);
        ydata = get(trace_handle,'YData');
        xdata = get(trace_handle,'XData');
        
        ydata = ydata(isfinite(ydata));
        xdata = xdata(isfinite(ydata));
        
        ydata = ydata(~isnan(xdata));       % Remove NaNs    
        xdata = xdata(~isnan(xdata));
        % Sort pts in order left to right data and fill in gaps with NaNs
        [xdata, ydata] = sortAndFillGaps(xdata, ydata);
        
        if length(unique(xdata)) ~= length(xdata)
            % Grabbing unique values only
            [xdata temp] = unique(xdata);
            ydata = ydata(temp);
        end 
        if isempty(ydata),
            warning('Trace data is empty.'), 
            error('exit function')
            return
        end
        
        left_y_ref = ydata(min(find(isfinite(ydata))));
        rite_y_ref = ydata(max(find(isfinite(ydata))));
        
        prompt = 'no thanks';
        ydata = normalize(xdata, ydata, prompt, getPolyfitDegree(handles));
        ydata = -ydata;     % Flip to account for image coordinates upside down

        % 3on6off, NAN HIDDEN LINES HERE
        if isequal(get(trace_handle,'Visible'),'off')
            disp(['NaNing hidden line ' num2str(get(trace_handle,'UserData'))])
            ydata(:) = NaN;
        end
        
        % Create gap arrays, splining if small
        if n==1
            leftGap = [];
            leftGapLen = 0;
        else
            leftX = min(get(trace_handle,'XData'));
            leftGapLen = round(leftX-leftMostPt(1)-(left_y_ref-leftMostPt(2))*leftSlope - 1);
            leftGap = zeros(leftGapLen,1);  leftGap(:) = NaN;
        end
        if n==num_lines
            riteGap = [];
            riteGapLen = 0;
        else
            riteX = max(get(trace_handle,'XData'));
            riteGapLen = round(riteMostPt(1) - riteX + (rite_y_ref-riteMostPt(2))*riteSlope - 1);
            riteGap = zeros(riteGapLen,1);  riteGap(:) = NaN;
        end
        
        % Spline lines together
        lineStart = length(final_data)+1;           % Save this index
        x1 = length(final_data)+1;
        while x1 > 1 & isnan(final_data(x1-1))
            x1 = x1-1;
        end
        x2 = length(final_data)+leftGapLen;

        final_data = [final_data(:); leftGap(:); ydata(:); riteGap(:)];
        
        if x2-x1 <= SPLICE_GAP   % x2-x1+1 (# of pts) <= SPLICE_GAP
            X = find(~isnan(final_data(max(1,x1-200):min(x2+200,end))))+max(1,x1-200)-1;
            Y = final_data(X);
            if x2>x1, 
                try
                    if max(X) < x2
                        error('Not enough points in range to spline.')
                    end
                    leftGap = spline(X,Y,x1:x2);
                    final_data(x1:x2) = leftGap;
                    
                catch
                    disp(strvcat(lasterr,['Aborted splining of line ' num2str(n)]))
                end
            end
        end
        
        % Get data to know where lines start and end to display in the Export window
        if n==1 | n==num_lines
            LineStartEnd(n,:) = [lineStart length(final_data)];  
        else
            LineStartEnd(n,:) = [lineStart length(final_data)];
            thisLineLen = length(final_data) - lineStart + 1;
        end
        
        if ishandle(wb)
            waitbar(.2+(n/num_lines)*.85,wb)                          % Update waitbar
        else
            error('exit function')
            return  % exit function
        end
    end%End Splining FOR Loop

    waitbar(1,wb)                          % Update waitbar
    close(wb)
    
% -----------------------------------------------------------
% Name: EXPORT_SAC
function export_sac(ret_val, eventdata, handles)
    if ~isempty(ret_val) & ~isequal(ret_val{1},'cancel_button')
        button = ret_val{1};
        SACdata = ret_val{2};
        
        % Add more information
        SACdata.SeisDig.min_per_line = getNumOrValue(handles.row_time);
        SACdata.SeisDig.physical_length = getNumOrValue(handles.row_length);
        SACdata.SeisDig.dpi = getNumOrValue(handles.px_per_inch_txt);
        SACdata.SeisDig.dpmm = SACdata.SeisDig.dpi/ 25.4;    % Dots per millimeter
        SACdata.SeisDig.image_filename = get(handles.filename_txt,'UserData');
        
        SeisData = ret_val{3};
    else
        return  % User canceled
    end
    
    if any(~isfinite(SeisData) & ~isnan(SeisData))
        warning('Infinite data still in data')
    end
    
    % Save as a Matlab mat file %
    if isequal(button,'mat_button') | isequal(button,'both_button')
        image_name = get(handles.filename_txt,'UserData');    % UserDat holds full path
        [path, name, ext] = fileparts(image_name);
        [filename, pathname] = myputfile(strcat(name, '.mat'), 'Save as .mat file...', handles, 'sacmat');
        if ~isempty(filename) & ~isequal(filename, 0)
            save([pathname filename], 'SACdata', 'SeisData')
            disp(['File ' pathname filename ' has been saved.'])
        else
            % User canceled
        end
    end
    %
    % Save as a SAC file %
    if isequal(button,'sac_button') | isequal(button,'both_button')
        % Replace NaNs and empty with SAC's no data flag
        EMPTY = -12345;  % No data flag
        SeisData(~isfinite(SeisData)) = EMPTY;      % It will be written out in float32
        SACdata = sacMissingData(SACdata, EMPTY);        
        
        % SAC MANUAL DEFINED CONSTANTS:
        HEADER_VERSION = 6;                 % Version number of header
        UNDEF = -12345;                
        K_UNDEF = '-12345  ';               % Undefined value for strings.
        FWORDS = 70;                        % Number of float words.
        IWORDS = 40;                        % Number of ints, logical, and 
                                            %  enumerated types. 'I' strangely
                                            %  stands for 'enumerated' according
                                            %  to the manual. They're all 
                                            %  int32s when fwriting.
        IFTYPE = 1;                         % enumerated value for ITIME
        LEVEN = 1;                          % True: data is evenly spaced.

        % Create arrays to hold values
        SAC_head_floats = single(zeros(FWORDS,1)); % Float32 area of header.
        SAC_head_ints = int32(zeros(IWORDS,1)); % Int32 area of header.
        SAC_head_strings=char(zeros(8,24));     % 23 strings, all are 8 chars each
                                                % Except KEVNM that is 16 chars
                                                % Note that we'll write this out all as one column later
        % Fill arrays with undefined flags                                    
        SAC_head_floats(:) = UNDEF;             % default undefined value
        SAC_head_ints(:) = UNDEF;
        for n=1:length(SAC_head_strings)        % Fill with undefine.
            SAC_head_strings(:,n) = K_UNDEF;  
        end
        SAC_head_strings(:,3) = ' ';  % KEVNM  fills up 8 extra spaces.
        
        % Insert numbers according the SAC binary data format.
        % Note that matlab indexes starting at 1. Typical header tables index 
        % starting at zero, so subtract 1 from matlab indices.
        SAC_head_floats(1) = SACdata.times.delta;     % DELTA - increment between evently spaced samples.

        % These values are removed from SACdata header
        SAC_head_floats(2) = nanmin(SeisData(SeisData ~= EMPTY)); %SACdata.depmin;    % DEPMIN of dependent var.
        SAC_head_floats(3) = nanmin(SeisData(SeisData ~= EMPTY)); %SACdata.depmax;    % Max of dependent var.
        SAC_head_floats(4) = 1;       %SACdata.scale;     % Scale factor
        SAC_head_floats(57) = nanmean(SeisData(SeisData ~= EMPTY));       %SACdata.depmen;   % Avg of dependent var.

        
        SAC_head_floats(6) = SACdata.times.b;   % Begnning value of independent variable.
        SAC_head_floats(7) = SACdata.times.e;   % Ending value of independent variable.
        SAC_head_floats(8) = SACdata.times.o;   % Event Origin
        SAC_head_floats(9) = SACdata.times.a;   % First Arrival
        
        SAC_head_floats(21) = SACdata.times.f;  % Fini (Event End time)
        
        SAC_head_floats(64) = SACdata.SeisDig.min_per_line;       % UNUSED
        SAC_head_floats(65) = SACdata.SeisDig.px_per_min;    % UNUSED
        SAC_head_floats(66) = SACdata.SeisDig.physical_length;        % UNUSED
        SAC_head_floats(67) = SACdata.SeisDig.dpi;              % UNUSED
        SAC_head_floats(68) = SACdata.SeisDig.dpmm;             % UNUSED
        SAC_head_floats(69) = SACdata.SeisDig.num_pixels;       % UNUSED
        SAC_head_floats(70) = SACdata.SeisDig.samples_per_sec;  % UNUSED
        
        SAC_head_ints(71-FWORDS) = SACdata.event.nzyear;
        SAC_head_ints(72-FWORDS) = SACdata.event.nzjday;
        SAC_head_ints(73-FWORDS) = SACdata.event.nzhour;
        SAC_head_ints(74-FWORDS) = SACdata.event.nzmin;
        SAC_head_ints(75-FWORDS) = SACdata.event.nzsec;
        SAC_head_ints(76-FWORDS) = SACdata.event.nzmsec;
        SAC_head_ints(77-FWORDS) = HEADER_VERSION;
        
%          assert(length(SeisData) == SACdata.trcLen);
        if length(SeisData) ~= SACdata.trcLen
            error('Mismatched SACdata.trcLen and length of data.')
        end
        SAC_head_ints(80-FWORDS) = length(SeisData);    % same as SACdata.trcLen
        SAC_head_ints(86-FWORDS) = IFTYPE;
        SAC_head_ints(106-FWORDS) = LEVEN;
        
        SAC_head_strings(:,1) = fit_to_n_chars(SACdata.station.kstnm, 8);    % Must be 8 chars
        eventName = fit_to_n_chars(SACdata.event.kevnm, 16);               % Must be 16 chars
        SAC_head_strings(:,2) = eventName(1:8);
        SAC_head_strings(:,3) = eventName(9:16);                    
        SAC_head_strings(:,21) = fit_to_n_chars(SACdata.station.kcmpnm, 8);  % Must be 8 chars
        SAC_head_strings(:,22) = K_UNDEF;                               % Must be 8 chars
    
        % Write out all the arrays we created in the proper format.
        % This is really long because we have to write out in
        % 3 different formats: floats, ints, and chars.
        STRING = get(handles.filename_txt,'UserData');    % UserDat holds full path
        [pathstr, name, ext, versn] = fileparts(STRING);
        [filename, pathname] = myputfile(strcat(name,'.SAC'),'Save to SAC...', handles, 'SAC');
        if 0 == filename
            return
        end
        
        [fid, msg] = fopen(strcat(pathname,filename),'w');    % Open for overwriting file
        if fid < 0
            errordlg(msg,'fopen error','modal'), return
        end
        PRECISION = 'float32';
        float_count = fwrite(fid, SAC_head_floats(:),PRECISION);
        err = fclose(fid);
        if err
            errordlg('fclose error','error','modal')
        end
        
        [fid, msg] = fopen(strcat(pathname,filename),'a');    % Open for appending file
        PRECISION = 'int32';
        int_count = fwrite(fid, SAC_head_ints(:),PRECISION);
        err = fclose(fid);
        
        [fid, msg] = fopen(strcat(pathname,filename),'a');    % Open for appending file
        PRECISION = 'uchar';
        char_count = fwrite(fid, SAC_head_strings(:),PRECISION);
        err = fclose(fid);
        
        [fid, msg] = fopen(strcat(pathname,filename),'a');    % Open for appending file
        PRECISION = 'float32';
        data_count = fwrite(fid, SeisData(:),PRECISION);
        err = fclose(fid);
        disp(['File ' strcat(pathname,filename) ' has been saved.'])
    end    

% -----------------------------------------
% Sub-function: SACMISSINGDATA - replace empty fields with
% the missing data flag.
function SACdata = sacMissingData(SACdata, EMPTY)
    EMPSTR = num2str(EMPTY);
    FN = fieldnames(SACdata);
    for I=1:length(FN)
        obj = eval(['SACdata.' FN{I}]);
        if isstruct(obj)
            Base = ['SACdata.' FN{I}];
            FN2 = fieldnames(eval(Base));
            for J=1:length(FN2)
                obj = eval([Base '.' FN2{J}]);
                if ischar(obj) & isempty(obj)
                    eval([Base '.' FN2{J} '=''' EMPSTR ''';'])
                elseif isnumeric(obj) & isempty(obj)
                    eval([Base '.' FN2{J} '=' EMPSTR ';'])
                elseif ~ischar(obj) & ~isnumeric(obj)
                    warning('Unknown object type in SACdata, export function.')
                end
            end
        elseif ischar(obj) & isempty(obj)
            eval(['SACdata.' FN{I} '=''' EMPSTR ''';'])
        elseif isnumeric(obj) & isempty(obj)
            eval(['SACdata.' FN{I} '=' EMPSTR ';'])
        elseif ~ischar(obj) & ~isnumeric(obj)
            warning('Unknown object type in SACdata, SAC export function')
        end
    end
% -----------------------------------------
% Sub-function: SORTANDFILLGAPS
function [xdata, ydata] = sortAndFillGaps(xdata, ydata)
    Data = sortrows([xdata(:) ydata(:)]);
    xdata = Data(:,1)';
    ydata = Data(:,2)';
    Jump = diff(xdata);
    Idx = find(Jump > 1);
    Gap = Jump(Idx);
    New = [];   % preallocate memory
    offset = min(xdata)-1;
    for j=1:length(Idx)         % Insert NaNs into gaps
        New = [Idx(j)+1:(Idx(j)+Gap(j)-1)] + offset;
        %for Q=1:length(New), xdata(xdata==New(Q)), end  % Slow check to see that New is indeed new
        Idx = Idx+Gap(j)-1;     % Shift things over cuz of missing spots
        xdata = [xdata New];
        New(:) = NaN;
        ydata = [ydata New];
    end
    Data = sortrows([xdata(:) ydata(:)]);
    xdata = Data(:,1)';
    ydata = Data(:,2)';
           

% -----------------------------------------
% Sub-function: FIT_TO_N_CHARS - takes in a string and 
% returns a string that is exactly n characters long.
function newString = fit_to_n_chars(string, n)
    string = char(string);
    newString = [string(1:min(n,end)), char(' '.*ones(1,n-length(string)))];
    
% -----------------------------------------
% Sub-function: SORT_TRACES
% Param: array of hangles to plots
% Returns that array sorted by the mean-y offset of each plot
% Uses insertion sort.
function Saved_traces = sort_traces(Saved_traces);
    % Get the mean of every line so we can do comparisons
    num_lines = length(Saved_traces);
    Means = zeros(num_lines,1);
    for n=1:num_lines
        Means(n) = nanmean(get(Saved_traces(n),'YData'));
    end
    
    % Sort according to the average y-value, smallest (top line) first.
    for P = 2:num_lines     % Insertion Sort
        temp = Means(P);
        temp2 = Saved_traces(P);
        j = P;
        while j > 1 & Means(j-1) > temp
            Means(j) = Means(j-1);
            Saved_traces(j) = Saved_traces(j-1);
            j = j-1;
        end
        Means(j) = temp;
        Saved_traces(j) = temp2;
    end
    

% -----------------------------------------
% Sub-function: NORMALIZE - corrects slope
% Normalization is done in chunks of fewer pixels 
% because imprefections in scanning will lead to a curving slope.
% A 2nd degree least square fit is also used to account for curve.
% Value should be larger enough such that an accurate
% mean value can be obtained, but small enough such that
% there is no distortion from scanning.
% Experience shows that values of 500 to 1000 visually 
% produce the same results with no noticeable borders.
function normalized_data = normalize(xdata, ydata, prompt, degree)
    normal_line = getNormalLine(xdata, ydata, degree);
    normalized_data = ydata-normal_line;
    
    if isequal('prompt',prompt)
        figure(33), plot(normalized_data)
        r = input('Would you like to manually correct the ydata (getNormalLine)? y/n ','s');
        if ~isempty(r) & (r(1) == 'y')
            disp('Keyboard mode - WAITING FOR INPUT:'), keyboard
        end
    end
    
% -----------------------------------------------
% GETNORMALLINE
function normal_line = getNormalLine(xdata, ydata, degree)
% NO CHUNK SIZE, USING WHOLE LINE
%     mu = mean(ydata);
%     sigma = std(ydata);
%     out_index = find(abs(ydata - mu) > 4*sigma);    % Find skewed values
    
    Idx = find(isfinite(ydata));
    xx = xdata(Idx);
    yy = ydata(Idx);
%     yy(out_index) = NaN;      % Optionally throwing out outliers
    normal_line = ydata;
    
    if length(xx) ~= length(yy)
        error('xx not same lenght as yy, in NORMALIZE')
    end
    minlen = degree + 1 + 200;
    if length(yy) > minlen & length(xx) > minlen
        [P S MU] = polyfit(xx,yy,degree);   % P is polynomial of correction line
        normal_line(Idx) = polyval(P,xx,S,MU);                  % Get correction values
    else
        %disp('Not enough points to use polyfit, using mean instead.')
        normal_line = nanmean(ydata(:));
        normal_line = normal_line(ones(size(ydata)));
    end

% --------------------------------------------------------------------
% Name:   SAVE_IMG_SUBMENU_Callback
% Description -     save image
function save_img_submenu_Callback(h, eventdata, handles, varargin)
    img_handle = get_img_h(handles);
    curfilename = get(handles.filename_txt, 'String')
    [p, curfilename, ext] = fileparts(curfilename);
    curfilename = [curfilename ext];
    
    [filename, pathname] = myputfile(curfilename,'Save image...', handles, 'saveimage');
    if ~isequal(filename,0) | ~isequal(pathname,0)
        [p, name, ext] = fileparts(filename);
        if ~isempty(ext) & isequal(ext(1),'.')
            ext(1) = [];
        end

        fullFileName = fullfile(pathname,filename);
        busy(handles.figure1)
        try
            switch lower(ext)
            case {'jpeg','jpg'}
                imwrite(get(img_handle,'CData'),fullFileName,ext,'Quality',100)
                disp(['Saved as ', fullFileName, ' in JPEG format.'])
            case {'tif','tiff'}
                imwrite(get(img_handle,'CData'),fullFileName,ext)
                disp(['Saved as ', fullFileName, ' in TIFF format.'])
            otherwise
                imwrite(get(img_handle,'CData'),fullFileName,'tif')
                disp(['Saved as ', fullFileName, ' in TIFF format.'])
            end          
            % Clear operations since this can be thought of as a new image now
            clear_operation_history(handles)
            update_titlebar(fullFileName, handles)
        catch
            errordlg(lasterr, 'Save Image Error', 'modal')
        end
        idle(handles.figure1)
    end



% --------------------------------------------------------------------
% Name:   CLOSE_SUBMENU_Callback
% Description -     Closes entire GUI
function varargout = close_submenu_Callback(h, eventdata, handles, varargin)
    if ~isstruct(handles)
        warning('No handle to figure. Deleting callback object.')
        delete(gcbo)
        return
    end

    if ishandle(handles.big_axes) & isempty(findobj(handles.big_axes,'type','line'))
        ButtonName = 'Quit';
    else
        ButtonName=questdlg('Are you sure you want to quit?', ...         
                       'Exit SeisDig?', ...
                       'Quit','Stay','Quit');
    end
    
    % Check for Preview window
    if ishandle(handles.save_submenu) & ishandle(get(handles.save_submenu,'UserData'))
        delete(get(handles.save_submenu,'UserData'))
    end

    
    switch ButtonName
    case 'Quit', 
        try
            delete(handles.figure1)    % Close the entire GUI
        catch
            delete(gcbf)
            warning('Non-existant handle while quitting')
        end
    end
    
% --------------------------------------------------------------------
% Name: GETFILTERSPECT
function Extensions = getfilterspecs(mode_string)
    switch mode_string
    case 'image'
        Extensions = { ...
        '*.tiff;*.tif;*.jpg;*.bmp;*.gif', 'Image files (*.tiff;*.tif;*.jpg;*.bmp;*.gif)';...
        '/Volumes/project/peter/digit/scanned/*.tif;*.tiff;*.jpg;*.gif', 'scanned dir';...
        '/Volumes/project/peter/digit/prepimg/*.tif;*.tiff;*.jpg;*.gif', 'prepimg dir';...
        };
    case 'data'
        Extensions = { ...
        '*.data', 'Data files (*.data)';...
        '/Volumes/project/peter/digit/matdata/*.data', 'matdata dir';...
        '/Volumes/project/peter/digit/matdata/tr-*.data', 'tr- matdata dir';...
        };
    case 'gaps'
        Extensions = { ...
        '/Volumes/project/peter/digit/matdata/gaps-*.mat', 'matdata dir';...
        };
    otherwise
        Extensions = '';
    end
% --------------------------------------------------------------------
% Name: LOAD_SUBMENU_Callback
function varargout = load_submenu_Callback(h, mode_string, handles, varargin)
    Extensions = getfilterspecs(mode_string);

    [filename, pathname] = mygetfile(Extensions ,['Load ' mode_string '...'], handles, mode_string);
    
    if isequal(pathname,0)
        % User canceled
    elseif isequal(filename,0)
        % User canceled
    else
        [ignore,name,ext,ver] = fileparts(filename);
        switch mode_string
        case 'image'
            switch lower(ext)
            case {'.tif', '.tiff', '.jpg', '.jpeg', '.bmp', '.gif'}
                f = Loading_bar;
                handles= load_img_from([pathname filename], handles);


                close(f)
            otherwise
                errordlg('Unrecognized image file extension','Cannot open','modal')
            end
        case 'data'
            busy(handles.figure1)
            [errors, answer] = load_pts_file([pathname filename], handles, 0);  
            idle(handles.figure1)
            
            if ~isempty(errors)
                errordlg(errors,'Load point failed.')
            elseif ~isequal(answer,'Cancel')
                update_queue(pathname, filename, handles);
            end
        otherwise 
            warndlg('Unrecognized file extension.','Cannot open','modal')
        end  
    end

% --------------------------------------------------------------------
% UPDATE_TITLEBAR
function update_titlebar(fullfilename, handles)
    set(handles.filename_txt, 'String', fullfilename, 'UserData', fullfilename) 
    set(handles.figure1, 'Name', ['SeisDig - ' fullfilename])
    
% --------------------------------------------------------------------
% LOAD_PTS_FILE
function [errors, answer] = load_pts_file(fName, handles, autoLoad)
    errors = '';
    answer = [];
    try
        load('-mat', fName)
    catch
        errors = lasterr, return
    end
    
    if ~exist('YLabelInfo')
         YLabelInfo = [];
    end
    
    % Check if all expected files exist
    if ~exist('fileName') | ~exist('LineInfo') | ~exist('Operations') | ~exist('YLabelInfo')
        errors = 'Corrupted file. Missing data. Aborting load.', return
    end
    
    if exist('px_per_minute')
        setNum(handles.px_per_minute_txt, px_per_minute);
    end
    if exist('max_period')
        setNum(handles.max_period,max_period);
    end
    if exist('dpi')
        setNum(handles.px_per_inch_txt, dpi);
    end
    
    currentFile = get(handles.filename_txt,'UserData');
    if ~isequal(currentFile,fileName)    
        if exist('autoLoad') & autoLoad ~= 0
            fileNotFound = isempty(dir(fileName));
            disp(['Loading file ' fileName])
            [handles errors] = load_img_from(fileName,handles);
            if ~isempty(errors)
                errordlg(errors,'Digitize queue error'), return
            end
            
            answer = 'autoLoad';
        else
            answer = questdlg(['File name does not match. '...
                               'Do you wish to open reference image (' fileName ')?'],...
                              'Mismatched File Names','Yes','No','Cancel','Cancel');
            % Load image
            if isequal(answer,'Yes')
                disp(['Loading file ' fileName])
                
                f = Loading_bar;
                [handles errors] = load_img_from(fileName,handles);
                close(f) % Close the loading message box
                
                if ~isempty(errors)
                    errordlg(errors,'Digitize queue error'), return
                end

            elseif isequal(answer,'Cancel')
                return
            end
        end
    end
    
    % Delete all current objects
    delete(findobj(handles.figure1,'type','line'))
    delete(findobj(handles.figure1,'type','text'))
    reset_ylabel(handles.big_axes)
    
    disp('Loading new data...')
    restore_objects(LineInfo, handles)
    
    % Remove non-functional display
    delete(findobj(handles.figure1,'Tag','display'))
    delete(findobj(handles.figure1,'Tag','highlighting'))
    
    test_current_state(handles)
    
    % Always fix up YLabel   
    fixYLabelInfo(handles);
    YLabelInfo = getYLabelInfo(handles);
    
    setYLabelInfo(YLabelInfo, handles)
    update_labeling(YLabelInfo, handles)
    line_number_txt_Callback(handles.line_number_txt,[],handles)
    reset_row_values(handles)
   
% --------------------------------------------------------------------
% Name: STRIPBASE
function base = stripBase(fileName)
    start_idx = findstr('brk_41_01_', fileName);
    end_idx = min([length(fileName), findstr('.', fileName)-1]);    % End or before '.'
    end_idx2 = min(findstr('-ex', fileName))-1;
    end_idx = min(end_idx,end_idx2);
    base = fileName(start_idx:end_idx);

% --------------------------------------------------------------------
% Name: FIXYLABELINFO - Since the ylabel often gets out of order right now
% this function goes through and rebuilds it.
function YLabelInfo = fixYLabelInfo(handles)
%disp('Call to fixYLabelInfo')
LineNumbers = [];
PolyCell = {};
N = getUniqueLineNums(handles.big_axes,'CtrlPt');
for n = sort(N)
    [X Y] = getCtrlPtData(n, handles);
    P = calc_P(X,Y);
    LineNumbers = [LineNumbers n];
    PolyCell = {PolyCell{:} P};
end
YLabelInfo = {LineNumbers PolyCell};
setYLabelInfo(YLabelInfo, handles);
update_labeling(YLabelInfo, handles, 'no short cut!');


% --------------------------------------------------------------------
% GETCTRLPTDATA
function [X,Y] = getCtrlPtData(lineNum, handles)
    CtrlPts = findobj(handles.big_axes,'Tag','CtrlPt','UserData',lineNum);
    X = get(CtrlPts,{'XData'});
    Y = get(CtrlPts,{'YData'});
    X = [X{:}];
    Y = [Y{:}];


% --------------------------------------------------------------------
% Name: TEST_CURRENT_STATE - test if everything okay with the GUI
% Return 0 if failure.
function bool = test_current_state(handles)

    % Test Edge Lines
    BigLeft = findobj(handles.figure1,'Tag','left_edge','Parent',handles.big_axes);
    BigRight = findobj(handles.figure1,'Tag','right_edge','Parent',handles.big_axes);
    SmLeft = findobj(handles.figure1,'Tag','left_edge','Parent',handles.small_axes);
    SmRight = findobj(handles.figure1,'Tag','right_edge','Parent',handles.small_axes);


    if length(BigLeft)  ~= 1
        warning(['Corrupted big left edge: ' num2str(BigLeft(:)')])
    end 
    if length(BigRight) ~= 1
        warning(['Corrupted big right edge: ' num2str(BigRight(:)')])
    end 
    if length(SmLeft)   ~= 1
        warning(['Corrupted small left edge: ' num2str(SmLeft(:)')])
    end 
    if length(SmRight)  ~= 1
        warning(['Corrupted small left edge: ' num2str(SmRight(:)')])
    end 
    init_edge_lines(handles)
    
    % Test YTick
    ytick = get(handles.big_axes,'YTick');
    if any(ytick ~= sort(ytick))
        warning('YTick not increasing')
        fixYLabelInfo(handles);
    end


% --------------------------------------------------------------------
% KEY_PRESS_FCN
function key_press_fcn(handles, varargin)

DEL = 127;  ESC = 27;
LEFT = 28;  RIGHT = 29;     UP = 30;    DOWN = 31;
ENTER = 13;

% button_state = get(handles.delete_mode,'Value');
if length(varargin) == 0
    cchar = get(handles.figure1, 'CurrentCharacter');
else
    cchar = varargin{1};
end
current_char = double(cchar);

% Hold numeric input in a buffer
buffer = handles.keypress_buffer;
switch cchar
case {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'}
    % Set buffer to hold the value
    set(buffer, 'UserData', [get(buffer,'UserData') cchar]) % Append new character
    set(buffer, 'String',get(buffer,'UserData'))
end

if ~isempty(current_char)       % Empty when user pressed control
    switch current_char
    case ENTER
        string = get(buffer,'UserData');
        num = str2double(string);
        if isfinite(num)
            set(handles.line_number_txt,'String',string)
            line_number_txt_Callback(handles.line_number_txt, [], handles)
        end
        set(buffer, 'String', [], 'UserData', [])
    case {double('x'), double('X'), double('y'), double('Y') }
        num = str2double(get(handles.stretchY,'String'));
        mult = 2;
        
        % Are we dealing with X or Y box in GUI? Code and GUI label switched.
        % Opposite of DataAspectRatio - x is y and y is x.
        switch current_char
        case {double('x'), double('X') }
            h = handles.stretchY;
        otherwise 
            h = handles.stretchX;
        end
        
        num = str2num(get(h,'String'));
        num = num(1);  %In case more than one
        mult = 2;
        
        switch current_char
        case {double('x'), double('y') }
            newStr = num2str(num/mult);    % Decrease value
        otherwise 
            newStr = num2str(num*mult);    % Increase value
        end
        set(h, 'String', newStr)
        stretch_Callback(handles)
    case {double(':')}          % Toggle center or exac
        toggle(handles.center_check)
        center_check_Callback(handles.center_check,[],handles)
        generic_motion_fcn(handles, 'recheck')
    case {double('?')}          % Toggle any line or one line only
        toggle(handles.any_line)
        any_line_Callback(handles.any_line, [], handles)
        generic_motion_fcn(handles, 'recheck')
    case { double(' ')}         % <space>
        set(handles.digitize,'Value',get(handles.digitize,'Max'))
        digitize_pts_Callback(handles.digitize, [], handles)
    case double('.')
        set_line_number(1,handles)
    case double(',')
        set_line_number(-1,handles)
    case {double('m'), double('M')}
        set_line_number(-Inf, handles)
    case {'=', '+'}
        set_zoom_txt(+1,guidata(gcbo))
    case '-'
        set_zoom_txt(-1,guidata(gcbo))
    case {double('i'), double('I')}
        toggle_button(handles.set_pt_button,[],handles)
    case {double('d'), double('D')}
        toggle_button(handles.delete_button,[],handles)
    case {double('l'), double('L')}
        toggle_button(handles.select_pts_button,[],handles)
    case {double('e'), double('E')}
        toggle_button(handles.bad_button,[],handles)
    case {double('h'), double('H')}
        toggle_button(handles.hand_button,[],handles)
    case {double('z'), double('Z')}
        toggle_button(handles.zoom_button,[],handles)
    case {double('u'), double('U')}
        undo(handles)
    case DEL
        Obj = findobj(handles.big_axes,'Selected','on')
        make_undoable('delete',Obj,handles)
        delete(Obj)
        lineNum = getLineNum(handles);
        update_ctrlpt_stuff(lineNum, handles)
    case {UP, DOWN} % y_slider
        maxx = get(handles.y_slider,'Max');
        minn = get(handles.y_slider,'Min');
        step = get(handles.y_slider,'SliderStep');
        value = get(handles.y_slider,'Value');
        if isequal(current_char,UP)
            value = min(max(minn,value+step(1).*(maxx-minn)),maxx);
        else
            value = min(max(minn,value-step(1).*(maxx-minn)),maxx);
        end
        set(handles.y_slider,'Value',value)
        slider_Callback(handles)
    case {LEFT, RIGHT}  % x_slider
        maxx = get(handles.x_slider,'Max');
        minn = get(handles.x_slider,'Min');
        step = get(handles.x_slider,'SliderStep');
        value = get(handles.x_slider,'Value');
        if isequal(current_char,RIGHT)
            value = min(max(minn,value+step(1).*(maxx-minn)),maxx);
        else
            value = min(max(minn,value-step(1).*(maxx-minn)),maxx);
        end
        set(handles.x_slider,'Value',value)
        slider_Callback(handles)
    case double('`')
        disp('Debug mode'), keyboard
    case double('~')
        disp('Crop image'), I = imcrop;
	    [fname, pname] = uiputfile('*.tif');
        if~(isequal(fname,0) | isequal(pname,0))
            imwrite(I, fullfile(pname, fname));
        	disp([fullfile(pname, fname) ' saved.'])
        end
    end
end


% --------------------------------------------------------------------
% Name: TOGGLE_BUTTON - use this to toggle a button using a keypress.
%   Toggles button and calls button_Callback as if the user had clicked
%   on the button.
function toggle_button(h, eventdata, handles)
toggle(h)
button_Callback(h, eventdata, handles)

% --------------------------------------------------------------------
% Name: TOGGLE - flip max to min
function toggle(h)
	if isequal(get(h,'Value'), get(h,'Min'))
        set(h,'Value',get(h,'Max'))
	else
        set(h,'Value',get(h,'Min'))
	end
    
% --------------------------------------------------------------------
% GETNUMORVALUE - read number in userdata or in string of handle.
%   Returns NaN if not a valid number.
function num = getNumOrValue(h)
    num = get(h,'Value');
    if isempty(num)
        num = str2double(get(h, 'String'));
    end
    if isfinite(str2double(get(h,'String'))) ...
        & round(get(h,'Value')) ~= round(str2double(get(h,'String')))
        setNum(h, num)
    end
    
% --------------------------------------------------------------------
% SETNUM - 1-3-02
function num = setNum(h, number, varargin)
if length(varargin) > 0
    num = num2str(chop(number,varargin{1}));
    set(h,'Value',number,'String', num)
else 
    set(h,'Value',number,'String',num2str(number))
end
% --------------------------------------------------------------------
% SETVALUE - synchronous Value with String (when user changes string)
function  num = setValue(h)
    num = str2double(get(h,'String'));
    set(h,'Value',num)


% --------------------------------------------------------------------
% PX_PER_MINUTE_TXT_Callback
% Changes the trace length to match the entered pixel per minute value.
function px_per_minute_txt_Callback(h, eventdata, handles)
    px_per_minute = setValue(h);
   
    [rightX, leftX] = get_edge_xdata(handles);

    if ~isempty(rightX) & ~isempty(leftX)
        if px_per_minute == 0
            setNum(handles.row_time, Inf);
        else
            % (R-1)-(L+1)+1 = number of pixels in in a line
            setNum(handles.row_time, ...
                  (rightX-leftX-1)/px_per_minute, 5); % Dispaly 5 sig figs
        end
    end

% --------------------------------------------------------------------
% GET_EDGE_XDATA
function [rightX, leftX] = get_edge_xdata(handles);
    rightX = mean(get(findobj(handles.big_axes,'Tag','right_edge'),'XData'));
    leftX = mean(get(findobj(handles.big_axes,'Tag','left_edge'),'XData'));
    
% --------------------------------------------------------------------
% PX_PER_INCH_TXT_Callback
% Changes the trace length to match the entered pixel per inch value.
function px_per_inch_txt_Callback(h, eventdata, handles)
    dpi = setValue(h);
    
    [rightX, leftX] = get_edge_xdata(handles);

    if ~isempty(rightX) & ~isempty(leftX)
        % (R-1)-(L+1)+1
        setNum(handles.row_length, (rightX-leftX-1)/dpi,5);     %Display 5 sig figs
    end
    
    set_scaled_xlabel(handles.big_axes, handles)


% --------------------------------------------------------------------
% Name: ROW_TIME_Callback - minutes per trace
function row_time_Callback(h, eventdata, handles, varargin)
    rowMinutes = setValue(h);
    
    [rightX, leftX] = get_edge_xdata(handles);
    if ~isempty(rightX) & ~isempty(leftX)
        px_per_minute = (rightX-leftX-1)/rowMinutes;   % (R-1)-(L+1)+1 
        setNum(handles.px_per_minute_txt, px_per_minute, 5);  % 5 sig fig
    end
    
% --------------------------------------------------------------------
% Name: ROW_LENGTH_Callback
function row_length_Callback(h, eventdata, handles, varargin)
    rowLength = setValue(h);

    [rightX, leftX] = get_edge_xdata(handles);
    if ~isempty(rightX) & ~isempty(leftX)
        dpi = (rightX-leftX-1)/rowLength;   % (R-1)-(L+1)+1
        setNum(handles.px_per_inch_txt,dpi, 5);    % 5 sig fig
    end
    
% --------------------------------------------------------------------
% FIGURE1_RESZIEFCN
function varargout = figure1_ResizeFcn(h, eventdata, handles, varargin)
    resize_both_axes(handles)
    slider_Callback(handles)

% --------------------------------------------------------------------
% REMOVE_LIST_Callback
function varargout = remove_list_Callback(h, eventdata, handles, varargin)
    List = get(handles.queue_listbox,'String');
    if ~isempty(List)
        idx = get(handles.queue_listbox,'Value');
        List(idx,:) = [];  
        set(handles.queue_listbox,'String',List,'Value',min(max(1,idx-1)))
    end

% --------------------------------------------------------------------
% ADD_LIST_Callback
function varargout = add_list_Callback(h, eventdata, handles, varargin)
[filename, pathname] = mygetfile('*.data', 'Load Data...', handles, 'data');


if isequal(pathname,0)
    % User canceled
elseif isequal(filename,0)
    % User canceled
else
    [pathname,name,ext,ver] = fileparts([pathname filename]);

    update_queue(pathname, filename, handles)
end


% --------------------------------------------------------------------
% UPDATE_QUEUE takes the full pathname and filename (as from
%  uigetfile) and updates the queue if the file name is new.
%  It saves the full path only if the current directory is
%  different.
function update_queue(pathname, filename, handles)
    TRUE = 1;   FALSE = 0;
    List = get(handles.queue_listbox,'String');
    
    % Check if this image already is in the queue
    num = size(List,1);    isnew = TRUE;
    for m=1:num
        [p n e] = fileparts(deblank(List(m,:)));  % Compare only file name + ext.
        if isequal([n e], filename)
            isnew = FALSE;
            break
        end
    end
    
    [path,name,ext,ver] = fileparts(fullfile(pathname, filename));
    expr_path = fileparts(get(handles.file_expr,'String'));

    if isnew
        if ~isequal(path, expr_path)
            string = fullfile(path,filename);
        else
            string = filename;
        end
        List = strvcat(List, string);
        newCount = size(List,1);
        set(handles.queue_listbox,'String',List,'Value', newCount)
        disp([string ' added to queue.'])
    else
        disp('File already in the queue.')
    end

% --------------------------------------------------------------------
% FILE_EXPR_Callback
function file_expr_Callback(h, eventdata, handles)
    if isempty(eventdata)
    	file_expr = get(h,'String');
    else file_expr = eventdata;
    end

    if isempty(file_expr)
        Dir = pwd;
        Expr = '*.data';
    elseif isdir(file_expr) & ~isequal(file_expr(end), filesep)
        Dir = file_expr;

        Expr = [];
    else
        [Dir, Expr, ext] = fileparts(file_expr);
        Expr = [Expr ext];
    end

    load_listbox(Dir, Expr,handles)

% ------------------------------------------------------------
% LOAD_LISTBOX - populate the queue_listbox
%     elseif length(dir_path) < 5 | ~isequal(dir_path(end-4:end),'.data')
%         dir_path = strcat(dir_path,'*.data');
function load_listbox(Dir, Expr, handles)
    if isempty(Expr)
        Expr = filesep;
    end
    set(handles.file_expr, 'String', fullfile(Dir,Expr)) 

    file_expr = fullfile(Dir, Expr);
	dir_struct = dir(Dir);

            
    
    if isempty(dir_struct)
        file_names = ['.' filesep];
    else
        file_names = char(dir_struct.name);
        is_dir = [dir_struct.isdir];
        file_names = add_filesep(file_names, is_dir);
    end    
	set(handles.queue_listbox,'String',file_names,'Value',1)
    
% ------------------------------------------------------------
% ADD_FILESEP
function file_names = add_filesep(file_names, is_dir)
    % Make sure at least one space to add filesep
    space = ' ';
    file_names = [file_names space(ones(size(file_names,1),1))];  
    temp = file_names(is_dir,:);
    for n=1:size(temp,1)
        temp(n,min(find(temp(n,:)==' '))) = filesep;
    end
    file_names(is_dir,:) = temp;
    
    % Concatenate on top a list of commonly used expressions
    file_names = strvcat(['.' filesep 'gaps-*.mat'], file_names);
    file_names = strvcat(['.' filesep '*.tif'], file_names);
    file_names = strvcat(['.' filesep 'tr-*.data'], file_names);
    file_names = strvcat(['.' filesep '*.data'], file_names);
        
        
% ------------------------------------------------------------
% QUEUE_LISTBOX_Callback
function queue_listbox_Callback(h, eventdata, handles)
if strcmp(get(handles.figure1,'SelectionType'),'open')
    index_selected = get(h,'Value');
	file_list = get(h,'String');
	filename = deblank(file_list(index_selected,:));
    file_expr = get(handles.file_expr,'String');
    [Dir, name, ext] = fileparts(file_expr);
    [FDir, Fname, Fext] = fileparts(filename);

    Lext = lower(Fext);  % Convert to lower case
    
    if any(filename == '*')
        if isequal(filename(1:min(end,2)), ['.' filesep])
            filename(1:2) = [];
        end
        load_listbox(Dir, filename, handles)
    elseif isdir(fullfile(Dir,filename))  % Don't do anything if not a directory
        Expr = [name ext];
        if isequal(filename,['..' filesep])
            if ~isempty(Dir)
                Dir = fileparts(Dir(1:max(1,end-1)));
            else
                Dir = get(handles.file_expr,'String'); % Stay same dir if no parent
            end
        elseif isequal(filename,['.' filesep])
            Expr = [];
        else
            Dir = fullfile(Dir, filename);
        end
        load_listbox(Dir, Expr,handles)
    elseif isequal(Lext, '.tiff') | isequal(Lext,'.tif') | isequal(Lext,'.jpg') ...
         | isequal(Lext, '.gif')  | isequal(Lext,'.bmp')
         handles = load_img_from( fullfile(Dir,filename), handles );
    elseif isequal(Fext,'.data')
        autoLoad = 0;
        if isempty(fileparts(filename))
            filename = fullfile(Dir,filename);
        end
        [errors, answer] = load_pts_file(filename, handles, autoLoad);
    elseif length(Fname) > 5 & isequal(Fname(1:5),'gaps-') & isequal(Fext,'.mat')
        plot_gaps([Dir filesep], filename, handles);
    end
end


% --------------------------------------------------------------------
% SET_LINE_NUMBER Sets how many minutes to pick out.
function set_line_number(increment, handles)
    if ~isfinite(increment)
        number = min(getLineNumbers(getYLabelInfo(handles)));
        if isempty(number), number = 1; end
        increment = 0;
    else
        number = getLineNum(handles) + increment;
    end
    set(handles.line_number_txt,'String',num2str(number))
    line_number_txt_Callback(handles.line_number_txt, 0, handles)

% --------------------------------------------------------------------
% Name: MOD_LINE_NUMBER - modify line numbers
function mod_line_number(increment, handles)
    line_number_txt_Callback(handles.line_number_txt,increment, handles)

% --------------------------------------------------------------------
% Name: LINE_NUMBER_TXT_Callback
% increment is empty if called from GUI
% increment holds increment value if passed from SET_LINE_NUMBER 
% 0 flags no changing the line numbers! Must use for UNDO chg_line_num
function line_number_txt_Callback(h, increment, handles)
    [TRACE_HIGHLITE, PT_HIGHLITE, TRACE_BACKGND, PT_BACKGND] = COLORS;
    
    number = str2double(get(handles.line_number_txt,'String'));

    if isnan(number) | number < 0 | number > 999
        helpdlg('Line numbers must be betweenn 0 and 999')
        oldNum = get(handles.line_number_txt,'Value');
        set(handles.line_number_txt,'String',num2str(oldNum))
        return
    else
        set(handles.line_number_txt,'Value',number) % Synchronize value and string
    end

    % Change selected points to current line number
    SelectedPts = findobj(handles.big_axes,'Selected','on','Tag','CtrlPt');
    Xcell = get(SelectedPts,{'XData'});
    OldDatCell = get(SelectedPts,{'UserData'});

    if isempty(increment)
        if ~isempty(SelectedPts)
            % Need up update all line number labels here.
            set(SelectedPts(:),'UserData',number,'Selected','off')
            update_ctrlpt_stuff(number, handles)
        end
    elseif increment ~= 0
        for n=1:length(SelectedPts) 
            lineNum = get(SelectedPts(n),'UserData');
            if isempty(lineNum)
                delete(SelectedPts(n)), error('Corrupted control pt w/o a line number.')
            end
            P = update_zero_line(lineNum, handles);
            YLabelInfo = update_ylabel_info(lineNum, P, handles);
            
            set(SelectedPts(n),'UserData', lineNum+increment);
            DESELECT_AFTER_MOD = 0;      % Deselect after modification or not
            if DESELECT_AFTER_MOD
                set(SelectedPts(n), 'Selected','off')
            end

            P = update_zero_line(lineNum+1, handles);
            YLabelInfo = update_ylabel_info(lineNum+1, P, handles);
        end
        if ~isempty(SelectedPts)
            YLabelInfo = fixYLabelInfo(handles);
        end
    end
    NewDatCell = get(SelectedPts, {'UserData'});
    make_undoable('chg_line_num', {Xcell, OldDatCell, NewDatCell}, handles);
    
    AllPts = findobj(handles.big_axes,'Tag','CtrlPt');
    LinePts = findobj(AllPts,'flat','UserData',number);

    AllTraces = findobj(handles.big_axes,'Tag','trace_line');
    lineTrace = findobj(AllTraces,'flat','UserData',number);

    % Turn all highlighted points and trace lines to background
    set(findobj(AllPts,'flat','Color',PT_HIGHLITE),'Color',PT_BACKGND)
    set(findobj(AllTraces,'flat','Color',TRACE_HIGHLITE),'Color',TRACE_BACKGND, 'MarkerSize', 3)
    
    % Turn matching line points and trace line to highlighted
    set(LinePts,'Color',PT_HIGHLITE)
    set(lineTrace,'Color',TRACE_HIGHLITE, 'MarkerSize', 6)
    
    update_zero_line(number, handles);   % YTick and YTickLabel stays the same.
    update_smooth_button(handles)
    
    drawnow
    
% --------------------------------------------------------------------
% CALC_P - calculate a new Polynomial for the base-line (aka zero-line)
function P = calc_P(X,Y)
    [X, idx] = unique(X);
    Y = Y(idx);
    if length(X) > 1 & length(Y) > 1
        P = polyfit(X,Y,1);
    else P = Y;
    end
    
% --------------------------------------------------------------------
% UPDATE_ZERO_LINE - whenever point added/deleted or line number changed.
% Create or delete the base_line (aka zero_line).
% Pass in the line number.
% Returns the 
function P = update_zero_line(lineNum, handles)
    LinePts = findobj(handles.big_axes,'Tag','CtrlPt','UserData',lineNum);
    
    X = get(LinePts(:),{'XData'}); X=[X{:}];
    Y = get(LinePts(:),{'YData'}); Y=[Y{:}];
    
    P = calc_P(X,Y);
    
    if ~isempty(P)
        xx = [min(X) max(X)];
        zeroLineAll = findobj(handles.figure1,'Tag','zero_line');
        set(zeroLineAll,'Visible','off')
        zeroLine = findobj(zeroLineAll,'flat','UserData',lineNum);
        
        if length(zeroLine) ~= 2
            if length(zeroLine) > 1
                warning(['Fixing corrupt baselines (' num2str(length(zeroLine)) ' found).'])
            end
            delete(zeroLine)
            zeroLine = [];
        end
        if isempty(zeroLine)
            plot_h = plot(xx, polyval(P,xx), 'y-.','Parent',handles.big_axes,'Tag','zero_line','UserData',lineNum);

            %% Obsolete Code Fragment: Line Numbers cannot be used as
            %% indices since they can be less than 1.
%             handles.zero_line{lineNum} = plot_h;
%             guidata(handles.figure1, handles)
            
            small_h = copyobj(plot_h, handles.small_axes);
            set(small_h, 'LineWidth', 2, 'Color', 'm')
        else
            set(zeroLine,'XData',xx,'YData',polyval(P,xx),'Visible','on')
            if get(zeroLine(1),'Parent') == handles.small_axes
                small_h = zeroLine(1);
            else small_h = zeroLine(2);
            end
            set(small_h, 'LineWidth', 2, 'Color', 'm')
        end

    % Single point, delete the line
    else
        delete(findobj(handles.figure1,'Tag','zero_line'))
    end


% --------------------------------------------------------------------
% Name: _GETYLABELINFO
function YLabelInfo = getYLabelInfo(handles)
    YLabelInfo = get(handles.line_number_txt,'UserData');
function setYLabelInfo(YLabelInfo, handles)
    set(handles.line_number_txt,'UserData', YLabelInfo);
% --------------------------------------------------------------------
% Name: _GETLINENUMBERS
function LineNumbers = getLineNumbers(YLabelInfo)
    if isempty(YLabelInfo)
        LineNumbers = []; return
    else
        LineNumbers = YLabelInfo{1};
    end
% --------------------------------------------------------------------
% Name: _GETPOLYCELL_
function PolyCell = getPolyCell(YLabelInfo)
    PolyCell = YLabelInfo{2};

% --------------------------------------------------------------------
% UPDATE_YLABEL_INFO
% Update the polyval data with the P value passed in.
function YLabelInfo = update_ylabel_info(lineNum, P, handles)
    % Update info for updating line text
    YLabelInfo = getYLabelInfo(handles);
  
    %Create new cell hold line number and P
    if ~isempty(P) & isempty(YLabelInfo)
        YLabelInfo = {lineNum {P}};
        set(handles.line_number_txt,'UserData', YLabelInfo);
    else 
        LineNumbers = getLineNumbers(YLabelInfo);
        PolyCell = getPolyCell(YLabelInfo);
        idx = find(LineNumbers == lineNum);
        
        if length(idx) > 1
            error('Error: corrupted line number labeling info. Duplicate entry found.'), return
        end
        
        % Add a new entry
        if isempty(idx)
            LineNumbers(end+1) = lineNum;
            PolyCell(end+1) = {P};
        % Replace an old entry
        else
            if isempty(P)  
                LineNumbers(idx) = [];
                PolyCell(idx) = [];
            else PolyCell(idx) = {P};
            end
        end
        
        if ~isempty(LineNumbers) & ~isempty(PolyCell)
            YLabelInfo = {LineNumbers PolyCell};
        else YLabelInfo = [];
        end
        [LineNumbers I] = sort(LineNumbers);
        PolyCell = PolyCell(I);
        set(handles.line_number_txt,'UserData', YLabelInfo)
    end
    
% --------------------------------------------------------------------
% GET_P - returns polynomial used to draw zero_line
function P = getP(lineNum, handles)
    YLabelInfo = get(handles.line_number_txt,'UserData');
    if ~isempty(YLabelInfo)    
        LineNumbers = YLabelInfo{1};
        PolyCell = YLabelInfo{2};
        idx = find(LineNumbers == lineNum);
        P = [PolyCell{idx}];    % Returns empty if idx is empty.
    else P = [];
    end


% --------------------------------------------------------------------
% UPDATE_LABELING - updates y-axis labels (ytick and yticklabel)
% Looks at LineLabelInfo and updates the text labels.
% Update when you set a new control point.
% Delete when you delete a point and no more left.
function update_labeling(YLabelInfo, handles, varargin)
if ~isempty(YLabelInfo)
    LineNumbers = YLabelInfo{1};
    PolyCell = YLabelInfo{2};
    bgXLim = get(handles.big_axes,'XLim');
    
    ytick = get(handles.big_axes,'YTick');
    yticklabel = get(handles.big_axes,'YTickLabel');
    
    take_short_cut = (nargin == 2);
    
    if take_short_cut & length(ytick) == length(LineNumbers)
        update_ytick_only(YLabelInfo, handles)
    else
        ytick = [];
        yticklabel = [];                % Reset to empty
        for n=1:length(LineNumbers)
            try
                y = polyval2(PolyCell{n}, bgXLim(1));
            catch
                PolyCell{n}
                P = update_zero_line(LineNumbers(n), handles)
                update_ylabel_info(LineNumbers(n), P, handles)
                error([num2str(n) ' => ' lasterr])
            end
            while ~isempty(ytick) & any(y == ytick) % Make sure no overlaps
                y = y+0.1;
            end
            ytick = [ytick; y];
            yticklabel = strvcat(yticklabel, num2strLUT(LineNumbers(n)));
        end
    end
    [ytick I] = sort(ytick);
    yticklabel = yticklabel(I,:);
    set(handles.big_axes,'YTick',ytick,'YTickLabel',yticklabel, 'YColor', 'b')
else
    reset_ylabel(handles.big_axes)
end
% --------------------------------------------------------------------
% UPDATE_YTICK_ONLY - use only when we know YTickLabels have not change.
%  In other words, no new line numbers added or deleted since YLabelInfo's
%  last update.
function update_ytick_only(YLabelInfo, handles)
if ~isempty(YLabelInfo)
    PolyCell = YLabelInfo{2};
    bgXLim = get(handles.big_axes,'XLim');
    
    ytick = get(handles.big_axes,'YTick');
    yticklabel = get(handles.big_axes,'YTickLabel');


    y = [];
    for n=1:length(ytick)
        try
            y = polyval2(PolyCell{n}, bgXLim(1));
        catch
            PolyCell{n}
            error([num2str(n) ' => ' lasterr])
        end
        while any(y==ytick)
            y = y+.1;
        end
        ytick(n) = y;
    end
    [ytick I] = sort(ytick);
    yticklabel = yticklabel(I,:);
    set(handles.big_axes,'YTick',ytick)
end


% -----------------------------------
% NUM2STRLUT - faster lookup table for creating y-labels.
function S = num2strLUT(num)
    switch num
    case 0,S='#0'; case 1,S='#1'; case 2,S='#2'; case 3,S='#3'; case 4,S='#4'; case 5,S='#5'; 
    case 6,S='#6'; case 7,S='#7'; case 8,S='#8'; case 9,S='#9'; case 10,S='#10'; case 11,S='#11'; 
    case 12,S='#12'; case 13,S='#13'; case 14,S='#14'; case 15,S='#15'; case 16,S='#16'; 
    case 17,S='#17'; case 18,S='#18'; case 19,S='#19'; case 20,S='#20'; case 21,S='#21'; 
    case 22,S='#22'; case 23,S='#23'; case 24,S='#24'; case 25,S='#25'; case 26,S='#26'; 
    case 27,S='#27'; case 28,S='#28'; case 29,S='#29'; case 30,S='#30'; case 31,S='#31'; 
    case 32,S='#32'; case 33,S='#33'; case 34,S='#34'; case 35,S='#35'; case 36,S='#36'; 
    case 37,S='#37'; case 38,S='#38'; case 39,S='#39'; case 40,S='#40'; case 41,S='#41'; 
    case 42,S='#42'; case 43,S='#43'; case 44,S='#44'; case 45,S='#45'; case 46,S='#46'; 
    case 47,S='#47'; case 48,S='#48'; case 49,S='#49'; case 50,S='#50'; case 51,S='#51'; 
    case 52,S='#52'; case 53,S='#53'; case 54,S='#54'; case 55,S='#55'; case 56,S='#56'; 
    case 57,S='#57'; case 58,S='#58'; case 59,S='#59'; case 60,S='#60'; case 61,S='#61'; 
    case 62,S='#62'; case 63,S='#63'; case 64,S='#64'; case 65,S='#65'; case 66,S='#66'; 
    case 67,S='#67'; case 68,S='#68'; case 69,S='#69'; case 70,S='#70'; case 71,S='#71'; 
    case 72,S='#72'; case 73,S='#73'; case 74,S='#74'; case 75,S='#75'; case 76,S='#76'; 
    case 77,S='#77'; case 78,S='#78'; case 79,S='#79'; case 80,S='#80'; case 81,S='#81'; 
    case 82,S='#82'; case 83,S='#83'; case 84,S='#84'; case 85,S='#85'; case 86,S='#86'; 
    case 87,S='#87'; case 88,S='#88'; case 89,S='#89'; case 90,S='#90'; case 91,S='#91'; 
    case 92,S='#92'; case 93,S='#93'; case 94,S='#94'; case 95,S='#95'; case 96,S='#96'; 
    case 97,S='#97'; case 98,S='#98'; case 99,S='#99'; 
    otherwise S = ['#' num2str(num)];   % Resort to slower version
    end

% --------------------------------------------------------------------
% RESET_YLABEL - Reset to no line numbers
function reset_ylabel(big_axes)
    % Two options: black pixel marks or no labels at all
    switch 'no ticks'
    case 'black'
        set(big_axes,'YTickMode','auto','YTickLabelMode','auto','YColor','k') % Black ticks
    otherwise
        set(big_axes,'YTick',[],'YTickLabel',[]) % No ticks
    end

% -------------------------------------------------------------------- 
% POLYVAL2 - 1 or 2 coefficients only.
function y = polyval2(ZERO_P, x);
    switch length(ZERO_P) 
    case 1, y = ZERO_P;
    case 2, y = ZERO_P(1)*x + ZERO_P(2);
    otherwise
        warning(['POLYVAL2 error: Invalid line polynomial => ''' num2str(ZERO_P) ''''])
        y = 1;
    end
    
% --------------------------------------------------------------------
% UPDATE_CTRLPT_STUFF - useful function to call
% all three related functions
function update_ctrlpt_stuff(lineNum, handles)
    P = update_zero_line(lineNum, handles);
    YLabelInfo = update_ylabel_info(lineNum, P, handles);
    update_labeling(YLabelInfo, handles)

% -------------------------------------------
function update_all_ctrlpt_stuff(handles)
    for lineNum=getUniqueLineNums(handles.big_axes, 'CtrlPt')
        update_ctrlpt_stuff(lineNum, handles)
    end


%----- Subfunction -----
% GETUNIQUELINENUMS - GET_UNIQUE_LINE_NUMS
% tagStr - tag identifying control points.
function LineNums = getUniqueLineNums(big_axes, tagStr)
    CtrlPts = findobj(big_axes,'Tag', tagStr);
    LineNums = get(CtrlPts(:),{'UserData'});
    LineNums = unique([LineNums{:}]);
 

%----- Subfunction - for keeping code clean. Group actions related to center_check
% ADJUST_VALUES_FOR_CENTER_POINT
function [PLOT_STYLE, y, Size] = adjust_values_for_center_point(x,y,handles)
    if isMax(handles.center_check)
        y = get_mid_y(x,y,handles);
        PLOT_STYLE = '.-';
        Size = 18;
    else
        PLOT_STYLE = 'x-';
        Size = 10;
    end


%---------------------------
% SET_NEW_POINT - interpolates with trace line if necessary.
% Calls the right make_undoable function, depending on if interpolation was done.
function plot_handle = set_new_point(x,y, lineNum, handles, plot_handle, varargin)
% profile on -detail builtin
    % - - - - - - - - - - - - - - - - - - - -
    % Initialize some variables
    % - - - - - - - - - - - - - - - - - - - -
    [TRACE_HIGHLITE, PT_HIGHLITE, TRACE_BACKGND, PT_BACKGND] = COLORS;
    [PLOT_STYLE,y,Size ]= adjust_values_for_center_point(x,y,handles);
    [Img, img_h] = get_image(handles);
    x = round(x);
    MERGE = 3;
    % - - - - - - - - - - - - - - - - - - - -
    
    if isMax(handles.center_check)
        y_index = get_connected_pixels(x,y, Img, handles.g_width, MERGE);
        y = mean(y_index);
    end

    if isempty(plot_handle)
        plot_handle = plotCtrlPt(x,y, PLOT_STYLE, Size,lineNum, PT_HIGHLITE,handles);
    end
    
    % Allow us to delete the new point if user undoes
    new_xdata = x;
    TouchPts = [];
    traceLine = findobj(handles.big_axes,'Tag','trace_line','UserData',lineNum);
    traceInfo = get(traceLine);     % Get struct of info

    %%
    % Future versions may wish to spline through the point smoothly here.
    %%
    

    if length(varargin) > 0
        set_undo = varargin{1};
    else 
        set_undo = 'yes';
    end
    if isequal(set_undo,'yes') | isequal(set_undo,'default')
        make_undoable('new ctrl pt', {plot_handle TouchPts new_xdata traceInfo}, handles);
    end
    
    % Update zero_line, polyvals in line_txt UserData, and y-axis labelings
    update_ctrlpt_stuff(lineNum, handles)
    

% -----------------------------
% PLOT_CTRLPT
function plot_handle = plotCtrlPt(x,y, PLOT_STYLE, Size, lineNum, PT_HIGHLITE,handles)
plot_handle = plot(x, y, PLOT_STYLE,'Parent',handles.big_axes,'Tag','CtrlPt'...
        ,'LineWidth',2,'MarkerSize',Size,'UserData', lineNum,'Color',PT_HIGHLITE);

% --------------------------------------------------------------------      
% ASSERTALLWHOLE - Checks if all data are whole numbers
function assertAllWhole(data)
    if any(data(:) ~= round(data(:)))
        error('Not all whole.')
    end

% --------------------------------------------------------------------
% GET_L_BOUNDARIES
function x_bound = get_L_boundaries(x, xdata, xCtrl, MAX_RANGE)
    idx = find(xdata == x);
    while idx > 0 & isfinite(xdata(idx))
        idx = idx-1;
    end
    xdata = idx:x;
        
    x_bound = max([min(xdata), x-MAX_RANGE, max(xCtrl(xCtrl<x))]);
    
% --------------------------------------------------------------------
% GET_R_BOUNDARIES
function x_bound = get_R_boundaries(x, xdata, xCtrl, MAX_RANGE)
    idx = find(xdata == x);
    while idx <= length(xdata) & isfinite(xdata(idx))
        idx = idx+1;
    end
    xdata = x:idx;
        
    x_bound = min([max(xdata), x+MAX_RANGE, min(xCtrl(xCtrl>x))]);


% -------------------------------------------------------------------------
% Name: INTERPOLATE_
function interpolate(line_handle, lineNumber, handles, USED_HIGHLITE)
% INTERPOLATE returns interpolated data between two points.
%   If the trace exists, the trace data is used.
%   If not, INTERPOLATE runs its own sample trace to get data.
%   INTERPOLATE both changes the trace and sets control points.
% Parameters:
%   line_handle - handle to ctrlLine that we'll interpolate

% ----- Initialize some variables with error checks --------------
xx = get(line_handle,'XData');
yy = get(line_handle,'YData');
% ----- End initialization ---------------------------------------

Img = get_image(handles);
trace_line = findobj(handles.big_axes,'Tag','trace_line','UserData',lineNumber);
if length(trace_line) > 1       % Error check
    errordlg('Corrupted trace line caught in interpolate.','interp error','modal'), return
end

spline_trace(Img, trace_line, xx, yy, lineNumber, handles)

    
% -------------------
% RESTACK - Check for missing restack function
function restack(varargin)
try
    restack(varargin{:})
catch
    % Do nothing if no restack function is available.
end

% ----------------------------------------------------------------------
% SPLINE_TRACE 
% xx, yy - coordinates of control points
function spline_trace(Img, trace_line, xx, yy, lineNum, handles)
    if length(xx) < 2 | length(yy) < 2
        [xx(:) yy(:)]
        error('Bad input: wrong number of xdata and yata')
    end
    maxIdx = find(xx == max(xx));
    minIdx = find(xx == min(xx));
    x1 = xx(minIdx);
    y1 = yy(minIdx);
    x2 = xx(maxIdx);
    y2 = yy(maxIdx);

    xxx = get(trace_line,'XData');  % Data needed later on too if trace exists
    yyy = get(trace_line,'YData');  % Data needed later on too if trace exists
    
    N_PXL = 10;   % Backward/Forward 5 pxls
    
    ENABLE_TRACE_ENDS_MATCHING = length(xx) > 2;
    LINE_ENDS_IGNORE = 3;
    idx = find(xxx < x1 & xxx >= x1-N_PXL & isfinite(yyy));
    if ENABLE_TRACE_ENDS_MATCHING ...
    &  length(idx) == N_PXL     % Consecutive x's found
        xdata1 = xxx(idx);
        ydata1 = yyy(idx);
    else
        [xdata1 ydata1] = safe_trace(x1,y1,Img,-1,handles.g_width,1,LINE_ENDS_IGNORE,0.3,N_PXL); % Backward
    end
    
    idx = find(xxx > x2 & xxx <= x2+N_PXL & isfinite(yyy));
    if ENABLE_TRACE_ENDS_MATCHING ...
    &  length(idx) == N_PXL     % Consecutive x's found
        xdata2 = xxx(idx);
        ydata2 = yyy(idx);
    else
        [xdata2 ydata2] = safe_trace(x2,y2,Img,1,handles.g_width,1,LINE_ENDS_IGNORE,0.3,N_PXL); % Forward  
    end
         
    if any(isnan(ydata1)) | any(isnan(ydata2))
        errordlg('Unexpected NaN in spline_trace','spline_trace error','modal')
    elseif isempty(ydata1) | isempty(ydata2)
        errordlg('No slope available to interpolate')
    end
    
    % Stick in points from controlLine
    xdata = x1:x2;
    X = [xdata1(:); xdata2(:); xx(:)];
    Y = [ydata1(:); ydata2(:); yy(:)];
    
    % Spline chosen as best strategy for our data set.
    switch 0
    case 1, ydata = interp1(X, Y, xdata, 'linear');
    case 2, ydata = interp1(X, Y, xdata, 'spline');
    case 3, ydata = interp1(X, Y, xdata, 'pchip');
    otherwise, ydata = spline(X, Y, xdata);
    end   
      
    if any(isnan(ydata))
        errordlg('Spline returned NaN.','Interpolate error','modal')
    end
    
    if isempty(trace_line)
        [TRACE_HIGHLITE, PT_HIGHLITE, TRACE_BACKGND, PT_BACKGND] = COLORS;
        STYLE = '*-';
        plot(xdata,ydata, STYLE, 'Tag','trace_line','MarkerSize',6,'Parent',handles.big_axes...
            ,'UserData', lineNum,'Color',TRACE_HIGHLITE)
    else
        [duplicates IA IB] = intersect(xxx, xdata);
        xxx(IA) = [];
        yyy(IA) = [];
    
        newXData = [xxx(:); xdata(:)];
        newYData = [yyy(:); ydata(:)];
        
        Data = sortrows([xdata(:) ydata(:)]);
        newXData = Data(:,1);
        newYData = Data(:,2);
        
        % Resize
        xMin = round(min([xxx(:); xdata(:)]));
        xMax = round(max([xxx(:); xdata(:)]));
        newXData = zeros(xMax-xMin+1,1);
        newXData(:) = NaN;
        newYData = newXData;
        IDXOFFSET = -xMin+1;
        
        % Stick in old points
        newXData(xxx(~isnan(yyy))+IDXOFFSET) = xxx(~isnan(yyy));
        newYData(xxx(~isnan(yyy))+IDXOFFSET) = yyy(~isnan(yyy));
        % Stick in new points, replacing some part of old points
        newXData(xdata(~isnan(ydata))+IDXOFFSET) = xdata(~isnan(ydata));
        newYData(xdata(~isnan(ydata))+IDXOFFSET) = ydata(~isnan(ydata));
        
        set(trace_line,'XData', newXData, 'YData', newYData)
    end
    

%---------------------------
% START_NEW_LINE - creates a new line to be updated, and sets window motion functions
% to respond to it
function plot_handle = start_new_line(x,y, lineNum, handles, plot_handle)
    % - - - - - - - - - - - - - - - - - - - -
    % Initialize some variables
    % - - - - - - - - - - - - - - - - - - - -
    [TRACE_HIGHLITE, PT_HIGHLITE, TRACE_BACKGND, PT_BACKGND] = COLORS;
    [PLOT_STYLE, y] = adjust_values_for_center_point(x,y,handles);
    % - - - - - - - - - - - - - - - - - - - -
    
    if isempty(plot_handle)
        plot_handle = plot(x, y, 'ro-','Parent',handles.big_axes,'Tag','CtrlPt'...
            ,'LineWidth',2,'MarkerSize',10,'UserData', lineNum,'Color',PT_HIGHLITE);
    else
        set(plot_handle,'Marker','o','Color','r','LineStyle','-')
    end
    set(plot_handle,'Tag','activeCtrlLine','EraseMode','xor')
    
    % Set motion function to update line and up function to finalize line
    set(handles.figure1,'WindowButtonMotionFcn','GUI(''update_line'',''activeCtrlLine'',guidata(gcbo))',...
        'WindowButtonUpFcn','GUI(''finalize_line'',''activeCtrlLine'',guidata(gcbo))')


% --------------------------------------------------------------------      
% UPDATE_CTRL_LINE - Update the control line to include a new point 
%                    with sorting.
% Used in both setting a new range and updating a range. Cannot handle
% the undoing.
function update_ctrl_line(xx, yy, line_handle)
    xdata = get(line_handle,'XData');
    ydata = get(line_handle,'YData');
    XY = [[xdata(:); xx(:)] [ydata(:); yy(:)]];
    XY = sortrows(XY);
    set(line_handle,'XData',XY(:,1),'YData',XY(:,2))
    
    
% --------------------------------------------------------------------   
% FIND_CTRLPTS
% Check if its in the range of a control line
% Returns [] if no interp range found.
function line_handle = find_CtrlPts(x, handles)
    CtrlPts = findobj(handles.big_axes,'Tag','CtrlPt'...
                    ,'UserData', getLineNum(handles));
    xx = [];    line_handle = [];
    for n=1:length(CtrlPts)
        xx = get(CtrlPts(n),'XData');
        if any(min(xx) <= x) & any(x <= max(xx))
            line_handle = [line_handle CtrlPts(n)];
        end
    end

% --------------------------------------------------------------------   
% Name: UPDATE_LINE - update line while user holding mouse button down.
function update_line(tag, handles)
    line_handle = findobj(handles.big_axes, 'Tag', tag);
    
    % Check if line is corrupt, delete the line if something is 
    % wrong and return.
    if length(line_handle) ~= 1
        delete(line_handle)
        set(handles.figure1,'WindowButtonMotionFcn', [])
        button_Callback(handles.set_pt_button,[],handles)
        return
    end
    pt = get(handles.big_axes, 'CurrentPoint');
    
    % Move image if it goes out of axes, Center if desired.
    % Future revisions may have image scroll with user drags out of bounds
    XLim = get(handles.big_axes,'XLim');
    YLim = get(handles.big_axes,'YLim');
    % Four regions, some overlap to create eight regions. N, NE, SE, S, SW, W, NW
    west = min(XLim);
    east = max(XLim);
    south = max(YLim);
    north = min(YLim);
    
    x = pt(1,1);
    y = pt(1,2);
    x0 = min(XLim);
    y0 = min(YLim);
    
    % Compute the jump distance to be the same as the slider
    slidstep = get(handles.x_slider, 'SliderStep');
    slidstep = slidstep(1);
    range = get(handles.x_slider, 'Max') - get(handles.x_slider, 'Min') +1;
    xjumpstep = range.*slidstep;
    
    slidstep = get(handles.y_slider, 'SliderStep');
    slidstep = slidstep(1);
    range = get(handles.y_slider, 'Max') - get(handles.y_slider, 'Min') +1;
    yjumpstep = range.*slidstep;
    
    if east < x
        show_img(x0+xjumpstep, y0, handles);
    elseif west > x
        show_img(x0-xjumpstep, y0, handles);
    end
    if north > y
        show_img(x0, y0-yjumpstep, handles);
    elseif south < y
        show_img(x0, y0+yjumpstep, handles);
    end
    
%     if max(XLim) < pt(1,1) | min(XLim) > pt(1,1) | ... 
%        min(YLim) > pt(1,2) | max(YLim) < pt(1,2)
%         delete(line_handle)
%         end_line_mode(handles)
%     else  
        if isMax(handles.center_check)
            pt(1,2) = get_mid_y(pt(1,1),pt(1,2),handles);
        end
        xdata = get(line_handle, 'XData');  ydata = get(line_handle, 'YData');
        xdata = [xdata(1), round(pt(1,1))];        ydata = [ydata(1), pt(1,2)];
        set(line_handle, 'XData', xdata, 'YData', ydata)
%     end   
    
% --------------------------------------------------------------------
% ISTRACEONCURRENTLINE - True if current line has a trace_line.
function bool = isTraceOnCurrentLine(lineNum, handles)
    bool = ~isempty(findobj(handles.big_axes,'Tag','trace_line','UserData',lineNum));
    
% --------------------------------------------------------------------   
% Name: FINALIZE_LINE Splines through a spline region.
function finalize_line(tag, handles)
    % --Initialize line_handle, xdata, ydata w/ error checks---------
    [TRACE_HIGHLITE, PT_HIGHLITE, TRACE_BACKGND, PT_BACKGND] = COLORS;
    pt = get(handles.big_axes, 'CurrentPoint'); 
    x = round(pt(1,1));     y = pt(1,2);
    
    line_handle = findobj(handles.big_axes, 'Tag', tag);
    if length(line_handle) ~= 1
        error('Corrupted active control line handles'), 
        delete(line_handle), return
    end
    xdata = get(line_handle,'XData');   ydata = get(line_handle,'YData');
    if length(xdata) ~= 2 | length(ydata) ~= 2
        % No line created, fix situation by using current pt.
        if length(xdata) == 1 & length(ydata) == 1
            set_new_point(xdata,ydata, get(line_handle,'UserData'), handles, []);
            delete(line_handle)
            end_line_mode(handles)  
            
            % Need to check if user wanted the line number to increment
            if isMax(handles.line_num_txt)
                set_line_number(+1, handles)
            end
            return
        else
            delete(line_handle), 
            set(handles.figure1,'WindowButtonMotionFcn',[])
            button_Callback(handles.set_pt_button,[],handles)
            error('Expected 2 points, found only single point.'), 
            return
        end
    elseif xdata(2) ~= x | ydata(2) ~= y
        xdata(2) = x; ydata(2) = get_mid_y(x,y,handles);
        set(line_handle,'XData', xdata, 'YData', ydata)
    end
    % --End intialization-------------------------------------------

    if is_white_px(x,y,handles) & xdata(1)~=xdata(2)
    % Check that it doesn't intersect anything else & only touch single data points.
        TouchPts = find_CtrlPts(min(xdata):max(xdata),handles);
        
        % Stick in overlapping points and delete the control points from existence
        Xtemp = get(TouchPts(:),{'XData'}); Ytemp = get(TouchPts(:),{'YData'});
        Xtemp2 = [Xtemp{:}];                Ytemp2 = [Ytemp{:}];

        new_xdata = sort([Xtemp2 get(line_handle,'XData')]);
        traceInfo = get(findobj(handles.big_axes,'Tag','trace_line','UserData',getLineNum(handles)));
        
        if ~isempty(traceInfo)
            [traceInfo.XData, traceInfo.YData] = sortAndFillGaps(traceInfo.XData, traceInfo.YData);
        end
        make_undoable('new ctrl pt', {line_handle TouchPts new_xdata traceInfo}, handles);
        
        update_ctrl_line(Xtemp2,Ytemp2,line_handle)
        delete(TouchPts);
    
        lineNum = getLineNum(handles);   % current line number
        if isTraceOnCurrentLine(lineNum, handles)
            try
                interpolate(line_handle, lineNum , handles, TRACE_HIGHLITE)
            catch
                undo(handles)  
                error(lasterr)
            end
        end
        set(line_handle,'Tag','CtrlPt','EraseMode','normal' ...,'Marker','.'
            ,'LineWidth',1 ,'MarkerSize',10 ...
            )

        update_ctrlpt_stuff(lineNum, handles)
    else
        delete(line_handle)
    end
    end_line_mode(handles)
    
    
% --------------------------------------------------------------------   
% Name: END_LINE_MODE
% Description: Helper function to do actions necessary to return to
%   normal state.
function end_line_mode(handles)
    set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
    set(handles.figure1,'WindowButtonUpFcn','')


% --------------------------------------------------------------------   
% Name: BUTTON_HELD_DOWN
% Test whether to change mode to line, depending on if user held down mouse.
function retval = button_held_down(figure1, action)
persistent start;

switch action
case 'test'
    TRUE = 1;   FALSE = 0;
    start = clock;      % Start timer
    
    % Conceptually fork off UP and MOTION processes
    oldMotionFcn = get(figure1,'WindowButtonMotionFcn');
    oldUpFcn = get(figure1,'WindowButtonUpFcn');
    set(figure1,'WindowButtonMotionFcn','GUI(''button_held_down'',gcbo,''motion'')',...
                'WindowButtonUpFcn','GUI(''button_held_down'',gcbo,''up'')')

    % Wait for mode to change
    set(figure1,'UserData', 'testing')
    waitfor(figure1, 'UserData')
    
    set(figure1,'WindowButtonUpFcn',oldUpFcn)
    set(figure1,'WindowButtonMotionFcn',oldMotionFcn)

    mode = get(figure1,'UserData');
    if isequal(mode, 'held down')
        retval = TRUE;
    else
        retval = FALSE;
    end
case 'motion'           % Use the mouse motion fcn to simulate a timer
    DELAY = 0.01;
    if isequal(get(figure1,'UserData'),'testing') ...
    & etime(clock,start) > DELAY
       set(figure1,'UserData', 'held down')
    end
case 'up'
    set(figure1,'UserData', 'not held')
end


% ----------------------------------
% SubFunction: TOGGLE_OTHER_TOOLS(stayOn, handles)
function toggle_other_tools(stayOn, handles)
    [TRACE_HIGHLITE, PT_HIGHLITE, TRACE_BACKGND, PT_BACKGND] = COLORS;
    MIN = 0;
    Tools = findobj(handles.figure1,'UserData','tool');
    set(setdiff(Tools,stayOn),'Value', MIN,'Enable','on'...
        ,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'))
    switch get(stayOn,'Tag')
    % Control point operations
    case {'set_pt_button', 'delete_button', 'select_pts_button'}
        set(stayOn, 'BackgroundColor', 'g')        
    % Trace operations
    case {'bad_button'}
        set(stayOn, 'BackgroundColor', 'g') 
    % Navigation operations
    case {'hand_button', 'zoom_button'}
        set(stayOn, 'BackgroundColor', BEIGE) % Same as auto scroll's color
    % Default do not change color of a button
    end
    
    % DeSelect any selected points
    Obj = findobj(handles.big_axes,'Selected','on');
    set(Obj,'Selected','off')

    % Make sure mouse cursor changes when tool switched
    set(handles.figure1,'UserData',[])


%%% ----------------------------------
%%%  Sub-function - WHERE_AM_I
%%%     returns the handle of the current axes or figure1.
function location = Where_am_I(handles)
    axes = handles.big_axes;
    pt = get(axes, 'CurrentPoint');
    
    XLim = get(axes, 'XLim');
    YLim = get(axes, 'YLim');
    x = pt(1,1); y = pt(1,2);
    if x>=XLim(1) & x<=XLim(2) & y>=YLim(1) & y<=YLim(2)
        % In big axes, check whether to return edge line or big_axes.
        [leftEdge, rightEdge] = getEdgeLines(handles);
        
        ZOOM = getZoom(handles);;
        STRETCH = getStretch(handles);
        if ~isempty(leftEdge) & ~isempty(rightEdge)
            RANGE = 2/ZOOM/STRETCH(2);       % Pad around edge lines to select it.
            XLim = get(leftEdge,'XData');
            YLim = abs(get(leftEdge,'YData')-y);
            idx = find(YLim == min(YLim));
            % Test if we're on the left Edge
            if XLim(idx)-RANGE < pt(1,1) & pt(1,1) < XLim(idx)+RANGE
                location = leftEdge;    return
            end
            XLim = get(rightEdge,'XData');
            YLim = abs(get(leftEdge,'YData')-y);
            idx = find(YLim == min(YLim));
            % Test if we're on the right Edge
            if XLim(idx)-RANGE < pt(1,1) & pt(1,1) < XLim(idx)+RANGE
                location = rightEdge;   return
            end
        end
        location = axes;    return
    end
    
    axes = handles.small_axes;
    pt = get(axes, 'CurrentPoint');
    XLim = get(axes, 'XLim');
    YLim = get(axes, 'YLim');
    x = pt(1,1); y = pt(1,2);
    if x>=XLim(1) & x<=XLim(2) & y>=YLim(1) & y<=YLim(2)
        location = axes; return
    end
    
    % If we reach this point without returning, we're not over
    % any of the axes. Return the figure
    location = handles.figure1;
    
% --------------------------------------
% SMALL_AXES_CLICK
function small_axes_click(handles)
    pt = get(handles.small_axes, 'CurrentPoint'); 
    x = pt(1,1);     y = pt(1,2);
    center_view_on(x,y,[],[],handles)
  
% ------------------------------------------------------------------
% Name: RESET_PREVLOC - resets the previous location in the
% figure so that the window motion function will know to update
% the mouse cursor.
function reset_prevLoc(handles)
    set(handles.figure1,'UserData',[])
    
% ------------------------------------------------------------------
% Name: _GENERIC_MOTION_FCN(handles)
% varargin - not empty means explicitly check state, do not skip out early
function generic_motion_fcn(handles, varargin)
    curLoc = Where_am_I(handles);
    
    % Code to make quit if nothing has changed to convserve CPU usage
    prevLoc = get(handles.figure1,'UserData');
    if ~isempty(prevLoc) & ishandle(prevLoc) & curLoc == prevLoc ...
    & isempty(varargin)
        return
    else
        set(handles.figure1,'UserData',curLoc)
    end

    % See what we're hovering over
    switch get(curLoc,'Tag')
    case {'left_edge', 'right_edge'}
        % Set to mover, even if in rotate region. Cursor changed to rotate
        % when user clicks.
        set_custom_pointer('mover', handles)
    case {'big_axes', 'left_edge', 'right_edge'}
        curTool = getCurrentTool(handles);
        
        % Identify tool
        if isempty(curTool)
            toolTag = 'none';
        else
            toolTag = get(curTool,'Tag');
        end
        
        switch toolTag
        case 'select_pts_button'
            type = 'default';
        case 'set_pt_button',
            % Centering checkbox on
            if isMax(handles.center_check)
                type = 'crosshair'; %'circle';
            else type = 'circle'; %'crosshair';
            end
        case 'zoom_button',             type = 'magnify';
        case 'mv_edge_button',          
            type = 'mover';
        case 'bad_button',              type = 'eraser';
        case 'hand_button',             type = 'hand';
        case 'delete_button',           type = 'X';
            
%% Erode and Dilate feature is currently not available
%         case 'erode_button',            type = 'default';
%         case 'dilate_button',           type = 'default';
        otherwise                       type = 'default';
        end

        % Change icon
        if isempty(type)
            set(handles.figure1, 'Pointer', 'default')
        else
            switch type
            % CASE 1: Use standard cursor
            case {'crosshair', 'arrow', 'watch', 'topl', ...
                  'topr', 'botl', 'botr', 'circle', 'cross', ...
                  'fleur', 'left', 'right', 'top', 'bottom', ...
                  'fullcrosshair', 'ibeam', 'default'}
                set(handles.figure1, 'Pointer',type)
            % CASE 2: Use custom cursor
            otherwise
                set_custom_pointer(type, handles)
            end
        end
    otherwise
        set(handles.figure1,'Pointer','default')
    end
 
% --------------------------------------------------------------------
% Name: _GENERIC_DOWN_FCN(handles)
function generic_down_fcn(handles)
curLoc = Where_am_I(handles);

invalid = [findobj(handles.big_axes,'Tag', 'activeCtrlLine') ...
           findobj(handles.big_axes,'Tag','activeLine')];
   
% Delete any invalid objects and return so user can try again
if ~isempty(invalid)
    delete(invalid)
    return
end

switch get(curLoc,'Tag')            % 1 LOCATION IN WINDOW
case {'big_axes', 'left_edge', 'right_edge'}
    button = getMouseButton(handles);
    curTool = getCurrentTool(handles);
	if isempty(curTool)
	    toolTag = 'none';
	else toolTag = get(curTool,'Tag');
	end
            
    % Do nothing if no buttons selected
    switch button                   % 2 MOUSE BUTTON
    case 'normal' % LEFT
        switch get(curLoc,'Tag')
        case 'big_axes'

            switch toolTag   % 3 WHICH TOOL
            case 'select_pts_button', select_pts_down_fcn(handles)
            case 'set_pt_button',   set_pt_down_fcn(handles)
            case 'bad_button',      bad_down_fcn(handles)
            case 'delete_button',   delete_down_fcn(handles)
            case 'hand_button',     hand_down_fcn(handles)
            case 'zoom_button'
                increment = 1;
                [x,y] = getMouseClk(handles.big_axes);
                ZOOM = set_zoom_txt(increment, handles);
                
                % Duplicate call to correct for x,y coordinate clicked on.
                center_view_on(x,y,ZOOM,[],handles)
                
%% Erode and Dilate feature is currently not available.
%             case 'erode_button',     erode_down_fcn(handles)
%             case 'dilate_button',  ;

            otherwise
                % Do Nothing
            end
        case {'left_edge', 'right_edge'}
            mv_edge_down_fcn(handles)
        otherwise % Do nothing
        end
    case 'extend' % MIDDLE
        switch toolTag
        case 'set_pt_button',   
            toggle(handles.center_check);
            set_pt_down_fcn(handles)
            toggle(handles.center_check);
        case 'zoom_button'
            % No change in zoom
            [x,y] = getMouseClk(handles.big_axes);
            ZOOM = getZoom(handles);
            center_view_on(x,y,ZOOM,[],handles)
        otherwise % Do nothing
        end
    case 'alt'    % RIGHT
        [x,y] = getMouseClk(handles.big_axes);
        switch toolTag
        case 'zoom_button'
            increment = -1;
            ZOOM = set_zoom_txt(increment, handles);
            
            % Duplicate call to correct for x,y coordinate clicked on.
            center_view_on(x,y,ZOOM,[],handles)
        otherwise 
            center_view_on(x,y,[],[],handles)
        end    
    case 'open'   % DOUBLE
    otherwise
    end
case {'small_axes'}
    small_axes_click(handles)
case 'figure1'
	lineSelect = select_line_num(handles);  % See if user clicked on a YLabel line number.
	if ~isempty(lineSelect)
        %color = get(handles.big_axes,'YColor');
        %set(handles.big_axes,'YColor','r'), drawnow
        %set(handles.big_axes,'YColor',color)
        setLineNum(lineSelect, handles)
        line_number_txt_Callback(handles.line_number_txt,[],handles)
	end
otherwise
    
end

% --------------------------------------------------------------------
% Name: _SELECT_LINE_NUM_
function y_nums = select_line_num(handles)
	[x,y] = getMouseClk(handles.big_axes);
    y_nums = [];
    AX_EDGE = min(get(handles.big_axes,'XLim'));  % Edge of big axis
    ZOOM = getZoom(handles);;
    STRETCH = getStretch(handles);
    RADIUS_X = 25/ZOOM/STRETCH(2);
    if x < AX_EDGE & x > AX_EDGE-RADIUS_X
        RADIUS_Y = 10/ZOOM/STRETCH(1);
        y_pos = getYTicks(handles);
        select = findInRange(y, y_pos, RADIUS_Y);
        if ~isempty(select)
            y_nums = getYNumbers(handles);
            if length(y_nums) ~= length(y_pos)
                y_nums
                y_pos
                error('Mismatch y num and y pos');
            end
            y_nums = y_nums(y_pos == select(1));
        end
    end

% --------------------------------------------------------------------
% Name: _GETYNUMBERS
function y_nums = getYNumbers(handles)
    y_nums = get(handles.big_axes,'YTickLabel');
    if ~isempty(y_nums) & ~isnumeric(y_nums)
        if y_nums(1,1) == '#'
            y_nums = str2num(y_nums(:, 2:end));
        else y_nums = str2num(y_nums);
        end
    end
% --------------------------------------------------------------------
% Name: _GETYTicks
function yticks = getYTicks(handles)
    yticks = get(handles.big_axes,'YTick');
% --------------------------------------------------------------------
% Name: _FINDINRANGE_(x, vector)
function vector = findInRange(x, vector, range)
    IX = find(abs(x-vector) < range);         % Find matching x
    vector = vector(IX);


% --------------------------------------------------------------------
% Name: _GETMOUSECLK
function [x,y] = getMouseClk(fig_or_axes)
    if ishandle(fig_or_axes)
        pt = get(fig_or_axes,'CurrentPoint');
        if nargout == 2
            x = pt(1,1);
            y = pt(1,2);
        else, x = pt;
        end
    else
        warning('getMouseClk: invalid handle')
        x = []; y=[];
    end


% --------------------------------------------------------------------
% Name: _GETCURRENTTOOL
function curTool = getCurrentTool(handles)
    MAX = get(handles.zoom_button,'Max');
    curTool = findobj(handles.figure1,'UserData','tool','Value',MAX);
    
    % Error check
    if length(curTool) > 1, 
        set(curTool,'Value',0)  % Turn them off, recover from error state
        error('More than one tool toggled'), return
    end
    
% --------------------------------------------------------------------
% Name: _GETMOUSEBUTTON
function mouseButton = getMouseButton(handles)
    %Possible values are:
    %  LEFT = 'normal'; MIDDLE = 'extend'; RIGHT = 'alt'; DOUBLE = 'open';
    mouseButton = get(handles.figure1,'SelectionType');
% --------------------------------------------------------------------
% Name: _BUTTON_Callback
function button_Callback(h, eventdata, handles)
if isempty(h) | ~isequal(get(h,'Value'),get(h,'Max'))
    reset_tools(h, handles)
else
    toggle_other_tools(h, handles)
    set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
    set(handles.figure1,'WindowButtonDownFcn','GUI(''generic_down_fcn'',guidata(gcbo))')
    set(handles.figure1,'WindowButtonUpFcn','')
end

% ----------------------------------
% SubFunction: RESET_TOOLS Reset the left side tools to default state
function reset_tools(button_h, handles)  
    set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
    set(handles.figure1,'WindowButtonDownFcn','GUI(''generic_down_fcn'',guidata(gcbo))')
    set(handles.figure1,'WindowButtonUpFcn','')
    set(button_h, 'BackgroundColor', get(0,'defaultUicontrolBackgroundColor'))
    
%% ZOOM Click-and-drag region currently disabled because it is not in the
%%  manual.  Implementations for each mouse button 
%%  are implemented in generic_down_fcn().
% % --------------------------------------------------------------------
% % Name: ZOOM_DOWN_Callback 
% function zoom_down_fcn(handles)
%     [x,y] = getMouseClk(handles.big_axes);
%     
%     if button_held_down(handles.figure1, 'test')
%         rec = rectangle('Position', [x y 1 1], 'Tag', 'activeBox'...
%                   ,'EdgeColor','r','EraseMode','xor');
%         set(handles.figure1,'WindowButtonMotionFcn','GUI(''draw_box'',''activeBox'',guidata(gcbo))')
%         set(handles.figure1,'WindowButtonUpFcn','GUI(''zoom_up_fcn'',''activeBox'',guidata(gcbo))')
%     else
% 		increment = 1;
% 		[x,y] = getMouseClk(handles.big_axes);
% 		ZOOM = set_zoom_txt(increment, handles);
% 		center_view_on(x,y,ZOOM,[],handles)
%     end
% % - - - - - - - - - - - - - - - - - - - -
% % ZOOM_UP_FCN
% function zoom_up_fcn(tag, handles)
%     rec = findobj(handles.big_axes, 'Tag', tag);
%     
%     % Delete invalid objects and return so user can try again
%     if length(rec) ~= 1
%         delete(rec)
%         set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
%         set(handles.figure1,'WindowButtonUpFcn','')
%         return
%     end
%     Pos = get(rec,'Position');
%     % Set Minimum constraints
%     MIN_PX = 10;
%     Pos(Pos < MIN_PX) = MIN_PX;
%     AxPos = get(handles.big_axes,'Position');
% 
%     % Zoom in temporarily
%     show_img(Pos(1),Pos(2),handles)
%     set(handles.big_axes,'XLim',[Pos(1) Pos(1)+Pos(3)]...
%         ,'YLim',[Pos(2) Pos(2)+Pos(4)]...
%         )
% 
%     delete(rec)
%     set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
%     set(handles.figure1,'WindowButtonUpFcn','')

% --------------------------------------------------------------------
% Name: SET_PT_DOWN_FCN
function varargout = set_pt_down_fcn(handles)
	WHITE = 1;
	[TRACE_HIGHLITE, PT_HIGHLITE, TRACE_BACKGND, PT_BACKGND] = COLORS;
	lineNum = getLineNum(handles);
    [x,y] = getMouseClk(handles.big_axes);
    x = round(x);
	% - - - - - - - - - - - - - - - - - - - -
    
    line_handle = find_CtrlPts(x, handles);
    isInterpZone = ~isempty(line_handle);
    is_white = is_white_px(x,y,handles);
    
    if is_white & is_new_pt(x,y,handles)
        if ~isInterpZone
            [PLOT_STYLE,y,Size ]= adjust_values_for_center_point(x,y,handles);
            plot_handle = plotCtrlPt(x,y, PLOT_STYLE, Size, lineNum, PT_HIGHLITE, handles);
            %drawnow
            if ~button_held_down(handles.figure1, 'test')           % Set new point and interpolate
                % state = uisuspend(handles.figure1);   % Very Slow!
                busy(handles.figure1)
                set_new_point(x,y, lineNum, handles, plot_handle);  % Create a new point
                
                % INCREMENT LINE NUMBER
                % Check if user wants to increment after setting a new ctrl point.
                % Also checked in FINALIZE_LINE because we might jump into that function
                % if button held down but there was no time to create a line.
                if isMax(handles.line_num_txt)
                    set(handles.line_num_txt, 'ForegroundColor','r')
                    drawnow
                    set_line_number(+1, handles)
                    set(handles.line_num_txt, 'ForegroundColor','k')
                end
                % uirestore(state)                      % Very Slow!
            else
                start_new_line(x,y, lineNum, handles, plot_handle); % Create new interp range 
            end
        else    % Update range and trace data
            make_undoable('interpolate', {x line_handle}, handles)
            if isMax(handles.center_check)
                y = get_mid_y(x,y,handles);
            end
            update_ctrl_line(x,y,line_handle) 
            if isTraceOnCurrentLine(lineNum, handles)
                interpolate(line_handle, lineNum, handles, TRACE_HIGHLITE)
            end
            update_ctrlpt_stuff(lineNum, handles)
        end
    % Black region, set line of contrl points
    % No need to wait for user to hold down mouse button before drawing line.
    elseif ~is_white
        plot_handle = plot(x,y, 'md-','Parent',handles.big_axes...
        ,'LineWidth',2,'Tag','activeLine','EraseMode','xor');
        set(handles.figure1,'WindowButtonMotionFcn','GUI(''update_line'',''activeLine'',guidata(gcbo))',...
            'WindowButtonUpFcn','GUI(''set_line_of_ctrl_pts'',''activeLine'',guidata(gcbo))')
    end


%----- Subfunction -----
% FInish job of START_NEW_LINE_
% SET_LINE_OF_CTRL_PTS, SET_NEW_LINE_OF
function set_line_of_ctrl_pts(tag, handles)
    pt = get(handles.big_axes, 'CurrentPoint'); 
%     x2 = round(pt(1,1));     y2 = pt(1,2);
    line_handle = findobj(handles.big_axes, 'Tag', tag);
    xi = get(line_handle,'XData');
    yi = get(line_handle,'YData');
    delete(line_handle)
    end_line_mode(handles)
    Img = get_image(handles);
    
    if length(xi) ~= 2 | length(yi) ~= 2
        return  % This could occur if user releases before update_line can get called once.
    end
    
    % If the user ends on a white region, then don't do anything
    %  to avoid some accidents. 
    if Img(round(yi(end)),round(xi(end))) == 1
        endOnWhite = 1;
    else endOnWhite = 0;
    end

    % If less than 40 degree angle, treat as placing CtrlPts on same line
    angle = 40;
    if abs(tan(angle)*diff(yi)) <abs( tan(angle)*diff(xi))
        chgLineNum = 0;
    elseif endOnWhite == 0
        chgLineNum = 1;
    else return % Not same line and ended on white, return to avoid mistakes.
    end
    
    Img = imcrop(Img,[min(xi-1) min(yi-1) abs(diff(xi))+1 abs(diff(yi))+1]);    % Crop
    if versionNumber < 6.1
        Img = dilate(Img, ones(3));
    else
        Img = imdilate(Img, ones(3));
    end
    X_OFFSET = min(xi)-1;
    Y_OFFSET = min(yi)-1;
    xi = xi - X_OFFSET;      % Offset to match cropped image
    yi = yi - Y_OFFSET;
    try
        [CX, CY, Slice] = improfile(Img, xi, yi);
    catch
        disp('improfile error'), return
    end
    if any(Slice)
        dif = diff([0; Slice; 0]);  % Column vector
        flipW = find(dif>0)+1;      % Indices of 0 to 1
        flipB = find(dif<0);        % Indices of 1 to 0
        Mid = round(flipW + (flipB-flipW)./2); % Indices of middle of lines.
        
        % Don't set unless more than one line, just abort
        if chgLineNum & length(Mid) < 2  
            return
        end    
        
        X = CX(Mid);
        Y = CY(Mid);
        lineNum = getLineNum(handles); %Current num.
        if chgLineNum
            make_undoable('line number',lineNum,handles)
        end
        if yi(2) < yi(1)
            incr = -1;
        else incr = +1;
        end
        MAX_NEW_PTS = 10;
        
        plot_handle = [];
        for n=1:length(X)
            if n>MAX_NEW_PTS
                helpdlg(['Maximum line of new points is ' num2str(MAX_NEW_PTS) ],'Help')
                break
            end
            if chgLineNum
                lineNum = lineNum + incr;
                setLineNum(lineNum,handles)
            end
            x = X(n)+X_OFFSET;
            y = Y(n)+Y_OFFSET;
            if is_white_px(x,y,handles) & is_new_pt(x,y,handles)
%                 plot_handle(n) = set_new_point(x,y, lineNum, handles, [], 'no');
                plot_handle(n) = set_new_point(x,y, lineNum, handles, []);
            end
        end
%         LinNums = get(plot_handle,'UserData');   
%         LinNums = [LinNums{:}];
%         xd
%         make_undoable('line of ctrlpts', plot_handle);
        line_number_txt_Callback(handles.line_number_txt,[],handles)
    end


% --------------------------------------------------------------------
% Name: BAD_DOWN_Callback 
function bad_down_fcn(handles)
    [x,y] = getMouseClk(handles.big_axes);
    rec = rectangle('Position', [x y 1 1], 'Tag', 'activeBox'...
              ,'EdgeColor','r','EraseMode','xor');
    set(handles.figure1,'WindowButtonMotionFcn','GUI(''draw_box'',''activeBox'',guidata(gcbo))')
    set(handles.figure1,'WindowButtonUpFcn','GUI(''bad_up_fcn'',''activeBox'',guidata(gcbo))')

% - - - - - - - - - - - - - - - - - - - -
% BAD_UP_FCN
function bad_up_fcn(tag, handles)
    rec = findobj(handles.big_axes, 'Tag', tag);
    if length(rec) ~= 1
        %disp(['Corrupted box handle: tag = ' tag])
        delete(rec)
        set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
        set(handles.figure1,'WindowButtonUpFcn','')
        return
    end
    Pos = get(rec,'Position');
    pt(1) = Pos(1) + Pos(3) -1; % Far right
    pt(2) = Pos(2) + Pos(4) -1; % Far bottom
    lineNum = getLineNum(handles);

    if ~isMax(handles.any_line)
        traceLine = findobj(handles.big_axes,'Tag','trace_line');
    else
        traceLine = findobj(handles.big_axes,'Tag','trace_line','UserData',lineNum);
    end
    
    % Data to make undoable
    traceInfo = get(traceLine); % Trace line datas
    changesMade = 0;            % See if any changes actually made
    
    for n = 1:length(traceLine)
        xdata = get(traceLine(n),'XData');  
        ydata = get(traceLine(n),'YData');
        IX = find(xdata >= Pos(1) & xdata <= pt(1));
        DO_NOT_DELETE_OUTSIDE_OF_BOX = 1;    % Do we delete all things in range or just in box?
        if ~isMax(handles.any_line) | DO_NOT_DELETE_OUTSIDE_OF_BOX
            IY = find(ydata >= Pos(2) & ydata <= pt(2));
            Idx = intersect(IX,IY);
        else Idx = IX;
        end
        if ~isempty(Idx)
            changesMade = 1;
            traceInfo(n).XData = xdata(Idx);    % Save only parts that are changed
            traceInfo(n).YData = ydata(Idx);    % Save only parts that get changed

            ydata(Idx) = NaN;
            if all(isnan(ydata))    % No more real data left, get rid of line
                delete(traceLine(n))   %  because user can't see it.
            else
                % Delete erased fields
                xdata = xdata(~isnan(ydata));
                ydata = ydata(~isnan(ydata));
                set(traceLine(n),'YData',ydata, 'XData',xdata)
            end
        end
    end
    delete(rec)

    % Make undoable only when changes have been made
    if changesMade
        make_undoable('digitize', {traceInfo, []}, handles)  % Make undoable using same code as digitize, both modify parts of the trace
    end

    set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
    set(handles.figure1,'WindowButtonUpFcn','')


% -----------------
% ISMAX
function bool = isMax(h)
    bool = isequal(get(h,'Value'), get(h,'Max'));
% --------------------
% ISCHECKED
function bool = isChecked(h)
    bool = isequal(get(h,'Checked'),'on');
    
% --------------------------------------------------------------------
% Name: HAND_DOWN_FCN
function hand_down_fcn(handles)
    [x,y] = getMouseClk(handles.figure1);
    set(handles.figure1,'WindowButtonMotionFcn',...
        ['GUI(''hand_motion_fcn'',' num2str(x) ',' num2str(y) ','...
            num2str(mean(get(handles.big_axes,'XLim'))) ',' ...
            num2str(mean(get(handles.big_axes,'YLim'))) ',guidata(gcbo))'])
    set(handles.figure1,'WindowButtonUpFcn','GUI(''hand_up_fcn'',guidata(gcbo))') 
% --------------------------------------------------------------------
% HAND_MOTION
function hand_motion_fcn(x_click, y_click, x_ref, y_ref, handles)
    pt = get(handles.figure1, 'CurrentPoint'); 
    x = pt(1,1);     y = pt(1,2);
    siz = size(get(handles.img_handle,'CData'));
    ZOOM = getZoom(handles);;
    STRETCH = getStretch(handles);  
    center_view_on(mean(x_ref+(x_click-x)./ZOOM./STRETCH(2)),mean(y_ref+(y-y_click)./ZOOM./STRETCH(1)),ZOOM,STRETCH,handles)
% --------------------------------------------------------------------
% HAND_UP
function hand_up_fcn(handles)
    set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
    set(handles.figure1,'WindowButtonUpFcn','')
  
% --------------------------------------------------------------------
% Name: GETEDGELINES
% First time create the lines
% Test Edge Lines - return all empty if edge line handles any invalid
function [BigLeft, BigRight, SmLeft, SmRight] = getEdgeLines(handles)
    % Check if fields initialized yet
    if ~isfield(handles, 'big_left_edge') ...
    |  ~isfield(handles, 'big_right_edge') ...
    |  ~isfield(handles, 'sm_left_edge') ...
    |  ~isfield(handles, 'sm_right_edge') ...
    |  isempty(handles.big_left_edge) ...
    |  isempty(handles.big_right_edge) ...
    |  isempty(handles.sm_left_edge) ...
    |  isempty(handles.sm_right_edge) ...
    |  ~ishandle(handles.big_left_edge) ...
    |  ~ishandle(handles.big_right_edge) ...
    |  ~ishandle(handles.sm_left_edge) ...
    |  ~ishandle(handles.sm_right_edge) ...

        handles.big_left_edge = findobj(handles.big_axes,'Tag','left_edge');
        handles.big_right_edge = findobj(handles.big_axes,'Tag','right_edge');
        handles.sm_left_edge = findobj(handles.big_axes,'Tag','left_edge');
        handles.sm_right_edge = findobj(handles.big_axes,'Tag','right_edge');
        
        guidata(handles.figure1, handles);
    end
    BigLeft = handles.big_left_edge;
    BigRight = handles.big_right_edge;
    SmLeft = handles.sm_left_edge;
    SmRight = handles.sm_right_edge;

    
% --------------------------------------------------------------------
% Name: CHECKIF_EDGE_HANDLESVALID
% Returns 
function bool = checkIfEdgeHandlesInvalid(BigLeft, BigRight, SmLeft, SmRight)
    bool = 0;
    if length(BigLeft) ~= 1 | length(SmLeft) ~= 1 ...
     | length(BigRight) ~= 1 | length(SmRight) ~= 1
        delete([BigLeft SmLeft BigRight SmRight])   % Remove bad handles
        bool = 1;
    end
    
% --------------------------------------------------------------------
% Name: INIT_EDGE_LINES
% Create
function init_edge_lines(handles)
    [BigLeft, BigRight, SmLeft, SmRight] = getEdgeLines(handles); % Return all empty if any bad edge
    
    % First time create the lines?
    if checkIfEdgeHandlesInvalid(BigLeft, BigRight, SmLeft, SmRight) == 1
        delete(findobj(handles.figure1,'Tag','left_edge'))
        delete(findobj(handles.figure1,'Tag','right_edge'))
        
        % Create the edge lines
        yy = 0 : handles.line_spacing : handles.img_siz(1);
        yy = union(yy, handles.img_siz(1));                     % Add on last point if needed
        xL = ceil(handles.img_siz(2)*.01) .*ones(size(yy));
        xR = floor(handles.img_siz(2)*.99) .*ones(size(yy));
        E1 = plot(xL, yy, 'g>-','Parent',handles.big_axes...
             ,'Tag','left_edge','LineWidth',3,'MarkerSize',4,'UserData',0);
        E2 = plot(xR, yy, 'g<-','Parent',handles.big_axes...
             ,'Tag','right_edge','LineWidth',3,'MarkerSize',4,'UserData',0);
        % Copy edge lines
        e1 = copyobj(E1, handles.small_axes);
        e2 = copyobj(E2, handles.small_axes);
        set([e1 e2],'Marker','none','LineWidth',1)
        
        handles.big_left_edge = E1;
        handles.big_right_edge = E2;
        handles.sm_left_edge = e1;
        handles.sm_right_edge = e2;
    end

% --------------------------------------------------------------------
% Name: MV_EDGE_DOWN_FCN
function mv_edge_down_fcn(handles)
    curobj = Where_am_I(handles);
    tag = get(curobj,'Tag');
    [x_ref,y_ref] = getMouseClk(handles.big_axes);
    x = round(mean(get(curobj,'XData')));

    [rightX, leftX] = get_edge_xdata(handles);

    if isChecked(handles.edge_img) & ~is_in_rotate_region(y_ref,handles)
        WrapImg = get(handles.img_handle,'CData');
        switch tag
        case 'left_edge'
            WrapImg = WrapImg(:,fix(rightX-x+2):min(end,fix(rightX)));
            wrapTag = 'left_edge_img';
            temp = image(WrapImg,'Parent',handles.big_axes,'XData',x-size(WrapImg,2),'Tag',wrapTag);
        case 'right_edge'
            WrapImg = WrapImg(:,max(1,fix(leftX)):end-x+fix(leftX));
            wrapTag = 'right_edge_img';
            temp = image(WrapImg,'Parent',handles.big_axes,'XData',x+1,'Tag',wrapTag);
        end
    else 
        wrapTag = '';
        temp = [];
    end
    
    if is_in_rotate_region(y_ref, handles)
        rotate_on = '1';
        set_custom_pointer('rotate',handles)
    else rotate_on = '0';
    end
    
    % Make undoable
    leftEdge = findobj(handles.big_axes,'Tag','left_edge');
    rightEdge = findobj(handles.big_axes,'Tag','right_edge');
    actionData = {get(leftEdge,'XData'), get(leftEdge,'YData') ...
                 ,get(rightEdge,'XData'), get(rightEdge,'YData')};
    make_undoable('move_edge', actionData, handles)
    
    set(handles.figure1,'WindowButtonMotionFcn',['GUI(''mv_edge_motion_fcn'',' num2str(x_ref) ',' num2str(y_ref) ',''' tag ''',''' wrapTag ''',' rotate_on ',guidata(gcbo))'])
    set(handles.figure1,'WindowButtonUpFcn','GUI(''button_Callback'',findobj(gcbo,''UserData'',''tool'',''Value'',1),[],guidata(gcbo))')
    waitfor(handles.figure1,'WindowButtonMotionFcn')        % Wait for button up to change fcn
    delete(temp)            % Executed after user releases mouse button.


% --------------------------------------------------------------------
% Name: IS_IN_ROTATE_REGION
% t_percent - percent of top thats in region
% b_percet - percent of bottom that's in region
function bool = is_in_rotate_region(y_ref,  handles)
    t_percent = 0.15;
    %b_percent = 0.8;
    YLim = get(handles.big_axes,'YLim');
    H = max(YLim) - min(YLim);
    TopLim = min(YLim) + H*t_percent;
    %BotmLim = min(YLim) + H*b_percent;
    if y_ref < TopLim %| y_ref > BotmLim
        bool = 1;
    else bool = 0;
    end
    
% --------------------------------------------------------------------
% Name: MV_EDGE_MOTION_FCN
function mv_edge_motion_fcn(x_ref, y_ref, tag, wrapTag, rotate_on, handles)
if rotate_on
    pt = get(handles.big_axes, 'CurrentPoint'); 
    x = pt(1,1);     y = pt(1,2);
    edge =  findobj(handles.big_axes,'Tag',tag);
    small_edge = findobj(handles.small_axes,'Tag',tag);
    xdata = get(edge,'XData');
    ydata = get(edge,'YData');
    
    slope = (x_ref-x)/(500);    % Side ways slope
    
    % Set new x values
    xdata = x_ref*ones(size(xdata)) + (ydata - y_ref)*slope;
    
    set([edge small_edge],'XData',xdata, 'UserData', slope)%, 'YData', ydata)
else
    % Move the line
    pt = get(handles.big_axes, 'CurrentPoint'); 
    x = round(pt(1,1));     y = pt(1,2);
    edge =  findobj(handles.big_axes,'Tag',tag);
    small_edge = findobj(handles.small_axes,'Tag',tag);
    slope = get(edge,'UserData');
    if isempty(slope)
        slope = 0;
    end
    xdata = get(edge,'XData');
    ydata = get(edge,'YData');
    xdata(:) = x_ref+slope.*(ydata-y_ref)-(x_ref-min(max(1,x),handles.img_siz(2)));
    
    set([edge small_edge],'XData',xdata    )%, 'YData', ydata)

    if ~isempty(wrapTag)
        % Move the wrap image
        WrapImg = findobj(handles.big_axes,'Type','image','Tag',wrapTag);
        switch wrapTag
        case 'left_edge_img'
            set(WrapImg,'XData',x-size(get(WrapImg,'CData'),2),'YData',y-y_ref)
        case 'right_edge_img'
            set(WrapImg,'XData',x+1,'YData',y-y_ref)
        otherwise
        end
    end
end
    
% Update pixels per unit
px_per_inch_txt_Callback(handles.px_per_inch_txt,[],handles)
px_per_minute_txt_Callback(handles.px_per_minute_txt,[],handles)



% --------------------------------------------------------------------
% FIND_PTS_UNDER_CURSOR
function pts = find_pts_under_cursor(x,y,handles)
    if ~isMax(handles.any_line)
        Points = findobj(handles.big_axes,'Tag','CtrlPt');
    else
        Points = findobj(handles.big_axes,'Tag','CtrlPt','UserData',getLineNum(handles));
    end
    RANGE = 4;
    pts = [];
    for n=1:length(Points)
        xp = get(Points(n),'XData');
        yp = get(Points(n),'YData');
        if all(abs(x-xp)<RANGE) & all(abs(y-yp)<RANGE)
            pts = [pts Points(n)];
        end
    end


% --------------------------------------------------------------------
% Name: DELETE_DOWN_Callback
function delete_down_fcn(handles)
    [x,y] = getMouseClk(handles.big_axes); 
    if button_held_down(handles.figure1, 'test')
        rec = rectangle('Position', [x y 1 1], 'Tag', 'activeBox'...
              ,'EdgeColor','g','EraseMode','xor');
        set(handles.figure1,'WindowButtonMotionFcn','GUI(''draw_box'',''activeBox'',guidata(gcbo))',...
            'WindowButtonUpFcn','GUI(''delete_pts_up_fcn'',''activeBox'',guidata(gcbo))')
    else
        pt = get(handles.big_axes, 'CurrentPoint'); 
        x = pt(1,1);     y = pt(1,2);
        obj = find_pts_under_cursor(x,y,handles); %curObj(handles.figure1);
        if isequal('CtrlPt', get(obj,'Tag')) % Is a ctrl pt
            if get(obj,'UserData') == getLineNum(handles) | ... % current line number
            ~isMax(handles.any_line)
                make_undoable('delete',obj,handles)
                lineNum = get(obj, 'UserData');
                delete(obj)
                update_ctrlpt_stuff( lineNum, handles)  % Covers only button not held case
            end
        end
    end


% --------------------------------------------------------------------   
% Name: DELETE_PTS_UP_FCN
function delete_pts_up_fcn(tag, handles)
    rec = findobj(handles.big_axes, 'Tag', tag);
    if length(rec) ~= 1
        %disp(['Corrupted box handle: tag = ' tag])
        delete(rec)
        set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
        set(handles.figure1,'WindowButtonUpFcn','')
        return
    end
    Pos = get(rec,'Position');
    pt(1) = Pos(1) + Pos(3) -1; % Far right
    pt(2) = Pos(2) + Pos(4) -1; % Far bottom
    lineNum = getLineNum(handles);
    
    if ~isMax(handles.any_line)
        CtrlPts = findobj(handles.big_axes,'Tag','CtrlPt');
    else
        CtrlPts = findobj(handles.big_axes,'Tag','CtrlPt','UserData', lineNum);
    end


    xCell = get(CtrlPts(:),{'XData'});
    yCell = get(CtrlPts(:),{'YData'});
    
    SelectedPts = [];
    for n=1:length(xCell)
        xx = xCell{n};  yy = yCell{n};
        if any( xx<pt(1) & xx>Pos(1) )&...
           any( yy<pt(2) & yy>Pos(2) )
            SelectedPts = [SelectedPts CtrlPts(n)];
        end
    end
    if ~isempty(SelectedPts)
        make_undoable('delete', SelectedPts, handles)
        if ~isMax(handles.any_line)
            lineNum = get(SelectedPts(:),{'UserData'});
            lineNum = unique([lineNum{:}]);
        end
        delete(SelectedPts)
        for n=1:length(lineNum)
            update_ctrlpt_stuff(lineNum(n), handles)  % Covers only button held case
        end
    end
    
    delete(rec)
    set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
    set(handles.figure1,'WindowButtonUpFcn','')




% --------------------------------------------------------------------
% SELECT_PTS_DOWN_FCN
function select_pts_down_fcn(handles)
    [x,y] = getMouseClk(handles.big_axes); 
    if button_held_down(handles.figure1, 'test')
        rec = rectangle('Position', [x y 1 1], 'Tag', 'activeBox'...
              ,'EdgeColor','b','EraseMode','xor');
        set(handles.figure1,'WindowButtonMotionFcn','GUI(''draw_box'',''activeBox'',guidata(gcbo))',...
            'WindowButtonUpFcn','GUI(''select_pts_up_fcn'',''activeBox'',guidata(gcbo))')
    else
        obj = get(handles.figure1,'CurrentObject');
        switch get(obj,'Tag')
        case {'CtrlPt'}
            switch get(obj,'Selected')
            case 'off', set(obj, 'Selected', 'on')
            case 'on',  set(obj, 'Selected', 'off')
            end
        otherwise
            Objects = findobj(handles.big_axes,'Selected','on');
            set(Objects,'Selected','off')
        end
    end


% --------------------------------------------------------------------   
% Name: SELECT_PTS_UP_FCN
function select_pts_up_fcn(tag, handles)
    rec = findobj(handles.big_axes, 'Tag', tag);
    if length(rec) ~= 1
        %disp(['Corrupted box handle: tag = ' tag])
        delete(rec)
        set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
        set(handles.figure1,'WindowButtonUpFcn','')
        return
    end
    Pos = get(rec,'Position');
    pt(1) = Pos(1) + Pos(3) -1; % Far right
    pt(2) = Pos(2) + Pos(4) -1; % Far bottom
    
    lineNum = getLineNum(handles);
    if ~isMax(handles.any_line)
        CtrlPts = findobj(handles.big_axes,'Tag','CtrlPt'  );%,'UserData', lineNum);
    else
        CtrlPts = findobj(handles.big_axes,'Tag','CtrlPt', 'UserData', lineNum);
    end
    
    xCell = get(CtrlPts(:),{'XData'});
    yCell = get(CtrlPts(:),{'YData'});
    
    for n=1:length(xCell)
        xx = xCell{n};  yy = yCell{n};
        if any( xx<pt(1) & xx>Pos(1) )&...
           any( yy<pt(2) & yy>Pos(2) )
            set(CtrlPts(n),'Selected','on')
        end
    end
    
    delete(rec)
    set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
    set(handles.figure1,'WindowButtonUpFcn','')


% --------------------------------------------------------------------   
% Name: DRAW_BOX
function draw_box(tag, handles)
    rec = findobj(handles.big_axes, 'Tag', tag);
    if length(rec) ~= 1
        %disp(['Corrupted box handle ' '"' num2str(rec) '"' ': tag = ' tag ])
        delete(rec)
        set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
        set(handles.figure1,'WindowButtonUpFcn','')
        return
    end
    
    pt1 = get(handles.big_axes, 'CurrentPoint');
    pt1 = pt1(1,:);
    pt0 = get(rec, 'UserData');
    
    % First instance, initialize a start position as position of rectangle
    if isempty(pt0)
        Pos = get(rec,'Position');
        pt0 = Pos(1:2);
        set(rec,'UserData',pt0)
    end
    
    for N=1:2
        if pt0(N) < pt1(N)+1
            Pos(N) = pt0(N);
        else
            Pos(N) = pt1(N);
        end
    end

    Pos(3) = abs(pt1(1) - pt0(1));   % Width 
    Pos(4) = abs(pt1(2) - pt0(2));   % Height
    
    % Make sure at least 1
    Pos(3) = max(1, Pos(3));
    Pos(4) = max(1, Pos(4));
    
    set(rec, 'Position', Pos)
    
    
    % Move image if it goes out of axes.
    XLim = get(handles.big_axes,'XLim');
    YLim = get(handles.big_axes,'YLim');
    % Four regions, some overlap to create eight regions. N, NE, SE, S, SW, W, NW
    west = min(XLim);
    east = max(XLim);
    south = max(YLim);
    north = min(YLim);
    x = pt1(1);
    y = pt1(2);
    x0 = min(XLim);
    y0 = min(YLim);
    
    % Compute the jump distance to be the same as the slider
    slidstep = get(handles.x_slider, 'SliderStep');
    slidstep = slidstep(1);
    range = get(handles.x_slider, 'Max') - get(handles.x_slider, 'Min') +1;
    xjumpstep = range.*slidstep;
    
    slidstep = get(handles.y_slider, 'SliderStep');
    slidstep = slidstep(1);
    range = get(handles.y_slider, 'Max') - get(handles.y_slider, 'Min') +1;
    yjumpstep = range.*slidstep;
    
    if east < x
        show_img(x0+xjumpstep, y0, handles);
    elseif west > x
        show_img(x0-xjumpstep, y0, handles);
    end
    if north > y
        show_img(x0, y0-yjumpstep, handles);
    elseif south < y
        show_img(x0, y0+yjumpstep, handles);
    end
    
% --------------------------------------------------------------------
% ANALYZE_IMAGE
function [g_width, line_spacing] = analyze_image(img_handle)

    MAX_PERIOD = 500;                   % Max line spacing
    Window = get(img_handle,'CData');
    
    if max(Window) == 0
        g_width = 0;
        line_spacing = 0;
        return
    end
    
    [m n] = size(Window);
    NUM_SLICES = min(n,16);            % Number of slices to sample
    
    % Crop image to not include empty borders
    L = 0.1; H = 0.8;
    rand('state', 7);       % Set random to same state so we get same result every time.
    n = round(rand(1,NUM_SLICES)*round(n*(H-L))+round(n*L));   % Choose random columns to avoid frequency dependent results.
    m = round(m*L:m*H);                 % Avoid space at top and bottom
    m = m(mod(2,length(m)):end);        % Make length divisible by 2 to avoid prime number to keep fast
    Window = double(Window(m,n));       % Look at image without 10% border around image.

    % Get line spacing
    Colfft = fft(Window);               % Perform FFT along each column.
    Colfft(1,:) = [];                   % Remove DC data.
    Y = mean(Colfft,2);                 % Take average of all samples
    L = floor(length(Y)/2);             % Half-Length
    power = abs(Y(1:L)).^2;
    nyquist = L/length(Y);
    freq = (1:L)/L*nyquist;
    period = 1./freq;
    pidx = find(period<=MAX_PERIOD);
    index = find(power(pidx)==max(power(pidx)));
    
    line_spacing = period(index);
    
    % Method to get result using avg width.
    Pad = zeros(1,size(Window,2));
    Window= [Pad; Window; Pad];         % Pad with zeros 
    [flips, C] = find(diff(Window)~=0); % Index of changes in color
    blockSizes = diff(flips);           % Difference between indices
    blockSizes = blockSizes(1:2:end);   % Grab white blockSizes (evens)
    
    g_width = mean(blockSizes(:));
    
    
% --------------------------------------------------------------------
% SAVE_BOTH_SUBMENU_Callback
function save_both_submenu_Callback(h, eventdata, handles)
    save_img_submenu_Callback(h,eventdata, handles)
    save_pts_submenu_Callback(h,eventdata, handles)
    
% --------------------------------------------------------------------
% SAVE_PTS_SUBMENU_Callback
function varargout = save_pts_submenu_Callback(h, eventdata, handles, varargin)

img_handle = get_img_h(handles);

% Check for corrupt image handle
if length(img_handle) ~= 1
    return
end

fileName = get(handles.filename_txt,'UserData');
Operations = get(img_handle,'UserData');
YLabelInfo = getYLabelInfo(handles);
dataVersion = 1.0;
px_per_minute = getNumOrValue(handles.px_per_minute_txt);
max_period = getNumOrValue(handles.max_period);
dpi = getNumOrValue(handles.px_per_inch_txt);
LineInfo = get(findobj(handles.figure1,'type','line'));
for n=1:length(LineInfo)
    LineInfo(n).ParentTag = get(LineInfo(n).Parent,'Tag');
end

try
    [path, name, ext] = fileparts(fileName);
catch
    warning('fileparts error'), return
end

PREFIX = [];
EXTENSION = '.data';
[saveName savePath] = myputfile([PREFIX name EXTENSION], 'Save Data...', handles, 'data');

if isequal(saveName,0)|isequal(savePath,0)
   return
else
    
    fullSaveName = fullfile(savePath, saveName);
    
    % Save the file, update the queue
    try
        busy(handles.figure1)
        save(fullSaveName...
            ,'fileName' ...
            ,'LineInfo' ...
            ,'YLabelInfo' ...
            ,'Operations' ...
            ,'dataVersion' ...
            ,'px_per_minute' ...
            ,'max_period' ...
            ,'dpi' ...
            ,'-mat')
        idle(handles.figure1)
        disp(['Saved to ' fullSaveName])
        update_queue(savePath, saveName, handles)
    catch
        idle(handles.figure1)
        errordlg(lasterr,'Save file error','modal'), return
    end
end    

% ------------------------------------------------------------------------
% MYPUTFILE - maintains the last directory the user was in.
function [filename, pathname] = myputfile(filterspec, title, handles, varargin)
    if length(varargin) == 0
        fileclass = 'default';
    else
        fileclass = varargin{1};
    end
    
    if ischar(filterspec)
       filterspec = fullfile(myCD(fileclass, handles), filterspec);  % Pre-pend the path
    elseif iscell(filterspec)
        a = {myCD(fileclass,handles), filterspec{:,1}}';
        b = {'Recent', filterspec{:,2}}';
        filterspec = {a{:}; b{:}}';
    end
    [filename, pathname] = uiputfile(filterspec, title);
    if ~isequal(pathname, 0)
        setMyCD(pathname, fileclass, handles)
    end
    
% ------------------------------------------------------------------------
% MYGETFILE - maintains the last directory the user was in.
function [filename, pathname] = mygetfile(filterspec, title, handles, varargin)
    if length(varargin) == 0
        fileclass = 'default';
    else
        fileclass = varargin{1};
    end
    
    if ischar(filterspec)
        filterspec = fullfile(myCD(fileclass,handles), filterspec); % Pre-pend the path
    elseif iscell(filterspec)
        a = {myCD(fileclass,handles), filterspec{:,1}}';
        b = {'Recent', filterspec{:,2}}';
        filterspec = {a{:}; b{:}}';
    end
    [filename, pathname] = uigetfile(filterspec, title);
    if ~isequal(pathname, 0)
        setMyCD(pathname, fileclass, handles)
    end
        
% ------------------------------------------------------------------------
% SETMYCD, MYCD - set and get user's current directory
function setMyCD(path, fileclass, handles)
    if nargin < 3
        error('Not enough parameters to setMyCD')
    end
    fileclass_struct = get(handles.file_menu,'UserData');
    fileclass_struct.(fileclass) = path;
    set(handles.file_menu,'UserData', fileclass_struct)
function path = myCD(fileclass, handles)
    if nargin < 2
        error('Not enough parameters to myCD')
    end
    fileclass_struct = get(handles.file_menu,'UserData');
    
    % If class has been referred to before.
    % fileclass_struct could be empty.
    if isfield(fileclass_struct, fileclass)
        path = fileclass_struct.(fileclass);
    % Just use default without initializing the class type.
    else
        % If no default set, return working directory.
        if ~isfield(fileclass_struct,'default')
            path = pwd;
        % Default exists. Use it.
        else
            path = fileclass_struct.default; 
        end
    end


% ------------------------------------------------------------------------
% _COLORS_ defines constants for plot colors
function [TRACE_HIGHLITE, PT_HIGHLITE, TRACE_BACKGND, PT_BACKGND] = COLORS
    TRACE_HIGHLITE = 'm';
    PT_HIGHLITE = 'r';
    TRACE_BACKGND = [.4 .4 .8];
    PT_BACKGND = 'b';%[.8 .5 .5];
    
% BEIGE color for Tool buttons
function color = BEIGE
    color = [1 0.8 0.5];

% ------------------------------------------------------------------------
% DELETE_TRACE
function  delete_trace(handles)
    [TRACE_HIGHLITE, PT_HIGHLITE, TRACE_BACKGND, PT_BACKGND] = COLORS;
    lineNumber = getLineNum(handles);
    traceLineH = findobj(handles.big_axes,'Tag','trace_line','UserData',lineNumber);
    make_undoable('delete trace',traceLineH,handles);
    
    if length(traceLineH) > 1
        warning('Programmer error: there was more than one trace with the same number')
    end
    delete(traceLineH)
    set(findobj(handles.big_axes,'Tag','CtrlPt','UserData',lineNumber),'Color',PT_HIGHLITE)
    
    update_smooth_button(handles)

% --------------------------------------------------------------------
% Name: PLOT_GAPS_Callback
function plot_gaps_Callback(h, eventdata, handles)
    [pathname filename ext] = fileparts( get(handles.filename_txt,'UserData') );
    GAP_FILE = ['gaps-' filename '.mat'];
    filename = GAP_FILE;

    % First look for file
    if isempty(dir(GAP_FILE))
        Extensions = getfilterspecs('gaps');
        % Then ask for file
        [filename, pathname] = mygetfile(Extensions, 'Select gap data file to load...', handles, 'gaps');
        if isequal(pathname,0) | isequal(pathname,0)
            return  % User canceled
        end
    end
    plot_gaps(pathname, filename, handles)
    
%-------------------------------------------
% PLOT_GAPS
function plot_gaps(pathname, filename, handles)
    load([pathname filename])
    if ~exist('PX_PER_MINUTE') | ~exist('MINUTES') | ~exist('PIXELS')
	    helpdlg('Not a valid gap file', 'Invalid Gap File')
	    return
	elseif isempty(PX_PER_MINUTE)
	    helpdlg('No pix/minute found.', 'Nothing To Plot')
    	return
	end
    
    oldPX_PER_MINUTE = PX_PER_MINUTE;
    [PX_PER_MINUTE, FEW_MINUTES] = removeGapOutliers(PX_PER_MINUTE, MINUTES, PIXELS);
	
	lines = 1:length(PX_PER_MINUTE);
	fig = figure;
    ax2 = axes;
    ax1 = axes;     % order matters, ax2 goes on top
    h = plot(lines,oldPX_PER_MINUTE,'co', lines,PX_PER_MINUTE, 'b.', 'Parent', ax1);
    legend(ax1, 'All Points', 'No Outliers', 0)
    plot(lines,MINUTES,'m-','Parent', ax2)
    set(ax1,'YAxisLocation','left', 'YColor', 'b', 'XGrid','on','YGrid','on' ...
           ,'XLim', [min(lines) max(lines)], 'Color', 'none')
    ylim = get(ax1,'YLim');
    set(ax1, 'YLim', [max(0,min(ylim)), max(ylim)])
    set(get(ax1,'YLabel'), 'String', 'Pix / Min')
    set(get(ax1,'XLabel'), 'String', 'Line Number')
    set(ax2, 'Color', 'white', 'YAxisLocation', 'right', 'XLim', [min(lines) max(lines)] ...
            ,'YLim', [0 max(MINUTES)*2],'YColor','m','TickLength',[0 0]);
    set(get(ax2,'YLabel'), 'String', 'Min Per Line')
    
        
    % Duplicate code as in COUNT_GAPS_Callback
    ButtonName=questdlg( ...
                strvcat(['Avg Pixel count: ' num2str(nanmean(nanmean(PIXELS)))]...
                       ,['Avg Minute count: ' num2str(nanmean(MINUTES))]...
                       ,['Avg Estimated Pixel/Minute = ' num2str(nanmean(PX_PER_MINUTE))]), ...
                'Set pix/min?', ...
                'Ok','Cancel','Ok');
    if isequal(ButtonName, 'Ok')
        setNum(handles.px_per_minute_txt, nanmean(PX_PER_MINUTE), 5);  % 5 sig fig
        px_per_minute_txt_Callback(handles.px_per_minute_txt,[],handles)
    end
    close(fig)
    
% --------------------------------------------------------------------
% Name: REMOVEGAPOUTLIERS
function [PX_PER_MINUTE, FEW_MINUTES] = removeGapOutliers(PX_PER_MINUTE, MINUTES, PIXELS);
	% Remove outliers
	FEW_MINUTES = 5;
	few_index = find(MINUTES < FEW_MINUTES);
	PX_PER_MINUTE(few_index) = NaN;
	
	UPPER = 475;    LOWER = 450;
	bounds_index = find(PX_PER_MINUTE > UPPER | PX_PER_MINUTE< LOWER);
	PX_PER_MINUTE(bounds_index,:) = NaN;
	
	mu = nanmean(PX_PER_MINUTE);
	sigma = nanstd(PX_PER_MINUTE);
	out_index = find(abs(PX_PER_MINUTE- mu) > sigma);    % Find skewed values
	PX_PER_MINUTE(out_index) = NaN;
    

    
% --------------------------------------------------------------------
% COUNT_GAPS_Callback
function count_gaps_Callback(h, eventdata, handles, varargin)
    lineNumber = getLineNum(handles);
    busy(handles.figure1)

    traceLine_h = findobj(handles.big_axes,'Tag','trace_line','UserData',lineNumber);
    
    if length(traceLine_h) > 1
        errordlg('Error: multiple lines with same line number found. Please clear trace line.'), return
    end
    
    AllPts = findobj(handles.big_axes,'Tag','CtrlPt','UserData',lineNumber);
    AllPts = Sort_the_point_handles(AllPts);

    img_handle = get_img_h(handles);
    if isempty(img_handle) & ~ishandle(img_handle)
        errordlg('Image object (ID=''Img'') not found!', 'Cannot trace', 'modal')
    end
    try
        [px_per_minute, gap_count, px_total] = count_gaps(img_handle,AllPts,handles.g_width,'gap label',getNumOrValue(handles.px_per_inch_txt));
    catch
        errordlg(lasterr, 'Cannot Trace', 'modal')
        return
    end
	ButtonName=questdlg( ...
                strvcat(['Pixel count: ' num2str(px_total)]...
                       ,['Minute count: ' num2str(gap_count)]...
                       ,['Estimated Pixel/Minute = ' num2str(px_per_minute)]), ...
                'Set pix/min?', ...
                'Ok','Cancel','Ok');
    if isequal(ButtonName, 'Ok')
        setNum(handles.px_per_minute_txt, px_per_minute, 5);  % 5 sig fig
        px_per_minute_txt_Callback(handles.px_per_minute_txt,[],handles)
    end
    
    idle(handles.figure1)


% ----------------
% TIME_Callback
function time_Callback(h, eventdata, handles)    
% Grab the pixel per minute value
px_per_minute = getNumOrValue(handles.px_per_minute_txt)


% --------------
% GETLINENUM
function lineNum = getLineNum(handles)
    lineNum = get(handles.line_number_txt,'Value');
    
    % ::DEBUG::
    if ~(get(handles.line_number_txt,'Value') == ...
    str2double(get(handles.line_number_txt,'String')))
        error('line_number_txt Value and String out of sync or NaN.')
    end
% --------------
% SETLINENUM
function setLineNum(lineNum, handles)
    if ~isnumeric(lineNum)
        lineNum = str2num(lineNum);
    end
    set(handles.line_number_txt,'String', num2str(lineNum),'Value', lineNum)

% --------------------------------------------------------------------
% ANY_LINE_Callback
function any_line_Callback(h, eventdata, handles)
    if get(h,'Value') == get(h,'Max')
        set(h,'String','Current', 'TooltipString', 'Modify any line number')
    else set(h,'String','All Lines', 'TooltipString', 'Modify current line number only')
    end
    
% --------------------------------------------------------------------
% CENTER_CHECK_Callback
function center_check_Callback(h, eventdata, handles)
    if get(h,'Value') == get(h,'Max')
        set(h,'String','Center', 'TooltipString','New control points are centered')
    else set(h,'String','Exact', 'TooltipString','New control points are NOT centered')
    end
    reset_prevLoc(handles)  % Indicate to motion function to update cursor
    eval(get(handles.figure1,'WindowButtonMotionFcn'))
    
% --------------------------------------------------------------------
% SET_SLIDER
function value = set_slider(text_h, slider_h)
    value = getNumOrValue(text_h);
    maxN = get(slider_h,'Max');
    minN = get(slider_h,'Min');
    if value > maxN
        value = setNum(h, maxN, 3);
    elseif value < minN
        value = setNum(h, minN, 3);
    end
    set(slider_h,'Value',value)
% --------------------------------------------------------------------
% SET_TEXT
function value = set_text(text_h, slider_h)
    value = get(slider_h,'Value');
    value = setNum(text_h, value, 2);

% --------------------------------------------------------------------
% ASSERT - Function that throws an error if statement is false.
function assert(bool, varargin)
if ~bool
    if nargin > 1
        error(varargin{1})
    else
        error
    end
end
    
% --------------------------------------------------------------------
% DEBUGGING PURPOSES DEBUG_ debugging DEBUGGER 
%  Currently attempts to fix any corrupted states in the GUI.
% --------------------------------------------------------------------
function varargout = debug_box_Callback(h, eventdata, handles)
    turn_buttons('on',handles)
    delete(findobj(handles.figure1,'Tag','display'))
    delete(findobj(handles.figure1,'Tag','highlighting'))
    delete(findobj(handles.figure1,'Tag','normal_line'))
    delete(findobj(handles.figure1,'Tag','activeBox'))
    update_smooth_button(handles)
    
    if ~ishandle(handles.img_handle) | length(handles.img_handle)~=1 ...
    | handles.img_handle ~= findobj(handles.big_axes,'Tag','Img')
        disp('handles.img_handle is not up to date')
    end
    
    YLabelInfo = fixYLabelInfo(handles)
    update_all_ctrlpt_stuff(handles)
    test_current_state(handles)

    set(handles.digitize, 'Value', get(handles.digitize,'Min'), 'String', 'Pt 2 Pt', 'BackgroundColor', [0.8 0.8 0.9], 'ForegroundColor','r')
    set(handles.digitize_all,'Value', get(handles.digitize_all,'Min'),  'String', 'Digitize Files', 'BackgroundColor', [0.8 0.8 0.9], 'ForegroundColor','r')
    set(handles.togglebutton33, 'Value', get(handles.togglebutton33,'Min'), 'String', 'Line', 'BackgroundColor', [0.8 0.8 0.9], 'ForegroundColor','r')

    trace_line = findobj(handles.big_axes,'Tag','trace_line');
    for I=1:length(trace_line)
        xdata = get(trace_line(I),'XData');
        ydata = get(trace_line(I),'YData');
        if length(unique(xdata)) ~= length(xdata)
            % Grabbing unique values only
            [xdata temp] = unique(xdata);
            ydata = ydata(temp);
            disp('Cleaned line')
        end 
        set(trace_line(I),'XData',xdata,'YData',ydata)
    end

    idle(handles.figure1)
    disp('Debug done')
    

% --------------------------------------------------------------------
% GETTRACEDATA
function [X,Y] = getTraceData(lineNum, handles)
    hh = findobj(handles.big_axes,'Tag','trace_line','UserData',lineNum);
    X = get(hh,'XData');
    Y = get(hh,'YData');
    
% --------------------------------------------
% PLOTNORMALLINE
function plotNormalLine(TAG, handles)
busy(handles.figure1)
for N = getUniqueLineNums(handles.big_axes,'trace_line')
    plotNormalLineN(N, 'g-', handles, TAG)
end
idle(handles.figure1)
% -----------------------------------------------
function plotNormalLineN(N, PLOT_STYLE, handles, TAG)
    [xdata ydata] = getTraceData(N, handles);
    if ~isempty(ydata(isfinite(ydata)))
        degree = getPolyfitDegree(handles);  % Degree of the polynomial fit
        normal_line = getNormalLine(xdata, ydata, degree);
        plot(xdata, normal_line, PLOT_STYLE,'Tag',TAG, 'Parent',handles.big_axes)
    end
    
% -----------------------------------------------
function degree = getPolyfitDegree(handles)
    isOn = findobj(handles.degree_123,'flat','Checked','on');
    
    if length(isOn) ~= 1
        error('Invalid polyfit degree chosen.')
    end
    
    degree = round(str2double(get(isOn, 'Label')));
    if degree > 3 | degree < 1
        degree = 3;
        warning(['Invalid degree created (' num2str(degree) '). Setting degree to ' num2str(degree) '.'])
    end

% --------------------------------------------
% SCAN_Callback  _AUTO_SCROLL_ AUTOSCROLL
function scan_Callback(h, eventdata, handles)
if get(h,'Value') == get(h,'Max')
    set(h,'BackgroundColor', BEIGE)
    
    oldYBottom = min(get(handles.big_axes,'YLim'));
    x_lim = get(handles.big_axes,'XLim');
    y_lim = get(handles.big_axes,'YLim');
    xdata = [1:diff(x_lim)/3:handles.img_siz(2)];    % One end to the other end

    uistate = uisuspend(handles.figure1);
    set(handles.figure1,'WindowButtonDownFcn','GUI(''scan_stop'',guidata(gcbo))')

    delay = str2double(get(handles.scan_delay,'String'));
    
    % Initial values in case we never enter loop
    x = min(x_lim);
    y = min(y_lim);
    oldx = x;
    oldy = y;
    
    % Bumper will remain constant even if zoom changes.
    % This should not be a problem because the zoom should not
    % change while scrolling.
    X_BUMPER = 100 / getZoom(handles);  
    
    while get(h,'Value') == get(h,'Max')
        % Delay and wait for click, then save previous values
        % If user clicks during delay, we'll go back to oldx and oldy
        pause(delay)
        oldx = x;
        oldy = y;
        
        x_lim = get(handles.big_axes,'XLim');
        y_lim = get(handles.big_axes,'YLim');
        if max(x_lim) >= handles.img_siz(2)
            if max(y_lim) >= handles.img_siz(1)  % Quit at bottom of image
                set(h,'Value',get(h,'Min'))
                break
            end
            Y_JUMP_DIST = diff(y_lim)*.8;  % Alternatively: max(y_lim)-100-y;

            oldYBottom = max(y_lim);
            y = min(y_lim) + Y_JUMP_DIST;
            x = 1;
            
            pause(1)    % Extra bit of delay at end of a line
        else 
            x = max(x_lim);
            y = min(y_lim);
        end

        show_img(x-X_BUMPER, y, handles)    % Scroll to right
        drawnow
    end
    
    % Go back to previous frame automatically, unless at bottom of image
    if get(h,'Value') ~= get(h,'Max')
        show_img(oldx-X_BUMPER, oldy, handles)
    end
    
    %%%%
%     delete(marker)
    uirestore(uistate)
end
set(h,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor') ...
     ,'Value',get(h,'Min'))

% --------------------------------------------
% SCAN_STOP
function scan_stop(handles)
    set(handles.scan,'Value',get(handles.scan,'Min')), drawnow

% --------------------------------------------------------------------
% Name: ERODE_BUTTON
function erode_button(handles)
    set(handles.figure1,'WindowButtonDownFcn','GUI(''erode_down_fcn'',guidata(gcbo))');
    
% --------------------------------------------------------------------
% Name: ERODE_DOWN_FCN
function erode_down_fcn(handles)
    [x,y] = getMouseClk(handles.big_axes);
    rec = rectangle('Position', [x y 1 1], 'Tag', 'activeBox'...
              ,'EdgeColor','r','EraseMode','xor');
    set(handles.figure1,'WindowButtonMotionFcn','GUI(''draw_box'',''activeBox'',guidata(gcbo))')
    set(handles.figure1,'WindowButtonUpFcn','GUI(''erode_up_fcn'',''activeBox'',guidata(gcbo))')

% --------------------------------------------------------------------
% Name: ERODE_UP_FCN 
% [x y w h]
function erode_up_fcn(tag, handles)
try
    box = findobj(handles.big_axes,'Tag',tag);
    pos = get(box,'Position');
    pos = round(pos);
    [Img img_handle] = get_image(handles);
    C = pos(1):pos(1)+pos(3)-1;
    R = pos(2):pos(2)+pos(4)-1;
    
    window = Img(R,C);
    
    busy(handles.figure1)
    window = myerode(window);

    Img(R,C) = window;
    set(img_handle,'CData',Img)
    idle(handles.figure1)
    delete(box)
    set(handles.figure1,'WindowButtonMotionFcn','GUI(''generic_motion_fcn'',guidata(gcbo))')
    set(handles.figure1,'WindowButtonDownFcn','GUI(''erode_down_fcn'',guidata(gcbo))');
    set(handles.figure1,'WindowButtonUpFcn','')
catch
    delete(box)
    set(handles.figure1,'WindowButtonMotionFcn','')
    set(handles.figure1,'WindowButtonDownFcn','GUI(''erode_down_fcn'',guidata(gcbo))');
    set(handles.figure1,'WindowButtonUpFcn','')
end
    
% --------------------------------------------------------------------
% FINDOBJ_CACHED
function h = findobj_cached(ObjectHandles, Tag, UserData)
    persistent Cache
    
    if ~isnumeric(UserData)
        error('UserData must be numeric to use findobj_cached')
    end

    if isnumeric(Tag)
        Tag = num2str(Tag);
    end
    
    if isfield(Cache, Tag) 
        ObjList = getfield(Cache, Tag);
        
        % Check if UserData in bounds of array
        if length(ObjList(:)) >= UserData
            
            h = ObjList{UserData};  % Get cached result
            
            % Check if cached result isvalid, 0 is uninitialized flag
            if isempty(h) | 0==h | any(~ishandle(h))
                % First Use or Invalid handle
                % use findobj and store result into list
                h = findobj(ObjectHandles, 'Tag', Tag, 'UserData', UserData);
                if ~isempty(h)
                    Cache = setfield(Cache, Tag, {UserData}, {h});
                end
            end
        else
            % First Use, Grow list to include UserData
            % use findobj and store result into list
            h = findobj(ObjectHandles, 'Tag', Tag, 'UserData', UserData);
            if ~isempty(h)
                Cache = setfield(Cache, Tag, {UserData}, {h});
            end
        end
    else
        % First Use of Tag
        % Add new field and set it to a list of length 1
        h = findobj(ObjectHandles, 'Tag', Tag, 'UserData', UserData);
        if ~isempty(h)
            Cache = setfield(Cache, Tag, {h});
        end
    end

    

% --------------------------------------------------------------------
function X = chop(Xin,n,unit)
%CHOP   CHOP(X,n) rounds elements of X to n significant figures.
%       CHOP(X,n,unit) rounds the elements of X to n significant
%   figures whose digits (mantissa) are exactly divisible
%   by unit. 
%       
%       e.g. chop(3.141592,5)   returns 3.141600000..
%       e.g. chop(3.141592,3,5) returns 3.150000000..
%            chop(3.141592,3,3) returns 3.150000000..
%            chop(3.141592,3,2) returns 3.140000000..
%      

%   Copyright 1986-2000 The MathWorks, Inc. 
%   $Revision: 1.75 $  $Date: 2003/02/27 06:44:52 $

% Set last sig. fig. rounding to 1 if only two input arguments.
if nargin<3,
   unit=1; 
end

% Cater for -ve numbers  and numbers = 0.
X = abs(Xin) +(Xin==0);
[nx,mx] = size(X);
exponent = unit.*((10*ones(nx,mx)).^(floor(log10(X))-n+1));
X = round(X./exponent).*exponent;

% Put back sign and zeros
X = sign(Xin).*X.*(Xin~=0);
% end chop


% --------------------------------------------------------------------
% Name: DIST - simple distance function
function d = dist(x1, y1, x2, y2)
    d = sqrt( (x2-x1)^2 + (y2-y1)^2 );

% --------------------------------------------------------------------
% IDLE
function idle(theFigure)
% Idle -- Set the arrow-cursor.
%  Idle(theFigure) sets the arrow-cursor in theFigure.
%   The companion routine is "Busy".
 
% Copyright (C) 1996 Dr. Charles R. Denham, ZYDECO.
% All Rights Reserved.
if ~any(findobj('Type', 'figure')), return, end
if nargin < 1, theFigure = gcf; end
set(theFigure, 'Pointer', 'arrow');

% --------------------------------------------------------------------
% BUSY
function busy(theFigure)
% Busy -- Set the watch-cursor.
%  Busy(theFigure) sets the watch-cursor in theFigure.
%   The companion routine is "Idle".F
 
% Copyright (C) 1996 Dr. Charles R. Denham, ZYDECO.
% All Rights Reserved.
if ~any(findobj('Type', 'figure')), return, end
if nargin < 1
   theFigure = gcf;
end
set(theFigure, 'Pointer', 'watch');

function ButtonName=myquestdlg(Question,Title,Btn1,Btn2,Btn3,Default)
%MYQUESTDLG 
% Modified from Matlab's version to be modal and take in 3 buttons

if nargin<1,error('Too few arguments for QUESTDLG');end

Interpreter='none';
if ~iscell(Question),Question=cellstr(Question);end

if strcmp(Question{1},'#FigKeyPressFcn'),
  QuestFig=get(0,'CurrentFigure');
  AsciiVal= abs(get(QuestFig,'CurrentCharacter'));
  if ~isempty(AsciiVal),
    if AsciiVal==32 | AsciiVal==13,
      set(QuestFig,'UserData',1);
      uiresume(QuestFig);
    end %if AsciiVal
  end %if ~isempty
  return
end
%%%%%%%%%%%%%%%%%%%%%
%%% General Info. %%%
%%%%%%%%%%%%%%%%%%%%%
Black      =[0       0        0      ]/255;
LightGray  =[192     192      192    ]/255;
LightGray2 =[160     160      164    ]/255;
MediumGray =[128     128      128    ]/255;
White      =[255     255      255    ]/255;

%%%%%%%%%%%%%%%%%%%%
%%% Nargin Check %%%
%%%%%%%%%%%%%%%%%%%%
if nargout>1,error('Wrong number of output arguments for QUESTDLG');end
if nargin==1,Title=' ';end
if nargin<=2, Default='Yes';end
if nargin==3, Default=Btn1;end
if nargin<=3, Btn1='Yes'; Btn2='No'; Btn3='Cancel';NumButtons=3;end
if nargin==4, Default=Btn2;Btn2=[];Btn3=[];NumButtons=1;end
if nargin==5, Default=Btn3;Btn3=[];NumButtons=2;end
if nargin==6, NumButtons=3;end
if nargin>6, error('Too many input arguments');NumButtons=3;end

if isstruct(Default),
  Interpreter=Default.Interpreter;
  Default=Default.Default;
end


%%%%%%%%%%%%%%%%%%%%%%%
%%% Create QuestFig %%%
%%%%%%%%%%%%%%%%%%%%%%%
FigPos=get(0,'DefaultFigurePosition');
FigWidth=75;FigHeight=45;
FigPos(3:4)=[FigWidth FigHeight];
QuestFig=dialog(                                               ...
               'Visible'         ,'off'                      , ...
               'Name'            ,Title                      , ...
               'Pointer'         ,'arrow'                    , ...
               'Units'           ,'points'                   , ...
               'Position'        ,FigPos                     , ...
               'KeyPressFcn'     ,'questdlg #FigKeyPressFcn;', ...
               'UserData'        ,0                          , ...
               'IntegerHandle'   ,'off'                      , ...
               'WindowStyle'     ,'modal'                   , ... 
               'HandleVisibility','callback'                 , ...
               'Tag'             ,Title                        ...
               );

%%%%%%%%%%%%%%%%%%%%%
%%% Set Positions %%%
%%%%%%%%%%%%%%%%%%%%%
DefOffset=3;

IconWidth=32;
IconHeight=32;
IconXOffset=DefOffset;
IconYOffset=FigHeight-DefOffset-IconHeight;
IconCMap=[Black;get(QuestFig,'Color')];

DefBtnWidth=40;
BtnHeight=20;
BtnYOffset=DefOffset;
BtnFontSize=get(0,'FactoryUIControlFontSize');
BtnFontName=get(0,'FactoryUIControlFontName');

BtnWidth=DefBtnWidth;

ExtControl=uicontrol(QuestFig   , ...
                     'Style'    ,'pushbutton', ...
                     'String'   ,' '         , ...
                     'FontUnits','points'    , ...                     
                     'FontSize' ,BtnFontSize , ...
                     'FontName' ,BtnFontName   ...
                     );
                     
set(ExtControl,'String',Btn1);
BtnExtent=get(ExtControl,'Extent');
BtnWidth=max(BtnWidth,BtnExtent(3)+8);
if NumButtons > 1
  set(ExtControl,'String',Btn2);
  BtnExtent=get(ExtControl,'Extent');
  BtnWidth=max(BtnWidth,BtnExtent(3)+8);
  if NumButtons > 2
    set(ExtControl,'String',Btn3);
    BtnExtent=get(ExtControl,'Extent');
    BtnWidth=max(BtnWidth,BtnExtent(3)+8);
  end 
end

delete(ExtControl);

MsgTxtXOffset=IconXOffset+IconWidth;

FigWidth=max(FigWidth,MsgTxtXOffset+NumButtons*(BtnWidth+2*DefOffset));
FigPos(3)=FigWidth;
set(QuestFig,'Position',FigPos);

BtnXOffset=zeros(NumButtons,1);

if NumButtons==1,
  BtnXOffset=(FigWidth-BtnWidth)/2;
elseif NumButtons==2,
  BtnXOffset=[MsgTxtXOffset
              FigWidth-DefOffset-BtnWidth];
elseif NumButtons==3,
  BtnXOffset=[MsgTxtXOffset
              0
              FigWidth-DefOffset-BtnWidth];
  BtnXOffset(2)=(BtnXOffset(1)+BtnXOffset(3))/2;
end

MsgTxtYOffset=DefOffset+BtnYOffset+BtnHeight;
MsgTxtWidth=FigWidth-DefOffset-MsgTxtXOffset-IconWidth;
MsgTxtHeight=FigHeight-DefOffset-MsgTxtYOffset;
MsgTxtForeClr=Black;
MsgTxtBackClr=get(QuestFig,'Color');

CBString='uiresume(gcf)';
ButtonString=Btn1;
ButtonTag='Btn1';
BtnHandle(1)=uicontrol(QuestFig            , ...
			'Style'              ,'pushbutton', ...
			'Units'              ,'points'    , ...
			'Position'           ,[ BtnXOffset(1) BtnYOffset  ...
		                                BtnWidth       BtnHeight   ...
		                              ]           , ...
			'CallBack'           ,CBString    , ...
			'String'             ,ButtonString, ...
			'HorizontalAlignment','center'    , ...
			'FontUnits'          ,'points'    , ...
			'FontSize'           ,BtnFontSize , ...
			'FontName'           ,BtnFontName , ...
			'Tag'                ,ButtonTag     ...
			);
if strcmp(ButtonString, Default)
    set(BtnHandle(1),'FontWeight','bold')
end

if NumButtons > 1
  ButtonString=Btn2;
  ButtonTag='Btn2';
  BtnHandle(2)=uicontrol(QuestFig            , ...
			  'Style'              ,'pushbutton', ...
			  'Units'              ,'points'    , ...
			  'Position'           ,[ BtnXOffset(2) BtnYOffset  ...
		                                  BtnWidth       BtnHeight   ...
		                                ]           , ...
			  'CallBack'           ,CBString    , ...
			  'String'             ,ButtonString, ...
			  'HorizontalAlignment','center'    , ...
			  'FontUnits'          ,'points'    , ...
			  'FontSize'           ,BtnFontSize , ...
			  'FontName'           ,BtnFontName , ...
			  'Tag'                ,ButtonTag     ...
			  );
  if strcmp(ButtonString, Default)
      set(BtnHandle(2),'FontWeight','bold')
  end

  if NumButtons > 2
    ButtonString=Btn3;
    ButtonTag='Btn3';
    BtnHandle(3)=uicontrol(QuestFig            , ...
			    'Style'              ,'pushbutton', ...
			    'Units'              ,'points'    , ...
			    'Position'           ,[ BtnXOffset(3) BtnYOffset  ...
                                		    BtnWidth       BtnHeight   ...
		                                  ]           , ...
			    'CallBack'           ,CBString    , ...
			    'String'             ,ButtonString, ...
			    'HorizontalAlignment','center'    , ...
			    'FontUnits'          ,'points'    , ...
			    'FontSize'           ,BtnFontSize , ...
                            'FontName'           ,BtnFontName , ...
			    'Tag'                ,ButtonTag     ...
			    );
    if strcmp(ButtonString, Default)
        set(BtnHandle(3),'FontWeight','bold')
    end
  end
end
MsgHandle=uicontrol(QuestFig            , ...
                   'Style'              ,'text'         , ...
                   'Units'              ,'points'       , ...
                   'Position'           ,[MsgTxtXOffset      ...
                                          MsgTxtYOffset      ...
                                          0.95*MsgTxtWidth   ...
                                          MsgTxtHeight       ...
                                         ]              , ...
                   'String'             ,{' '}          , ...
                   'Tag'                ,'Question'     , ...
                   'HorizontalAlignment','left'         , ...    
                   'FontUnits'          ,'points'       , ...
                   'FontWeight'         ,'bold'         , ...
                   'FontSize'           ,BtnFontSize    , ...
                   'FontName'           ,BtnFontName    , ...
                   'BackgroundColor'    ,MsgTxtBackClr  , ...
                   'ForegroundColor'    ,MsgTxtForeClr    ...
                   );

[WrapString,NewMsgTxtPos]=textwrap(MsgHandle,Question,75);

NumLines=size(WrapString,1);

% The +2 is to add some slop for the border of the control.
MsgTxtWidth=max(MsgTxtWidth,NewMsgTxtPos(3)+2);
MsgTxtHeight=NewMsgTxtPos(4)+2;

MsgTxtXOffset=IconXOffset+IconWidth+DefOffset;
FigWidth=max(NumButtons*(BtnWidth+DefOffset)+DefOffset, ...
             MsgTxtXOffset+MsgTxtWidth+DefOffset);

        
% Center Vertically around icon  
if IconHeight>MsgTxtHeight,
  IconYOffset=BtnYOffset+BtnHeight+DefOffset;
  MsgTxtYOffset=IconYOffset+(IconHeight-MsgTxtHeight)/2;
  FigHeight=IconYOffset+IconHeight+DefOffset;    
% center around text    
else,
  MsgTxtYOffset=BtnYOffset+BtnHeight+DefOffset;
  IconYOffset=MsgTxtYOffset+(MsgTxtHeight-IconHeight)/2;
  FigHeight=MsgTxtYOffset+MsgTxtHeight+DefOffset;    
end    

if NumButtons==1,
  BtnXOffset=(FigWidth-BtnWidth)/2;
elseif NumButtons==2,
  BtnXOffset=[(FigWidth-DefOffset)/2-BtnWidth
              (FigWidth+DefOffset)/2      
              ];
          
elseif NumButtons==3,
  BtnXOffset(2)=(FigWidth-BtnWidth)/2;
  BtnXOffset=[BtnXOffset(2)-DefOffset-BtnWidth
              BtnXOffset(2)
              BtnXOffset(2)+BtnWidth+DefOffset
             ];              
end

ScreenUnits=get(0,'Units');
set(0,'Units','points');
ScreenSize=get(0,'ScreenSize');
set(0,'Units',ScreenUnits);

FigPos(1)=(ScreenSize(3)-FigWidth)/2;
FigPos(2)=(ScreenSize(4)-FigHeight)/2;
FigPos(3:4)=[FigWidth FigHeight];

set(QuestFig ,'Position',FigPos);

BtnPos=get(BtnHandle,{'Position'});BtnPos=cat(1,BtnPos{:});
BtnPos(:,1)=BtnXOffset;
BtnPos=num2cell(BtnPos,2);  
set(BtnHandle,{'Position'},BtnPos);  

delete(MsgHandle);
AxesHandle=axes('Parent',QuestFig,'Position',[0 0 1 1],'Visible','off');

MsgHandle=text( ...
    'Parent'              ,AxesHandle                      , ...
    'Units'               ,'points'                        , ...
    'FontUnits'           ,'points'                        , ...
    'FontSize'            ,BtnFontSize                     , ...
    'FontName'            ,BtnFontName                     , ...
    'HorizontalAlignment' ,'left'                          , ...
    'VerticalAlignment'   ,'bottom'                        , ...
    'HandleVisibility'    ,'callback'                      , ...
    'Position'            ,[MsgTxtXOffset MsgTxtYOffset 0] , ...
    'String'              ,WrapString                      , ...
    'Interpreter'         ,Interpreter                     , ...
    'Tag'                 ,'Question'                        ...
    );

IconAxes=axes(                                      ...
             'Units'       ,'points'              , ...
             'Parent'      ,QuestFig              , ...  
             'Position'    ,[IconXOffset IconYOffset  ...
                             IconWidth IconHeight], ...
             'NextPlot'    ,'replace'             , ...
             'Tag'         ,'IconAxes'              ...
             );         
 
set(QuestFig ,'NextPlot','add');

IconData= ...
[2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2;
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2;
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2;
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2;
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 0 0 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 2 2 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 0 0 0 2 2 2 2 0 0 0 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 0 0 0 2 2 2 2 0 0 0 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 0 0 0 0 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2; 
 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2];

Img=image('CData',IconData,'Parent',IconAxes);
set(QuestFig, 'Colormap', IconCMap);
set(IconAxes, ...
   'Visible','off'           , ...
   'YDir'   ,'reverse'       , ...
   'XLim'   ,get(Img,'XData'), ...
   'YLim'   ,get(Img,'YData')  ...
   );
set(findobj(QuestFig),'HandleVisibility','callback');
set(QuestFig ,'WindowStyle','modal','Visible','on');
drawnow;

uiwait(QuestFig);

TempHide=get(0,'ShowHiddenHandles');
set(0,'ShowHiddenHandles','on');

if any(get(0,'Children')==QuestFig),
  if get(QuestFig,'UserData'),
    ButtonName=Default;
  else,
    ButtonName=get(get(QuestFig,'CurrentObject'),'String');
  end
  delete(QuestFig);
else
  ButtonName=Default;
end

set(0,'ShowHiddenHandles',TempHide);



% --------------------------------------------------------------------
% WAITBAR2
function fout = waitbar2(x1,x2,whichbar)
% WAITBAR2 create a 2 line waitbar for use in two loops
% Use:
%   h = waitbar2(0,0,'Title');  % Initialize
%   for i=1:N
%     for j=1:M
%       waitbar2(j/M,i/N,h)     % Update
%     end
%   end
if nargin == 3
    if ischar(whichbar)
        if isempty(x2)
            type=1;     %we are initializing 1 bar
        else
            type=2;     %we are initializing 2 bars
        end
        name = whichbar;
    elseif isnumeric(whichbar)
        type=0; %we are updating, given a handle
        if ishandle(whichbar)
            f=whichbar;
        else
            error('Invald waitbar handle.')
        end
    else
        error(['Input arguments of type ' class(whichbar) ' not valid.'])
    end
else
    error('Input arguments not valid.');
end

x1 = max(0,min(100.*x1,100));
x2 = max(0,min(100.*x2,100));

switch type
case 0      % Update waitbar
	p1 = findobj(f,'Tag','p1');
    p2 = findobj(f,'Tag','p2');
    t = findobj(f,'Tag','title');
    
	if isempty(f)
      error('Couldn''t find waitbar handles.'); 
	end
    
    if ~isempty(x1) & ~isempty(p1)
		xpatch1 = get(p1,'XData');
		xpatch1 = [0 x1 x1 0];
		set(p1,'XData',xpatch1)  
    end
    
    if ~isempty(x2) & ~isempty(p2)
        xpatch2 = get(p2,'XData');
        xpatch2 = [0 x2 x2 0];
        set(p2,'XData',xpatch2)
    end
    
case 1      % Initialize waitbar with 1 bar
	f = figure('CreateFcn','movegui(gcbo,''center'')','MenuBar','none','Interruptible','off',...
        'Resize','off','NumberTitle','off','IntegerHandle','off','Units','pixels',...
        'Position',[1 1 360 65],'Name',whichbar,'WindowStyle','modal');
    
	ax1 = axes('XLim',[0 100],'YLim',[0 1],'Box','on','Units','Pixels',...
         'Position',[20 30 320 20],'XTickMode','manual','YTickMode','manual',...
         'XTick',[],'YTick',[],'Tag','ax1');
	cancelButt = uicontrol('Parent',f, ...
			'Units','Pixels', ...
			'Callback','close(gcbf)', ...
			'ButtonDownFcn', 'close(gcbf)', ...
			'Enable','on', ...
			'Interruptible','off', ...
			'Position', [150 5 60 20], ...
			'String','Cancel', ...
			'Tag','TMWWaitbarCancelButton');
    
    % Create patch object
    p1 = patch([0 x1 x1 0],[0 0 1 1],[1 .6 .6],'EdgeColor','r','Parent',ax1,'Tag','p1');

case 2      % Initialize waitbar with 2 bars
	f = figure('CreateFcn','movegui(gcbo,''center'')','MenuBar','none','Interruptible','off',...
        'Resize','off','NumberTitle','off','IntegerHandle','off','Units','pixels',...
        'Position',[1 1 360 85],'Name',whichbar,'WindowStyle','normal');
	
	ax1 = axes('XLim',[0 100],'YLim',[0 1],'Box','on','Units','Pixels',...
         'Position',[20 55 320 20],'XTickMode','manual','YTickMode','manual',...
         'XTick',[],'YTick',[],'Tag','ax1');
	ax2 = axes('XLim',[0 100],'YLim',[0 1],'Box','on','Units','Pixels',...
         'Position',[20 30 320 20],'XTickMode','manual','YTickMode','manual',...
         'XTick',[],'YTick',[],'Tag','ax2');
	cancelButt = uicontrol('Parent',f, ...
			'Units','Pixels', ...
			'Callback','close(gcbf)', ...
			'ButtonDownFcn', 'close(gcbf)', ...
			'Enable','on', ...
			'Interruptible','off', ...
			'Position', [150 5 60 20], ...
			'String','Cancel', ...
			'Tag','TMWWaitbarCancelButton');
    
    % Create patch objects
    p1 = patch([0 x1 x1 0],[0 0 1 1],[1 .6 .6],'EdgeColor','r','Parent',ax1,'Tag','p1');
    p2 = patch([0 x2 x2 0],[0 0 1 1],'r','EdgeColor','r','Parent',ax2,'Tag','p2');

    
%     label1 = text(50,0.5,'','Color',[0 0 0],'EraseMode','xor','Parent',ax1,...
%                 'HorizontalAlignment','center','Tag','label1')
%     label2 = text(50,0.5,'','Color',[0 0 0],'EraseMode','xor','Parent',ax2,...
%                 'HorizontalAlignment','center','Tag','label2')
end
             
if nargout==1,
    fout = f;
end 



% --------------------------------------------------------------------
%%% 
%%%  Sub-function - CreatePointer - Creates cursors
%%% 
function [pointerShape, pointerHotSpot] = CreatePointer(type)
% Usage:
% [pointerShape, pointerHotSpot] = CreatePointer;
% set(handles.figure1, 'Pointer', 'custom', ...
%    'PointerShapeCData', pointerShape, ...
%    'PointerShapeHotSpot', pointerHotSpot);
% 

switch type
case 'one'
pointerHotSpot = [1 1];
pointerShape = [ ...
      2   2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
      2   1     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
      2   1     1     2   NaN   NaN   NaN   NaN   NaN   NaN     2     2     2     2     2   NaN
      2   1     1     1     2   NaN   NaN   NaN   NaN   NaN     2     1     1     1     2   NaN
      2   1     1     1     1     2   NaN   NaN   NaN   NaN     2     1   NaN   NaN     1     2
      2   1     1     1     1     1     2   NaN   NaN   NaN     2     1   NaN   NaN     1     2
      2   1     1     1     1     1     1     2   NaN   NaN     2     1   NaN   NaN     1     2
      2   1     1     1     1     1     1     1     2   NaN     2     1   NaN   NaN     1     2
      2   1     1     1     2     2     2     2   NaN   NaN     2     2     2     2     2   NaN
      2   1     1     2   NaN   NaN   NaN   NaN   NaN   NaN     2     1     1     1     2   NaN
      2   1     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     1   NaN   NaN     1     2
    NaN   2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     1   NaN   NaN     1     2
    NaN NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     1   NaN   NaN     1     2
    NaN NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     1   NaN   NaN     1     2
    NaN NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     2     2     2     2   NaN
    NaN NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     1     1   NaN   NaN];
case 'end'
pointerHotSpot = [1 1];
pointerShape = [ ...
      2   2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
      2   1     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
      2   1     1     2   NaN   NaN   NaN   NaN   NaN   NaN     2     2     2     2     2     2
      2   1     1     1     2   NaN   NaN   NaN   NaN   NaN     2     1     1     1     1     1
      2   1     1     1     1     2   NaN   NaN   NaN   NaN     2     1   NaN   NaN   NaN   NaN
      2   1     1     1     1     1     2   NaN   NaN   NaN     2     1   NaN   NaN   NaN   NaN
      2   1     1     1     1     1     1     2   NaN   NaN     2     1   NaN   NaN   NaN   NaN
      2   1     1     1     1     1     1     1     2   NaN     2     1   NaN   NaN   NaN   NaN
      2   1     1     1     2     2     2     2   NaN   NaN     2     1     2     2     2     2
      2   1     1     2   NaN   NaN   NaN   NaN   NaN   NaN     2     1     1     1     1     1
      2   1     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     1   NaN   NaN   NaN   NaN
    NaN   2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     1   NaN   NaN   NaN   NaN
    NaN NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     1   NaN   NaN   NaN   NaN
    NaN NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     1   NaN   NaN   NaN   NaN
    NaN NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     2     2     2     2     2
    NaN NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     1     1     1     1];
case 'rotate'
pointerHotSpot = [8 8];
pointerShape = [ ...
   NaN   NaN   NaN   NaN     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN     1     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN     1     2     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN     1     2     2     1     1     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
     1     2     2     2     2     2     2     1     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN     1     2     2     1     1     1     2     2     1   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN     1     2     1   NaN   NaN     1     1     2     1   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN     1     1   NaN   NaN   NaN   NaN     1     2     1   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN     1   NaN   NaN   NaN   NaN     1     2     1   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     2     1   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     2     1   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     1     1     1     2     1     1     1     1
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     2     2     2     2     2     1   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     2     2     2     1   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     2     1   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1   NaN   NaN   NaN   NaN];


case 'X'
pointerHotSpot = [8 8];
pointerShape = [ ...
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1   NaN   NaN   NaN   NaN
   NaN   NaN     2   NaN     1   NaN   NaN   NaN   NaN   NaN     1   NaN   NaN     2   NaN   NaN
   NaN   NaN   NaN     2   NaN     1   NaN   NaN   NaN     1   NaN   NaN     2   NaN     1   NaN
   NaN     1   NaN   NaN     2   NaN     1   NaN     1   NaN   NaN     2   NaN     1   NaN   NaN
   NaN   NaN     1   NaN   NaN     2   NaN     1   NaN   NaN     2   NaN     1   NaN   NaN   NaN
   NaN   NaN   NaN     1   NaN   NaN     2   NaN   NaN     2   NaN     1   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN     1   NaN   NaN   NaN   NaN   NaN     1   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN     1   NaN   NaN   NaN   NaN   NaN     1   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN     1   NaN   NaN     2   NaN   NaN     2   NaN     1   NaN   NaN   NaN   NaN
   NaN   NaN     1   NaN   NaN     2   NaN     1   NaN   NaN     2   NaN     1   NaN   NaN   NaN
   NaN     1   NaN   NaN     2   NaN     1   NaN     1   NaN   NaN     2   NaN     1   NaN   NaN
   NaN   NaN   NaN     2   NaN     1   NaN   NaN   NaN     1   NaN   NaN     2   NaN   NaN   NaN
   NaN   NaN     2   NaN     1   NaN   NaN   NaN   NaN   NaN     1   NaN   NaN     2   NaN   NaN
   NaN   NaN   NaN     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN];
case 'mover'
pointerHotSpot = [8 8];
pointerShape = [ ...
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN     2   NaN   NaN   NaN   NaN   NaN   NaN     2   NaN   NaN   NaN   NaN
   NaN   NaN   NaN     2     2   NaN   NaN   NaN   NaN   NaN   NaN     2     2   NaN   NaN   NaN
   NaN   NaN     2     1     2     2     2     2     2     2     2     2     1     2   NaN   NaN
   NaN     2     1     1     1     1     1     1     1     1     1     1     1     1     2   NaN
     2     1     1     1     1     1     1     1     1     1     1     1     1     1     1     2
   NaN     2     1     1     1     1     1     1     1     1     1     1     1     1     2   NaN
   NaN   NaN     2     1     2     2     2     2     2     2     2     2     1     2   NaN   NaN
   NaN   NaN   NaN     2     2   NaN   NaN   NaN   NaN   NaN   NaN     2     2   NaN   NaN   NaN
   NaN   NaN   NaN   NaN     2   NaN   NaN   NaN   NaN   NaN   NaN     2   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN];
case 'eraser'
pointerHotSpot = [16 1];
pointerShape = [ ...
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     1     1     1     1     1     1     1   NaN
   NaN   NaN   NaN   NaN   NaN   NaN     1     2     2     2     2     2     2     1     1   NaN
   NaN   NaN   NaN   NaN   NaN     1     2     2     2     2     2     2     1     2     1   NaN
   NaN   NaN   NaN   NaN     1     2     2     2     2     2     2     1     2     2     1   NaN
   NaN   NaN   NaN     1     2     2     2     2     2     2     1     2     2     1   NaN   NaN
   NaN   NaN     1     2     2     2     2     2     2     1     2     2     1   NaN   NaN   NaN
   NaN     1     2     2     2     2     2     2     1     2     2     1   NaN   NaN   NaN   NaN
     1     1     1     1     1     1     1     1     2     2     1   NaN   NaN   NaN   NaN   NaN
     1     2     2     2     2     2     2     1     2     1   NaN   NaN   NaN   NaN   NaN   NaN
     1     2     2     2     2     2     2     1     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN
     1     1     1     1     1     1     1     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN];
case 'my cross'
pointerHotSpot = [8 8];
pointerShape = [ ...
   NaN NaN NaN NaN NaN   1   2 NaN   2   1 NaN NaN NaN NaN NaN NaN
   NaN NaN NaN NaN NaN   1   2 NaN   2   1 NaN NaN NaN NaN NaN NaN
   NaN NaN NaN NaN NaN   1   2 NaN   2   1 NaN NaN NaN NaN NaN NaN
   NaN NaN NaN NaN NaN   1   2 NaN   2   1 NaN NaN NaN NaN NaN NaN
   NaN NaN NaN NaN NaN   1   2 NaN   2   1 NaN NaN NaN NaN NaN NaN
     1   1   1   1   1   1   2 NaN   2   1   1   1   1   1   1   1
     2   2   2   2   2   2   2 NaN   2   2   2   2   2   2   2   2
   NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN
     2   2   2   2   2   2   2 NaN   2   2   2   2   2   2   2   2
     1   1   1   1   1   1   2 NaN   2   1   1   1   1   1   1   1
   NaN NaN NaN NaN NaN   1   2 NaN   2   1 NaN NaN NaN NaN NaN NaN
   NaN NaN NaN NaN NaN   1   2 NaN   2   1 NaN NaN NaN NaN NaN NaN
   NaN NaN NaN NaN NaN   1   2 NaN   2   1 NaN NaN NaN NaN NaN NaN
   NaN NaN NaN NaN NaN   1   2 NaN   2   1 NaN NaN NaN NaN NaN NaN
   NaN NaN NaN NaN NaN   1   2 NaN   2   1 NaN NaN NaN NaN NaN NaN
   NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN];
case 'arrow cross'
pointerHotSpot = [1 1];
pointerShape = [ ...
      2   2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
      2   1     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
      2   1     1     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
      2   1     1     1     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
      2   1     1     1     1     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
      2   1     1     1     1     1     2   NaN   NaN   NaN   NaN     2   NaN   NaN   NaN   NaN
      2   1     1     1     1     1     1     2   NaN   NaN     2     1     2   NaN   NaN   NaN
      2   1     1     1     1     1     1     1     2   NaN     2     1     2   NaN   NaN   NaN
      2   1     1     1     2     2     2     2   NaN   NaN     2     1     2   NaN   NaN   NaN
      2   1     1     2   NaN   NaN   NaN     2     2     2     2     1     2     2     2     2
      2   1     2   NaN   NaN   NaN     2     1     1     1     1     1     1     1     1     1
    NaN   2   NaN   NaN   NaN   NaN   NaN     2     2     2     2     1     2     2     2     2
    NaN NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     1     2   NaN   NaN   NaN
    NaN NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     1     2   NaN   NaN   NaN
    NaN NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2     1     2   NaN   NaN   NaN
    NaN NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     2   NaN   NaN   NaN   NaN];
case 'magnify'
pointerHotSpot = [4 4];
pointerShape = [ ...
    NaN   NaN   NaN     1     1     1     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN
    NaN     1     1     2     2     2     2     1     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN
    NaN     1     2   NaN   NaN   NaN   NaN     2     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN
      1     2   NaN   NaN   NaN   NaN   NaN   NaN     2     1   NaN   NaN   NaN   NaN   NaN   NaN
      1     2   NaN   NaN   NaN   NaN   NaN   NaN     2     1   NaN   NaN   NaN   NaN   NaN   NaN
      1     2   NaN   NaN   NaN   NaN     2     1     2     1   NaN   NaN   NaN   NaN   NaN   NaN
      1     2   NaN   NaN   NaN     2     2     1     2     1   NaN   NaN   NaN   NaN   NaN   NaN
    NaN     1     2   NaN   NaN     1     1     2     1     1   NaN   NaN   NaN   NaN   NaN   NaN
    NaN     1     1     2     2     2     2     1     2     2     1   NaN   NaN   NaN   NaN   NaN
    NaN   NaN   NaN     1     1     1     1     1     2     2     2     1   NaN   NaN   NaN   NaN
    NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     2     2     2     1   NaN   NaN   NaN
    NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     2     2     2     1   NaN   NaN
    NaN     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     2     2     2     1   NaN
      2     2     2   NaN     2     2     2   NaN   NaN   NaN   NaN     1     2     2     2     1
    NaN     2   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     2     1   NaN
    NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1   NaN   NaN];
case 'hand'
pointerHotSpot = [8 8];
pointerShape = [ ...
   NaN   NaN   NaN   NaN   NaN   NaN   NaN     1     1   NaN   NaN   NaN   NaN   NaN   NaN   NaN
   NaN   NaN   NaN     1     1   NaN     1     2     2     1     1     1   NaN   NaN   NaN   NaN
   NaN   NaN     1     2     2     1     1     2     2     1     2     2     1   NaN   NaN   NaN
   NaN   NaN     1     2     2     1     1     2     2     1     2     2     1   NaN     1   NaN
   NaN   NaN   NaN     1     2     2     1     2     2     1     2     2     1     1     2     1
   NaN   NaN   NaN     1     2     2     1     2     2     1     2     2     1     2     2     1
   NaN     1     1   NaN     1     2     2     2     2     2     2     2     1     2     2     1
     1     2     2     1     1     2     2     2     2     2     2     2     2     2     2     1
     1     2     2     2     1     2     2     2     2     2     2     2     2     2     1   NaN
   NaN     1     2     2     2     2     2     2     2     2     2     2     2     2     1   NaN
   NaN   NaN     1     2     2     2     2     2     2     2     2     2     2     2     1   NaN
   NaN   NaN     1     2     2     2     2     2     2     2     2     2     2     1   NaN   NaN
   NaN   NaN   NaN     1     2     2     2     2     2     2     2     2     2     1   NaN   NaN
   NaN   NaN   NaN   NaN     1     2     2     2     2     2     2     2     1   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN     1     2     2     2     2     2     2     1   NaN   NaN   NaN
   NaN   NaN   NaN   NaN   NaN     1     2     2     2     2     2     2     1   NaN   NaN   NaN];
otherwise
    pointerHotSpot = [1 1];
    pointerShape = zeros(16);
    pointerShape(:) = NaN;
    pointerShape(1,1) = 2;
end



% --------------------------------------------------------------------
% SAFE_TRACE_LOCAL - Version of safe trace that returns the x
%   coordinate of where the overlap ends.  Returns NaN if
%   it could not find the end of overlap.
function end_overlap = safe_trace_local(x0,y0,Img, RATE, g_width, MERGE, DistLimit)


	xdata = [];
	ydata = [];
    x = x0;
    y = y0;
    end_overlap = NaN;
    
    while 1
        if No_next_pt(x,y) 
            endCondition = 'gap';       break
        elseif Multiple_paths(x,y)
            endCondition = 'endOverlap';   
            end_overlap = min(x);
            break
        elseif abs(x-x0) > DistLimit
            endCondition = 'distance';  break
        end
        [y_index, top, botm] = get_connected_pixels(x,y,Img, g_width, MERGE);  
        
        y = center_y(x, top, botm, Img); 
            
        if x ~= x0  % Do NOT include point passed in.
            xdata = [xdata; x];
            ydata = [ydata; y];
        end
        
        try
            [x, y, dif] = get_next_pt(x+RATE,y,y_index, Img);
        catch, 
            endCondition = 'gap';   break
        end   % Generally to catch index out of bounds.
    end%END WHILE

% --------------------------------------------------------------------
% CENTER_Y - center the y value in the middle of the line
function y = center_y(x, top, botm, Img)
    y = top+(botm-top)/2;

    
% --------------------------------------------------------------------
% NO_NEXT_PT - tells us if get_next_pt could not find the next point
function bool = No_next_pt(x, y)
    if isempty(x) | isempty(y) | isnan(x) | isnan(y)
        bool = 1; % Must use if statement for ORs to short circuit to avoid [].
    else bool = 0; end
    
% --------------------------------------------------------------------
% MULTIPLE_PATHS - Tells us if we should record a forking point.
function bool = Multiple_paths(x,y)
    bool = length(y) > 1;
    % Check data valid
    if length(y) ~= length(x)
        error('mismatch x y')
    end
    

%----------------------------------------------------------
%COUNT_GAPS
%   [px_per_minute, gap_count, px_total] = count_gaps( image_h, Points_h, g_width, GAP_TAG_STRING)
%   Count gaps return the locations of the gaps on a line
function [px_per_minute, gap_count, px_total]...
          = count_gaps( image_h, Points_h, g_width, GAP_TAG_STRING, dpi)

BLACK = 0;  WHITE = 1;

px_total = 0;
px_per_minute = 0;
gap_count = 0;

if ~ishandle(image_h) | any(~ishandle(Points_h))
    error('Programmer Error: Non-handle given to trace function.')
    return
% elseif ~isempty(traceLine_h) & ~ishandle(traceLine_h)
%     errordlg('Programmer Error: Non-handle given to trace function.','Trace error','modal')
%     return
end
if length(Points_h) < 2
%     helpdlg('User Error: Need at least two control points.','Trace error','modal')
    error('User Error: Need at least two control points.')
    return
end

bigAxes_h = get(image_h,'Parent');

xPts = get(Points_h(:),{'XData'});  xPts = [xPts{:}];
yPts = get(Points_h(:),{'YData'});  yPts = [yPts{:}];

% Create a cropped image from the whole image for faster
% matrix access times. Also initialize the crop offset in the
% y direction so that we can change the values to match the
% the value it would have relative to the whole image.
MAXAMP = 100;                    % Crop distance from highest and lowest control point.
Img = get(image_h,'CData');
% ImgInfo = get(image_h,'UserData');
% g_width = ImgInfo.g_width;
topRow = max([1,round(min(yPts))-MAXAMP]);             % Make sure in bounds
botmRow = min([size(Img,1), round(max(yPts))+MAXAMP]); % Make sure in bounds
Img = Img(topRow:botmRow,:);    % top row to bottom row, all columns.
NUMROWS = size(Img,1);          % Constant for max y value
NUMCOLS = size(Img,2);          % Constant for max x value
CROP_Y_OFFSET = topRow-1;       % Note: since we include all columns, there is
                                % no need to offset in the x direction.

% Create enough space for every pixel from start to end.
% Fill with NaNs so we don't have to fill in gaps later. 
% Also initialize the index offset to change from x value to the index it has
% in the trace data array.   (Example use: TraceData(x+IDXOFFSET) = ...)
LENGTH = max(xPts) - min(xPts) + 1;             % Number of pixels from start to end
TraceData = zeros(1,LENGTH);                    % Row vector
TraceData(:) = NaN;                             % Not-a-Number
IDXOFFSET = -min(xPts)+1;                       % Offset to convert x value to ...
                                                % corresponding index in TraceData.
                                                
RATE = 4;                   % Sampling rate in pixels per sample.
HEIGHT = ceil(2*g_width+1); % Max width that window guaranteed large enough to see.
WIDTH = 1;                  % Pixels to left and right to look.

thickness = g_width;

x1=[]; x2=[]; x=[]; y=[]; old_x=[]; y1=[];
rows=[]; top=[]; botm=[]; slope=[]; project =[];
winX=[]; winY=[]; Window=[];
colSum=[]; avg=[]; rowSum=[];
offset=[];

consecutive = 0;
begin = 0;
GapLocation = [];
MIN_SPACE = 410*(dpi/400);          % Min amount of space between identified minute gaps.
TIME_LIMIT = 1000;                   % Upper bound on time we want to let this run (seconds).
startGap = 0;
secondToEndGap = 1;
secondToLastGapCount = 1;
endGap = 1;
noise_gap_count = 0;
MAX_NOISE = 2;
MINUTE_MARK = 5;    % Minimum width of a minute mark.
NOISE_LIMIT = 10;  % Gap of 10 pixels wide is too noisy.
lineNum = get(Points_h(1),'UserData');
startTime = cputime;
delete(findobj(bigAxes_h,'Tag',GAP_TAG_STRING,'UserData',lineNum))
for N = 2:length(Points_h)
  
    x1 = get(Points_h(N-1),'XData');
    x2 = get(Points_h(N),'XData');
    
    y1 = get(Points_h(N-1),'YData')-CROP_Y_OFFSET;
    y = y1(end);                % Starting y-value
    x = x1(end);                % Starting x-value
    old_x = NaN;
    TraceData(x+IDXOFFSET) = y;
    
	[winX winY Window] = getWindow(x,y,Img,HEIGHT,0);
    rows = find(~sum(Window,2));                % Find all black rows
	top = max([1; rows(rows<=winY)]);           % Row above top of line
	botm = min([size(Window,1);...
                 rows(rows>=winY)]);             % Row below bottom of line
    if botm-top+1 >= thickness
        thickness = botm-top+1;
    end
    
    while x < x2(1);
        slope = visionSlope(x,y,Img, thickness);;
        project = y + slope*RATE;
        project = min(NUMROWS,max(1,project));      % Keep inside image boundries
        old_x = x;
        x = x+RATE;
        
        if x >= x2(1)
            break
        end
  
        % Get next window at which we will look
		[winX winY Window] = getWindow(x,project,Img,HEIGHT,WIDTH);

        % Check for a gap
        top = max([1; rows(rows<=winY)]);           % Row above top of line
	    botm = min([size(Window,1);...
                 rows(rows>=winY)]);             % Row below bottom of line
        colSum = any(Window(top:botm,:));
        if any(colSum == 0)                     % Any column all black, a gap

            avg = getAvg(x+IDXOFFSET,TraceData, 500);
            H = 50;
            [x project] = Jump_gap(x,y,Img,avg, H);     % Jump the gap
            
            if consecutive >= MIN_SPACE
                begin = 1;
            else
                consecutive = 0;    % Start over looking for a good block
                startGap = x;   % Reset what might be the starting location
            end
            
            % LABEL GAPS HERE
            if begin == 1
                if x-old_x > NOISE_LIMIT
                    noise_gap_count = noise_gap_count + 1;
                end
                if noise_gap_count >= MAX_NOISE
                    noise_gap_count = noise_gap_count -1;
                    break
                else
                    if isempty(GapLocation)
                        GapLocation = startGap;
                        gap_count = 0;
                        noise_gap_count = 0;
                        h = text(startGap,project+CROP_Y_OFFSET, '0'...
                            ,'Parent',bigAxes_h,'Tag',GAP_TAG_STRING,'UserData',lineNum ...
                            ,'FontSize',16,'Color','c','HorizontalAlignment','center');
                    end
                    if length(GapLocation) == 1
                        [GapLocation, gap_count] = updateMinuteMark(GapLocation, x, gap_count, MIN_SPACE);
                        endGap = x;
                    elseif isMinuteMark(GapLocation, x, old_x, MIN_SPACE, MINUTE_MARK)
                        secondToEndGap = endGap;
                        secondToLastGapCount = gap_count;
                        h = text(secondToEndGap,project+CROP_Y_OFFSET,num2str(gap_count)...
                            ,'Parent',bigAxes_h,'Tag',GAP_TAG_STRING,'UserData',lineNum ...
                            ,'FontSize',16,'Color','c','HorizontalAlignment','center');
                        [GapLocation, gap_count] = updateMinuteMark(GapLocation, x, gap_count, MIN_SPACE);
                        endGap = x;
                        noise_gap_count = 0;
                    end
                end
            end
            
            % Get a new window from the new point from which we will center the point.
            [winX winY Window] = getWindow(x,project,Img,HEIGHT,WIDTH);  
        else
            consecutive = consecutive + x-old_x;
        end
        
        % Find the nearest line
        rowSum = sum(Window,2);
        offset = find(rowSum > 0) - winY;
        if isempty(offset)
            winY = [];
        else
            winY = winY + offset(min(find(abs(offset)==min(abs(offset)))));
        end
        
        if isempty(winY)                            % Cannot find the line in the window
            disp('Lost the line. Sorry. Going back to mean.')
            
            % Find the line again by searching along the mean of the trace.
            project = round(getAvg(x+IDXOFFSET,TraceData, 500));
            while x < size(Img,2) & Img(project,x) == BLACK,
                x = x+1;
            end
            if Img(project,x) == BLACK,
                disp('Cannot find white along mean. Skipping to next pt.')
                continue
            end
        end
        
        % Data to center the point on the line and get the thickness
		rows = find(~sum(Window,2));                % Find all black rows
		top = max([1; rows(rows<=winY)]);           % Row above top of line
		botm = min([size(Window,1);...
                    rows(rows>=winY)]);             % Row below bottom of line
        

		[r c] = find(Window(top:botm,:));
        y = project-winY+mean(r)+top-1;     % y centered

        
        if x < x2(1)
            % Save point into array
            TraceData(x+IDXOFFSET) = y;
        end
        if cputime-startTime > TIME_LIMIT, break, end
    end
	if cputime-startTime >TIME_LIMIT, break, end
end

gap_count = secondToLastGapCount;
px_total = max(0,secondToEndGap-startGap);
px_per_minute = px_total/gap_count;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% --------------------------------------------------------------------
% ISMINUTEMARK
function bool = isMinuteMark(GapLocation, x, old_x, MIN_SPACE, MINUTE_MARK)
    if isempty(GapLocation), bool = 1; return, end
        
    if x-GapLocation(end) >= MIN_SPACE & x-old_x > MINUTE_MARK
        bool = 1;
    else
        bool = 0;
    end
% --------------------------------------------------------------------
% UPDATEMINUTEMARK
function [GapLocation, gap_count] = updateMinuteMark(GapLocation, x, gap_count, MIN_SPACE)
    if isempty(GapLocation)
        GapLocation = x;
        gap_count = 1;
    elseif length(GapLocation) == 1
        gap_count = gap_count+1;
        GapLocation(end+1) = x;
    else
        Space = x-GapLocation(end);
        Estimate = (GapLocation(end)-GapLocation(1))/gap_count;
        gap_count = gap_count + round(max(1,Space/Estimate));
        GapLocation(end+1) = x;
    end
    
% --------------------------------------------------------------------
% JUMP_GAP
% HEIGHT - gives height of window to look
function [next_x, next_y] = Jump_gap(x,y,Img,avg, HEIGHT)
	MAXGAP = 15;                % Pixels forward to look.
    R = round(avg);
    
    topRow = max(1,R-HEIGHT);                   % Make sure index in bounds
    botmRow = min(size(Img,1),R+HEIGHT);        % Make sure index in bounds
    leftCol = x+1;
    rightCol = min(size(Img,2),x+MAXGAP);       % Make sure index in bounds
    Window = Img(topRow:botmRow,leftCol:rightCol);

    if length(y) ~= 1
        disp('debug, trace_it, 260'), keyboard
    end
    
    winY = y-topRow+1;                          % y pixel in Window's referece
    winAvg = avg-topRow+1;
    
    while ~any(Window) & rightCol < size(Img,2)
        rightCol = rightCol+1;                  % Keep expanding window until we find something
        Window = Img(topRow:botmRow,leftCol:rightCol);
    end
    
    if ~any(Window)
        next_x = x+1;   next_y = avg;
        return
    end

%% 
%% Optionally avoid the minute splotch by tossing out objects not touching
%%  the left edge.
%     siz = size(Window);
%     r = [1:siz(1)   ones(1,ceil(siz(2)/2)+1) siz(1).*ones(1, ceil(siz(2)/2)+1)];
%     c = [siz(2).*ones(1,siz(1)) floor(siz(2)/2):siz(2)  floor(siz(2)/2):siz(2)];
%     Objects = bwselect(Window,c,r,8);
    Objects = Window;
    
    % ::DEBUG::
    %figure(x), imshow(Objects), hold on, plot(c,r,'r.'), hold off
        
    if ~any(Objects)
        Objects = Window;
    end
    
    
    % Create an array, D, with all the distances from the point of interest to 
    % each white pixel.
    [R C] = find(Objects);                          % Find every white pixel
    pt = [winY 0];                                  % Point of interest
    D_from_pt = sum(abs([R C] - pt(ones(size(R)),:)).^2, 2); % inline call to repmat
    D_from_avg = abs(R - winAvg(ones(size(R))));
    D = D_from_pt + D_from_avg.^2;
    
    % Find the nearest pixel 
    i = min(find(D == min(D)));     % Pick left-top most pixel in case of a tie.
    
    if length(y) ~= 1 | length(i) ~= 1 | length(winY) ~= 1
        disp('debug, trace_it, 310')
    end
    
    % ::DEBUG::
    if size(x) ~= size(C(i)) | size(y) ~= size(R(i)) | size(winY) ~= 1
        disp('debug, trace_it, 315')
    end
    
    next_x = x+C(i);
    next_y = y+R(i)-winY;
   
    
% --------------------------------------------------------------------
% GETWINDOW
function [winX, winY, Window] = getWindow(x,y,Img, HEIGHT, WIDTH)   
	% Get the next image window to look at.
	tmpY = round(y);
	topRow = max([1,tmpY-HEIGHT]);            % Make sure index in bounds
	botmRow = min([size(Img,1),tmpY+HEIGHT]);     % Make sure index in bounds
	leftCol = max([1,x-WIDTH]);               % Make sure index in bounds
	rightCol = min([size(Img,2),x+WIDTH]);        % Make sure index in bounds
	Window = Img(topRow:botmRow,leftCol:rightCol);
	winY = tmpY-topRow+1;                   % y pixel in Window's referece
	winX = x-leftCol+1;                % x pixel in Window's reference
    
% --------------------------------------------------------------------
% VISIONSLOPE   
function slope = visionSlope(x,y,Img, H)
    W = 5;
    [x, y, Window] = getWindow(x,round(y),Img,round(H),W);
    
    [R C] = find(Window);
    Rise = R-y;
    Run = C - x;
    ERROR_MARGIN = 1;
    I = find(abs(Run) > ERROR_MARGIN);   % Limit voting pixels to at least N pixel(s) away
    if isempty(I)
%         disp('Not enough pixels for visionSlope. Returning slope = 0.')
        slope = 0;
    else
        slope = mean(Rise(I)./Run(I));
    end

% --------------------------------------------------------------------
% GETTHICKNESS  
function thickness = getThickness(Window, winX, top, botm);
    slice = Window(top:botm,winX);
	thickness = sum(slice);

    
% --------------------------------------------------------------------
% GETAVG  
function avg = getAvg(idx, TraceData, SAMP_SIZ)
	temp = TraceData(1:idx);            % Pts before current pt
	temp = temp(max(1,length(temp)-SAMP_SIZ-1)...
            :length(temp));                     % Get only last 500
	temp = temp(isfinite(temp));                  % Remove Inf and NaNs
    if isempty(temp)
        disp('Avg error, no values to avg')
        avg = nanmean(temp);
    else
    	avg = mean(temp);
    end


% --------------------------------------------------------------------
% --------------------------------------------------------------------
% STACK DATA STRUCTURE
% --------------------------------------------------------------------
% --------------------------------------------------------------------
% STACK
function ST = stack(varargin)
% STACK - create an empty stack

% Default values
ST.List = {};
ST.name = 'stack';
ST.maxByteSize = Inf;

switch nargin
% 1 argument: maximum byte size of stack 
case 1
    % Copy constructor
    if isa(varargin{1}, 'stack')
        ST = varargin{1};
    elseif ischar(varargin{1})
        ST.name = varargin{1};
    elseif isnumeric(varargin{1}) & length(varargin{2}) == 1
        ST.maxByteSize = varargin{1};
    else
        error('Input argument is not a stack object, character string, or singleton number.')
    end
% 2 arguments: name and max byte size of stack
case 2
    % 1. Character string input: stack name
    if ischar(varargin{1})
        ST.name = varargin{1};
    else 
        error('Stack name must be a character string.')
    end
    % 2. Numeric input: maxByteSize
    if isnumeric(varargin{2})
        if length(varargin{2}) ~= 1, 
            error('Max stack size must be singleton number')
        else 
            ST.maxByteSize = varargin{2};
        end
    end
end
% ---- TOP -----
function object = top(ST)
    if ~isempty(ST.List)
        % Get object from top of stack, index 1 consider top of stack
        object = ST.List{1};
    else
        error('Stack is empty')
    end
% POP
function [ST, varargout] = pop(ST)
    if ~isempty(ST.List)
        % Remove object from top of stack, index 1 consider top of stack
        object = ST.List{1};
        ST.List = ST.List(2:end);
    else
        error('Stack is empty')
    end
    
    if nargout > 1
        varargout{1} = object;
    end
% PUSH
function ST = push(ST, object)
    if ~isempty(ST.List)
        ST.List = {object ST.List{:}};
    else
        ST.List = {object};
    end
    
    L = length(ST.List);
    array_stats = whos('ST');
    showWarning = 'true';   % Show warning first loop only
    count = 0;
    
    % Bring down to 50% full OR 3 items only to conserve space.
    while length(ST.List) > 3 ...
    & array_stats.bytes > ST.maxByteSize * 0.50  
%         if isequal(showWarning,'true')
%             disp([' Stack "' ST.name '" is larger than ' num2str(ST.maxByteSize) ', clearing list...'])
%             showWarning = 'false';
%         end
        ST.List = ST.List(1:end-1);     % Throw away oldest item
        count = count +1;
        array_stats = whos('ST');
    end
%     if count > 0
%         disp(['    ' num2str(count) ' of ' num2str(L) ' events removed.'])
%     end
% STACKEMPTY
function boolean = stackempty(ST)
    boolean = isempty(ST.List);
    

    
% -----------------------------------------
% HIDE36_TRACE_LINES
% Search for "3on6off" to find if statement that NaNs hidden line.
function hide36_trace_lines(handles)
    disp('Showing 3, hiding 6')
    Traces = findobj(handles.big_axes,'Tag','trace_line');
    LineNums = get(Traces,{'UserData'});
    minLN = min([LineNums{:}]);
    maxLN = max([LineNums{:}]);
    for I=1:length(Traces)
        lineNum =LineNums{I};
        if lineNum == maxLN | lineNum == minLN
            set(Traces(I), 'Visible', 'on') % Always keep first and last line on.
        else
            switch mod(lineNum,9)
            case {0, 1, 2}
                set(Traces(I), 'Visible', 'on')
            otherwise
                set(Traces(I), 'Visible', 'off')
            end
        end
    end
